#ifndef MOBILE_SDK_H
#define MOBILE_SDK_H

/// @mainpage Ubisoft Mobile SDK
///
/// @section intro Introduction
///
/// You can download the dev packages of MSDK :
/// <ul>
///		<li><a href = "http://r-mob.ubi.com/?a=latest-msdk-ios">ios</a></li>
///		<li><a href = "http://r-mob.ubi.com/?a=latest-msdk-android">android</a></li>
///		<li><a href = "http://r-mob.ubi.com/?a=latest-msdk-wp8">windows phone</a></li>
///		<li><a href = "http://r-mob.ubi.com/?a=latest-msdk-w81">windows 8.1</a></li>
///		<li><a href = "http://r-mob.ubi.com/?a=latest-msdk-unity">unity</a></li>
/// </ul>
/// The SDK provides the developer functions to handle most of the mobile specific features
/// all function defined are available on all ubisoft mobile platform (iOS, android, Windows 8.0/8.1, Windows Phone 8).
///
///	for example GameCenter features friend list + leaderboard + scoreboard + achievement
/// all those feature are exposed seperately throught the mobile SDK so only the feature needed can be used
///
/// Here is the roadmap of the MobileSDK :
///
/// <table align="center" style="border-collapse:collapse; border:3px solid black;">
///
///	<tr>
///		<th rowspan="3" colspan="2" style="border-left:3px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Functionalities</th>
///		<th colspan="6" style="border-left:3px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Platform and store</th>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:3px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">iOS</td>
///		<td style="border-left:3px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">WP8.1</td>
///		<td style="border-left:3px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">W8.1</td>
///		<td colspan="3" style="border-left:3px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:1px solid black">Android</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:3px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">itunes</td>
///		<td style="border-left:3px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Windows Store</td>
///		<td style="border-left:3px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Windows Store</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Google Play</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Amazon</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Samsung</td>
///	</tr>
///
///	<tr align="center">
///		<td rowspan="3" style="border-left:3px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Advertisement</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Flurry ads</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#BFBFBF>N/A</td>
///		<td bgcolor=#BFBFBF>N/A</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Tapjoy</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#BFBFBF>N/A</td>
///		<td bgcolor=#BFBFBF >N/A</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Trialpay</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">N/A</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///	</tr>
///
///	<tr align="center">
///		<td rowspan="2" style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Push Notifications</td>
///		<td style="border-left:0px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">Server PN</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">Not Planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:0px solid black">Not Planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:1px solid black; border-right:0px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:1px solid black; border-right:0px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:1px solid black; border-right:0px solid black; border-bottom:0px solid black">Not planned</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Local PN</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Not Planned</td>
///	</tr>
///
///	<tr align="center">
///		<td rowspan="6" style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Social network</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Facebook</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Game Center</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF>N/A</td>
///		<td bgcolor=#BFBFBF>N/A</td>
///		<td bgcolor=#BFBFBF>N/A</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Game Services</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Not planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Not planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Not planned</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Game Circle</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Not planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Not planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Not planned</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///		<td bgcolor=#C4D79B>Done</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Xbox Live</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:0px solid black">N/A</td>
///		<td bgcolor=#BFBFBF style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:0px solid black">N/A</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Sina weibo</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///	<tr align="center">
///		<td rowspan="3" style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">IAB</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Purchase</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Receipt validation</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:0px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:0px solid black">Not planned</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Purchase/Receipt Split</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Not planned</td>
///	</tr>
///
///	<tr align="center">
///		<td rowspan="2" style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Tracking</td>
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black;">Flurry</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///	<tr align="center">
///		<td style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">DNA</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///	<tr align="center">
///		<td colspan="2" style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Dynamic store</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///	<tr align="center">
///		<td colspan="2" style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:0px solid black">Ubi services</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:1px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///	<tr align="center">
///		<td colspan="2" style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">User profile</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#E6B8B7 style="border-left:0px solid black; border-top:0px solid black; border-right:3px solid black; border-bottom:1px solid black">Not Planned</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///		<td bgcolor=#C4D79B style="border-left:0px solid black; border-top:0px solid black; border-right:0px solid black; border-bottom:1px solid black">Done</td>
///	</tr>
///
///
/// </table>
///
/// The SDK is design to be used in an asynchronous way.
/// Every action that need an answer is composed with 4 steps :
/// <ul>
/// 	<li><strong>MyFile_Call</strong>MyAction this will perform the request</li>
/// 	<li><strong>MyFile_Status</strong>MyAction this will give you the current status of the request</li>
/// 	<li><strong>MyFile_Result</strong>MyAction this will return the result of the request</li>
/// 	<li><strong>MyFile_Release</strong>MyAction this will release the resources allocated to perform the request</li>
/// </ul>
/// If you try to release a request that is not ended an error will occured(assert).
/// Same: Trying to get the status of an unexisting request (or Released request) will result in a failing assert.
///
/// Each call to a function is preceded by NameOfTheSection_ such as Achievement_CallUpdateAchievement
/// When the result of a request is a list a special structure is returned:
/// @code {.c}
/// struct PointerArray_MyType
/// {
///	msdk_u32 	count;
///	MyType**	elem;
/// }
/// @endcode
/// This result will be released once you'll call the MyFile_Release function. If you want to keep it you need to create a copy.
///
/// @section getting_started Getting Started
///
/// <table>
/// <tr>
/// 	<td>@ref getting_started_ios</td>
/// 	<td>@ref getting_started_android</td>
/// 	<td>@ref getting_started_wp8</td>
/// 	<td>@ref getting_started_win8</td>
/// </tr>
/// </table>
///
/// For unity integration please click here @ref unity_integration_main
///
///
/// @section how_to  How Tos
/// - Social connection (@ref  SocialConnection.h)
/// <ul>
///	<li>@ref social_connection</li>
///	<li>@ref current_user_info</li>
/// </ul>
/// - Friend list (@ref  FriendList.h)
/// <ul>
///	<li>@ref friendlist</li>
/// </ul>
/// - Leaderboard (@ref Leaderboard.h)
/// <ul>
///	<li>@ref leaderboard</li>
/// </ul>
/// - Achievement (@ref  Achievement.h)
/// <ul>
///	<li>@ref achievement</li>
/// </ul>
/// - inAppBilling (@ref  IAPManager.h)
/// <ul>
///	<li>@ref InAppBilling</li>
/// </ul>
/// - Wall (@ref  Wall.h)
/// <ul>
///	<li>@ref wall</li>
///	<li>@ref invitation</li>
/// </ul>
/// - Tracking (@ref Analytics.h)
/// <ul>
///	<li>@ref tracking</li>
///	<li>@ref tracking_Flurry</li>
///	<li>@ref tracking_DNA</li>
/// </ul>
/// - Ad (@ref Ad.h)
/// <ul>
///	<li>@ref ad_display</li>
///	<li>@ref ad_currency</li>
///	<li>@ref ad_interface</li>
/// </ul>
/// - Dynamic Store (@ref DynamicStore.h)
/// <ul>
///	<li>@ref dynamic_store</li>
/// </ul>
/// </ul>
/// - Notifications (@ref Notification.h)
/// <ul>
///	<li>@ref push_notifications</li>
///	<li>@ref local_notifications</li>
/// </ul>
/// - UserProfile (@ref UserProfile.h)
/// <ul>
///	<li>@ref user_profile_init</li>
///	<li>@ref user_profile_data</li>
/// </ul>
/// - Mailbox (@ref Mailbox.h)
/// <ul>
///	<li>@ref user_profile_mailbox</li>
/// </ul>
/// - GameSession (@ref GameSession.h)
/// <ul>
///	<li>@ref user_profile_game_session</li>
/// </ul>
/// - Helpers (@ref GamePreferences.h , @ref DeviceInfo.h , @ref MemManager.h)
/// <ul>
///	<li>@ref gamePreferences</li>
///	<li>@ref deviceInfo</li>
///	<li>@ref memManager</li>
///	<li>@ref unactivateService</li>
///	<li>@ref customHouston</li>
///	<li>@ref logging</li>
/// </ul>
///
///
/// @section support Support
/// To have support contact <A HREF="mailto://mobileSDKSupport@ubisoft.com"> mobile SDK Support </A>
///
/// @author David Bellanger
/// @author Damien Mabin
/// @author Brice Ramard
///
///
/// @page social_connection How to connect to social networks
/// @section title_social_connection Connect to social networks
/// @ref SocialConnection.h handles connection to social networks.
///
/// There is 2 different step to handle while trying to connect your user to a social network:
/// <ol>
///     <li>Handle the first connection of the user to a social network</li>
///     <li>Handle the automatic reconnection of the user to a social network he previously connect.</li>
/// </ol>
/// For <b>step 1</b> we will use the function @ref SocialConnection_CallConnect. It must be called only after a user interaction (clicking on a button "Connect to XXX"):
/// As any functions of the mobile SDK it is separated in 4 sub-functions:
/// 	- A "Call" function: @ref SocialConnection_CallConnect
/// 	- A "Status" function used to know is the request is ended: @ref SocialConnection_StatusConnect
/// 	- A "Result" function to know if you successfully connected to the network or not: @ref SocialConnection_ResultConnect
/// 	- A "Release" function that must be called after you processed the result: @ref SocialConnection_ReleaseConnect. <b>It'll not disconnect you from the social network.</b>
/// <br>You can check at any time if a social network is connected using: @ref SocialConnection_IsConnected
///
/// <strong>SAMPLE</strong> :
/// <div class="usual">
///  <ul class="idTabs">
///    <li><a href="#socialconnectCpp" class="selected">C/C++</a></li>
///    <li><a href="#socialconnectUnity">Unity</a></li>
///  </ul>
///  <div id="socialconnectCpp">
///     @code {.cpp}
///     #include "MobileSDK/social/SocialConnection.h"
///     msdk_s8 requestId;
///
///     void ConnectFacebook()
///     {
///	    	SocialConnection_CallConnect(MSDK_FACEBOOK);
///     }
///
///     void RunLoop()
///     {
///	    if (SocialConnection_StatusConnect(MSDK_FACEBOOK) == MSDK_ENDED)
///	    {
///	    	msdk_Result result = SocialConnection_ResultConnect(MSDK_FACEBOOK);
///
///	    	if(result == MSDK_SUCCESS)
///	    		std_cout << "MSDK_SUCCESS";
///	    	else if(result == MSDK_USER_CANCEL)
///	    		std_cout << "USER CANCEL";
///
///	    	SocialConnection_ReleaseConnect(MSDK_FACEBOOK);
///     	}
///     }
/// @endcode
///  </div>
///  <div id="socialconnectUnity"><pre><code class="cs">static char connectRequestId = MSDK.SocialConnection_CallConnectFacebook();
///MSDK.msdk_Status status = MSDK.SocialConnection_StatusConnect(connectRequestId);</code></pre></div>
/// </div>
///
/// For <strong>step 2</strong> at the launch of the game you must try to automatically reconnect the user to any available social network in your game.<br/>
/// To msdk provide:
///     - @ref SocialConnection_CallAutoConnect that will try to automatically connect to all social network <strong>silently</strong> that mean: without displaying any UI.<br/>
///     - @ref SocialConnection_StatusAutoConnect to get the status of the request.
///     - @ref SocialConnection_ReleaseAutoConnect that must be called after the status is MSDK_ENDED.<br/>
/// There is no result function to autoconnect, but you can check if you are connected to a social network using: @ref SocialConnection_IsConnected .<br/>
///
/// Game circle specific: In Android, if you don't want that your game try to autoconnect to game circle on certain store (like Google play for example) you can define a special "specifics" key in Houston environment switcher. This key must be named "GAME_CIRCLE" and have as value "0" or "1". "1" stand for Game circle activated, "0" for desactivated.<br/>
/// More information about the environment switcher <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=242752527" target="_blank">here</a>.<br/>
///
/// @page current_user_info How to get informations about the current user
/// @section title_current_user Current User
/// function @ref SocialConnection_GetMyInfo returns the informations about the current users on each network he's connected.
/// It returns a PointerArray_msdk_UserInfo pointer. You can check the associated network of the msdk_UserInfo* in with the m_networkId field.
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// msdk_u32 mask = MSDK_GAME_CENTER | MSDK_FACEBOOK;
/// PointerArray_msdk_UserInfo* infos = SocialConnection_GetMyInfo(mask);
///
/// for(int i  = 0; i < infos->count; ++i)
/// {
/// 	if(infos->elem[i]->networkId == MSDK_GAME_CENTER)
///		std_cout << "Game Center username : " << infos->elem[i]->username;
///	else if(infos->elem[i]->networkId == MSDK_FACEBOOK)
///		std_cout << "Facebook username : " << infos->elem[i]->username;
/// }
///
/// @endcode
///
/// @page friendlist How to retreive the friends of the current user
/// @section title_friendlist Retreive the friends of the current user
/// function @ref FriendList_CallFriendsList gets the list of the user's friends
/// This function will try to retreive the friends of the user on all the current connected networks. You can also provide a mask composed by the networks you want the friends to be retreived (for instance if you only want to get friends from GameCenter on not from Facebook even if the user is currently connected to Facebook too)
///
/// function @ref FriendList_StatusFriendsList gives you the status of the request.
///
/// @ref function @ref FriendList_ResultFriendsList returns a reference to a vector of @ref UserInfo pointers.
///
/// function @ref FriendList_ReleaseFriendsList release the request.
///
/// <strong>SAMPLE</strong> :
/// @code {.cpp}
/// msdk_s8 requestId;
///
/// void GetFriendsList()
/// {
///	requestId = FriendList_CallFriendsList();
/// }
///
/// void RunLoop()
/// {
///	if (FriendList_StatusFriendsList(requestId) == MSDK_ENDED)
///	{
///		PointerArray_msdk_UserInfo* users = FriendsList_ResultFriendslist(requestId);
///
/// 		for(int i  = 0; i < users->count; ++i)
///			std_cout << "USERNAME : " << users->elem[i]->username;
///
///		FriendList_ReleaseFriendsList(requestId);
/// 	}
/// }
/// @endcode
///
/// @page leaderboard How to handle scores
/// @section title_leaderboard Leaderboard management
/// @ref Leaderboard handles scores. You can add or retreive scores on Facebook, GameCenter and GameServices. You can add scores on GameCircle.
/// GameCenter is only supported on iOS.
///
/// To add a score use @ref Leaderboard_CallAddScore .
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// msdk_s8 requestId;
/// const char_utf8* leaderboardId = "leaderboard1";
/// const char_utf8* score = "12345";
/// const msdk_u32 mask = MSDK_GAME_CENTER | MSDK_FACEBOOK;
///
/// void AddScore()
/// {
///		requestId = Leaderboard_CallPublishScore(MSDK_FACEBOOK, leaderboardId, score);
/// }
///
/// void RunLoop()
/// {
///	if (Leaderboard_StatusAddScore(requestId) == MSDK_ENDED)
///	{
///		msdk_Result result = Leaderboard_ResultAddScore(requestId);
///		if(result == MSUCCESS)
///			std_cout << "SCORE ADDED";
///
///		Leaderboard_ReleaseAddScore(requestId);
/// 	}
/// }
/// @endcode
/// You have several ways to retreive the scores :
/// <ul>
/// 	<li>@ref Leaderboard_CallScoresList retreive the scores of everybody</li>
/// 	<li>@ref Leaderboard_CallSurroundingScoresList retreive the scores of the players around the current user</li>
/// 	<li>@ref Leaderboard_CallPlayersScoresList retreive the scores of the given players</li>
/// </ul>
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// const char_utf8* leaderboardId = "leaderboard1";
/// const E_RequestScope scope = MSDK_GLOBAL;
///
/// void GetScores()
/// {
///		Leaderboard_CallScoreslist(MSDK_FACEBOOK, scope, leaderboardId, 0, 20);
/// }
///
/// void RunLoop()
/// {
///	if (Leaderboard_StatusScoresList(MSDK_FACEBOOK) == MSDK_ENDED)
///	{
///		Array_msdk_UserScore* scores = Leaderboard_ResultScoresList(MSDK_FACEBOOK);
///
///		for(int = 0; i < scores->count; ++i)
///			std_cout << scores->elem[i].rank;
///
///		Leaderboard_ReleaseScoresList(MSDK_FACEBOOK);
/// 	}
/// }
/// @endcode
///
/// @page achievement How to handle achievements
/// @section title_achievement Achievements
/// @ref Achievement manages the achievements. For now only GameCenter on iOS can be used.
///
/// function @ref Achievement_CallUpdateAchievement allows to add or update an achivement for the current user.
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// msdk_s8 requestId;
/// msdk_UserAchievement* userAchievement = new msdk_UserAchievement(); // In a real project it should be retreived from a call to Achievemnt_CallGameAchievements(MSDK_FACEBOOK)
/// userAchievement->achievementId = "505245669527065";
/// userAchievement->networkId = MSDK_FACEBOOK;
/// userAchievement->achievementUrl = "http://www.dev-gamecfg-mob.ubi.com/facebook/achievement.php?achievement_id=51";
/// userAchievement->percentComplete = 75;
///
/// void UpdateAchievement()
/// {
///		requestId = Achievement_CallUpdateAchievement(userAchievement);
/// }
///
/// void RunLoop()
/// {
///	if (Achievement_StatusUpdateAchievement(requestId) == MSDK_ENDED)
///	{
///		msdk_Result result = Achievement_ResultUpdateAchievement(requestId);
///		if(result == MSUCCESS)
///			std_cout << "SCORE ADDED";
///
///		Achievement_ReleaseUpdateAchievement(requestId);
/// 	}
/// }
/// @endcode
///
/// Several functions are available to get the achievements :
/// <ul>
/// 	<li>@ref Achievement_CallUserAchievements retreive the achievements of the current user</li>
/// 	<li>@ref Achievement_CallGameAchievements retreive the achievements of the game</li>
/// 	<li>@ref Achievement_CallFriendsAchievements retreive the achievements of the user's friends</li>
/// </ul>
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
///
/// void GetGameAchievements()
/// {
///	requestId = Achievement_CallGameAchievements(MSDK_GAME_CENTER);
/// }
///
/// void RunLoop()
/// {
///	if (Achievement_StatusAchievements(requestId) == MSDK_ENDED)
///	{
///		Array_msdk_UserAchievement* achievements = Achievement_ResultAchievements(MSDK_GAME_CENTER);
///
///		for(int i = 0; i < achievements->count; ++i)
///			std_cout << achievement->elem[i].title;
///
///		Achievement_ReleaseAchievements(MSDK_GAME_CENTER);
/// 	}
/// }
/// @endcode
///
/// @page wall How to post on a Facebook wall
/// @section title_wall Post on a wall
/// function @ref Wall_CallPublish try to post on a Facebook wall.
///
/// A BaseMessage contains a @ref msdk_UserInfo and the content of the message as a char_utf8 pointer.
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/social/communication/Wall.h"
/// void PostOnWall()
/// {
///	msdk_BaseMessage* message = NULL;
///	if(SocialConnection_IsConnected(MSDK_FACEBOOK) == MSDK_TRUE)
///		message = Message_Create("This is a test message !", SocialConnection_GetMyInfo(MSDK_FACEBOOK));
///
///	Wall_CallPublish(message);
/// }
///
/// void RunLoop()
/// {
///	if (Wall_StatusPublish() == MSDK_ENDED)
///	{
///		msdk_Result result = Wall_ResultPublish();
///		if(result == MSUCCESS)
///			std_cout << "MESSAGE POSTED";
///
///		Wall_ReleasePublish();
/// 	}
/// }
/// @endcode
///
/// A message can be specialized. For example we can add a Facebook action/object to a message.
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// void PostActionObject()
/// {
///	msdk_BaseMessage* message = NULL;
///	if(SocialConnection_IsConnected(MSDK_FACEBOOK) == MSDK_TRUE)
/// {
///		message = Message_Create("This is a test message !", SocialConnection_GetMyInfo(MSDK_FACEBOOK));
///		Message_AddActionObjectFacebook(message, "write", "article", "http://samples.ogp.me/434264856596891");
///	}
///
///	Wall_CallPublish(message);
/// }
///
/// void RunLoop()
/// {
///	if (Wall_StatusPublish() == MSDK_ENDED)
///	{
///		msdk_Result result = Wall_ResultPublish();
///		if(result == MSUCCESS)
///			std_cout << "SCORE ADDED";
///
///		Wall_ReleasePublish();
/// 	}
/// }
/// @endcode
///
/// @page invitation How send an invitation
/// @section title_wall Invite
/// function @ref Invitation_CallInvite send a request to a friend.
/// Only Facebook is currently supported
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/social/communication/Invitation.h"
///
/// void InviteFriend()
/// {
///	    Invitation_CallInvite(MSDK_FACEBOOK, "Test Invite", "join me", NULL, NULL);
/// }
///
/// void RunLoop()
/// {
///	    if Invitation_StatusInvite(MSDK_FACEBOOK) == MSDK_ENDED)
///	    {
///		    msdk_Result result = Invitation_ResultInvite(MSDK_FACEBOOK);
///		    if(result == SUCCESS)
///				std_cout << "INVITE SUCCESS";
///			else
///				std_cout << "INVITE FAILED";
/// 	}
/// }
/// @endcode
///
/// @page InAppBilling How to use in-application billing
/// @section title_inappbilling InAppBilling management
///
/// |     iOS                  |      Android              |   Windows8          | Windows Phone8    |
/// | :----------------------: | :-----------------------: |:------------------: |:----------------: |
/// | Apple InAppBiling        | Google Play InAppBilling  | Windows Store IAB   | Windows Store IAB |
/// |                          | Amazon InAppBilling       |                     |                   |
/// |                          | Samsung InAppBilling      |                     |                   |
///
/// To configure in-app billing on iOS follow instructions at <a href="https://developer.apple.com/library/ios/documentation/LanguagesUtilities/Conceptual/iTunesConnect_Guide/13_ManagingIn-AppPurchases/ManagingIn-AppPurchases.html">"ios dev center"</a>
///
/// To configure in-app billing on Google play follow instructions at <a href="https://developer.android.com/google/play/billing/billing_admin.html">"Administering In-app billing"</a>
///
/// To configure in-app billing on Amazon follow instructions at <a href="https://developer.amazon.com/sdk/in-app-purchasing/reference/faq.html">"Amazon Distribution portal FAQ"</a>
///
///
///
///
/// The MobileSDK handles two type of products : MSDK_PT_MANAGED, MSDK_PT_CONSUMABLE (@ref msdk_ProductType).
///
/// Managed items can be bought only once and the user will have it forever (map, level, ...).
/// If the user changes his device he'll be able to refund these items. You must provide an option to do this.
///
/// Consumable items can be bought several times. You must consume the item once it is bought and credit the user with it inside your game.
/// The purchase will fail if you try to buy a consumable item that has already been bought but not consumed.
///
/// Here is the flow to follow in your game to handle in app billing
/// @image html iab-flow.png
///
/// <h2>Amazon specific</h2>
/// For Amazon you have to insert the following receiver inside the application element in your application's manifest:
/// @code {.xml}
/// <receiver
///     android:name="com.amazon.inapp.purchasing.ResponseReceiver"
///     android:exported="true" >
///     <intent-filter>
///         <action
///             android:name="com.amazon.inapp.purchasing.NOTIFY"
///             android:permission="com.amazon.inapp.purchasing.Permission.NOTIFY" />
///     </intent-filter>
/// </receiver>
/// @endcode
/// For amazon it is also compulsory to setup the id of each pack you plan to purchase on HOUSTON, more information <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925274">here</a>.
///
/// <h2>Google play specific</h2>
/// You must also add in your manifest, before the <application ...> tag:
/// @code {.xml}
/// <uses-permission android:name="com.android.vending.BILLING" />
/// @endcode
///
/// <h2>Samsung apps specific</h2>
/// For samsung apps the init function of in app purchase may return specific error code:
/// 	* MSDK_SAMSUNG_IAP_NOT_INSTALLED
/// 	* MSDK_SAMSUNG_IAP_NEED_UPDATE
/// In both cases you should display a pop up to the user to explain that he needs to update samsung In App Purchase application to be able to purchase the in game content.
/// After the user click on "OK" you should call the function : @ref IAPManager_SamsungApps_IapRedirection to redirect him to the Samsung Apps page of the In App Purchase Application of Samsung.
///
/// You must also add in your manifest, before the <application ...> tag:
/// @code {.xml}
/// <!-- ============================= Samsung In App Billing ================================= -->
/// <uses-permission android:name="com.sec.android.iap.permission.BILLING"/>
/// <!-- ====================================================================================== -->
/// @endcode
/// <h2>Windows 8 specific</h2>
/// On windows 8 your are able to test in app purchase before making any server side configuration.
/// You must create a file at the root of your project in the folder "data" (If you don't have tht folder please create it)
/// The file must be named: "in-app-purchase.xml". It will contain the list of in app purchase available as well as in app purchase title, price, type (Consumable / durable) etc.
/// The format documentation is available <a href="http://msdn.microsoft.com/en-us/library/windows/apps/windows.applicationmodel.store.currentappsimulator" target="_blank">here</a> (in the examples section).
/// Below is a example of an in-app-purchase.xml file:
/// @code {.xml}
/// <?xml version="1.0" encoding="utf-16" ?>
/// <CurrentApp>
///   <ListingInformation>
///     <App>
///       <AppId>00000000-0000-0000-0000-000000000000</AppId>
///       <LinkUri>http://apps.microsoft.com/webpdp/app/00000000-0000-0000-0000-000000000000</LinkUri>
///       <CurrentMarket>en-US</CurrentMarket>
///       <AgeRating>3</AgeRating>
///       <MarketData xml:lang="en-us">
///         <Name>AppName</Name>
///         <Description>AppDescription</Description>
///         <Price>1.00</Price>
///         <CurrencySymbol>$</CurrencySymbol>
///         <CurrencyCode>USD</CurrencyCode>
///       </MarketData>
///     </App>
///     <Product ProductId="product1" ProductType="Durable">
///       <MarketData xml:lang="en-us">
///         <Name>Product1Name</Name>
///         <Price>1.00</Price>
///         <CurrencySymbol>$</CurrencySymbol>
///         <CurrencyCode>USD</CurrencyCode>
///       </MarketData>
///     </Product>
///     <Product ProductId="product2" ProductType="Consumable">
///       <MarketData xml:lang="en-us">
///         <Name>Product2Name</Name>
///         <Price>2.00</Price>
///         <CurrencySymbol>$</CurrencySymbol>
///         <CurrencyCode>USD</CurrencyCode>
///       </MarketData>
///     </Product>
///   </ListingInformation>
///   <LicenseInformation>
///     <App>
///       <IsActive>true</IsActive>
///       <IsTrial>true</IsTrial>
///     </App>
///   </LicenseInformation>
/// </CurrentApp>
/// @endcode
///
/// <h2>iOS specific</h2>
///	In order to test in app purchase you need to create a test account on itunesConnect and use it when asked for credentials during the buy request.
///
/// <h2>SAMPLE :</h2>
///
/// IAB initialization :
/// @code {.cpp}
/// #include "MobileSDK/iab/IAPManager.h"
///
/// std::map<msdk_s8, const char_utf8*, MyCharComparator> consumingProdcuts;
/// std::map<msdk_s8, const char_utf8*, MyCharComparator>::iterator it;
/// int state = NO_STATE;
/// void IABInit()
/// {
///	Array_msdk_IAPProduct* products = new Array_msdk_IAPProduct();
///	products->count = 2;
///	products->elem = (msdk_IAPProduct*)malloc(2 * sizeof(msdk_IAPProduct));
/// IAPProduct_Init(&(products->elem[0]));
///	products->elem[0].productType = MSDK_PT_CONSUMABLE;
///	products->elem[0].productIdentifier = "ubi.mobilesdk.samplegame.goldpack1";
/// IAPProduct_Init(&(products->elem[1]));
///	products->elem[1].productType = MSDK_PT_MANAGED;
///	products->elem[1].productIdentifier = "ubi.mobilesdk.samplegame.map1";
/// 	IAPManager_CallInit(products);
///	state = IAB_INIT;
/// }
///
/// void RunLoop()
/// {
///
///		if(state == IAB_INIT && IAPManager_StatusInit() == MSDK_ENDED)
///		{
///			if(IAPManager_ResultInit() == MSDK_SUCCESS)
///			{
///				const Array_msdk_IAPProduct* productMap = IAPManager_GetKnownProductMap();
///				for(unsigned int i = 0; i < productMap->count; ++i)
///				{
///					if(productMap->elem[i].productType == MSDK_PT_CONSUMABLE && productMap->elem[i].productStatus == MSDK_PS_OWNED)
///					{
///						consumingProducts[productMap->elem[i].productIdentifier] = IAPManager_CallConsumeProduct(productMap->elem[i].productIdentifier);
///					}
///				}
///				state = CONSUME;
///			}
///			IAPManager_ReleaseInit();
///		}
///		if(state == CONSUME)
///		{
///			for(it = consumingProducts.begin(); it != consumingProducts.end(); ++it)
///			{
///				if(IAPManager_StatusConsumeProduct(it->first) == MSDK_ENDED)
///				{
///					if(IAPManager_ResultConsumeProduct(it->first) == MSDK_SUCCESS)
///						CreditItemForUser(it->second);
///					IAPManager_ReleaseConsumeProduct(it->first);
///					it = consumingProducts.erase(it);
///				}
///			}
///			if(consumingProducts.size() == 0)
///				state = IAB_INIT_END;
///		}
/// }
/// @endcode
/// To buy a product use @ref IAPManager_CallBuyUsingProductId.
/// You <strong>MUST</strong> init the store before trying to buy a product.
/// If it is a consumable item you have to consume it.
///
/// <strong>SAMPLE :</strong>
/// Buy an item
/// @code {.cpp}
/// #include "MobileSDK/iab/IAPManager.h"
///
/// msdk_s8 requestId;
/// int state = BUY;
///
/// BuyUsingProductId()
/// {
///     requestId = IAPManager_CallBuyUsingProductId("ubi.mobilesdk.samplegame.goldpack1");
/// }
///
/// RunLoop()
/// {
///     if (state == BUY && IAPManager_StatusBuyUsingProductId(requestId) == MSDK_ENDED)
///     {
/// 		msdk_PurchaseResult* result = IAPManager_ResultBuyUsingProductId(buyRequestId);
/// 		if(result->transactionStatus == MSDK_TS_COMPLETED)
///			{
/// 			// Transaction success
///				if(result->product->productType == MSDK_PT_CONSUMABLE)
///				{
///					IAPManager_ReleaseBuyUsingProductId(requestId);
///					requestId = IAPManager_CallConsumeProduct(result->product->productIdentifier);
///					state = CONSUME;
///				}
///				else
///					IAPManager_ReleaseBuyUsingProductId(requestId);
///			}
/// 		else
///			{
/// 			// Transaction failed
///				IAPManager_ReleaseBuyUsingProductId(requestId);
///			}
///     }
///		if(state == CONSUME && IAPManager_StatusConsumeProduct(requestId) == MSDK_ENDED)
///		{
///			msdk_PurchaseResult result = IAPManager_ResultConsumeProduct(requestId);
///			if(result->transactionStatus == MSDK_TS_COMPLETED)
///				CreditUserItem();
///			IAPManager_ReleaseConsumeProduct(requestId);
///		}
/// }
/// @endcode
///
///
/// @page tracking How to track events
/// @section Tracking
/// @ref Analytics allows you to track some event
///
/// You can track custom events with custom parameters.
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/tracking/Analytics.h"
///
/// PointerArray_msdk_key_value* keyValues = static_cast<PointerArray_msdk_key_value*>(malloc(sizeof(PointerArray_msdk_key_value)));
/// keyValues->elem = static_cast<msdk_key_value**>(malloc(sizeof(msdk_key_value*)*2));
/// keyValues->elem[0] = static_cast<msdk_key_value*>(malloc(sizeof(msdk_key_value));
/// keyValues->elem[1] = static_cast<msdk_key_value*>(malloc(sizeof(msdk_key_value));
/// keyValues->count = 2;
/// keyValues->elem[0]->key = "key1";
/// keyValues->elem[0]->value = "value1";
/// keyValues->elem[1]->key = "key2";
/// keyValues->elem[1]->value = "value2";
/// Analytics_TrackEvent("MyEvent", keyValues, MSDK_FALSE);
/// @endcode
///
/// It is also possible to track <strong>timed</strong> events.
/// To do so you only have to add MSDK_TRUE add the end of the Analytics_TrackCustomEvent function :
///
/// @code {.cpp}
/// Analytics_TrackEvent("MyEvent", keyValues, MSDK_TRUE);
/// @endcode
///
/// To end a timed event you need to call @ref Analytics_EndTimedEvent function to end it.
/// If you don't call this function the timed event will be ended when the application goes to background or is killed
///
/// @page tracking_Flurry How to use Flurry
/// To use Flurry you need to link with the Flurry library.
/// Once it is done you can use Flurry with the following code before the init of the MobileSDK:
///
/// @code {.cpp}
/// #include "MobileSDK/tracking/FlurryTracking.h"
/// Analytics_ActivateFlurry();
/// @endcode
///
///
/// @page tracking_DNA How to use DNA
/// To use DNA you need to link with the Ubiservices library.
/// This library is available here : https://pdc-mob-hwh01.ubisoft.org/buildServer/ubiservices/release/?C=M;O=D
/// For more information use the following links :
/// <ul>
/// <li><a href="https://mdc-web-tomcat17.ubisoft.org/confluence/display/oms/Ubiservices+iOS">iOS setup</a></li>
/// <li><a href="https://mdc-web-tomcat17.ubisoft.org/confluence/display/oms/Ubiservices">android setup</a></li>
/// </ul>
///
/// You also need to specify your UbiServices App ID and UbiServices App Version on Houston.
///
/// Once it is done you can use DNA with the following code before the init of the MobileSDK:
///
/// @code {.cpp}
/// #include "MobileSDK/tracking/DNATracking.h"
/// Analytics_ActivateDNA(); // During development you might need to use Analytics_ActivateDNAOnUAT instead
/// @endcode
///
/// By default DNA handles custom events and some special events. In order to use these special events you need to use specific strings as parameters of the TrackEvent function.
/// <table>
/// <tr>
/// 	<td>EventId</td>
/// 	<td>Parameters</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_UNLOCK_ACHIEVEMENT</td>
/// 	<td>@ref MSDK_DNA_ACHIEVEMENT_ID</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_PLAYER_ACHIEVEMENT</td>
/// 	<td>@ref MSDK_DNA_ACHIEVEMENT_ID</td>
/// </tr>
/// <tr>
/// 	<td rowspan="2">@ref MSDK_DNA_CONTEXT_START</td>
/// 	<td>@ref MSDK_DNA_CONTEXT_TYPE</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_CONTEXT_NAME</td>
/// </tr>
///
/// <tr>
/// 	<td rowspan="2">@ref MSDK_DNA_CONTEXT_STOP</td>
/// 	<td>@ref MSDK_DNA_CONTEXT_TYPE</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_CONTEXT_NAME</td>
/// </tr>
/// <tr>
/// 	<td rowspan="2">@ref MSDK_DNA_PLAYER_STAT</td>
/// 	<td>@ref MSDK_DNA_STAT_NAME</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_STAT_VALUE</td>
/// </tr>
/// <tr>
/// 	<td rowspan="4">@ref MSDK_DNA_LOCALIZATION</td>
/// 	<td>@ref MSDK_DNA_LOCALE_CODE</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_GAME_LANGUAGE</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_SUBTITLES_ACTIVATION</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_SUBTITLES_LANGUAGE</td>
/// </tr>
/// <tr>
/// 	<td rowspan="2">@ref MSDK_DNA_PROGRESSION</td>
/// 	<td>@ref MSDK_DNA_PROGRESSION_TYPE</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_PROGRESSION_VALUE</td>
/// </tr>
/// <tr>
/// 	<td>@ref MSDK_DNA_PLAYER_DLC</td>
/// 	<td>@ref MSDK_DNA_DLC_ID</td>
/// </tr>
/// </table>
///
/// @page ad_display How to display an ad
/// @section ad_display_section Display an ad
/// Before displaying an ad you have to initialize the ad module.
///
/// <strong>EXAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/ad/Ad.h"
///
/// msdk_s8 requestId;
///
/// InitAds()
/// {
///     requestId = Ad_Callinit();
/// }
///
/// CheckInitAdRequest()
/// {
///     msdk_Status status = Ad_StatusInit(requestId);
///     if (MSDK_ENDED == status || MSDK_TIMED_OUT == status)
///     {
///         if (MSDK_ENDED == status)
///         {
///             msdk_Result result = Ad_ResultInit(requestId);
///             if(result == MSDK_SUCCESS)
///                 std::cout << "Init success";
///             else
///                 std::cout << "Init failed";
///         }
///         Ad_ReleaseInit(requestId);
///     }
/// }
/// @endcode
///
/// Once the ad module is initialized you're able to display an ad. To display an ad you first need to preload it. When you preload an ad
/// you can choose its type (@ref msdk_AdType). You also need to give the view you want the ad to be displayed to (RelativeLayout on android, UIView* on iOS).
///
/// <strong>EXAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/ad/Ad.h"
///
/// msdk_s8 requestId;
///
/// PreloadAd()
/// {
///		msdk_AdInfo adInfo;
///		adInfo.type = MSDK_OFFERWALL;
///		adInfo.adView = [[[[UIApplication sharedApplication] keyWindow] rootViewController]view];
///     requestId = Ad_Preload(&adInfo);
/// }
///
/// CheckPreloadAdRequest()
/// {
///     msdk_Status status = Ad_StatusPreload(requestId);
///     if (MSDK_ENDED == status || MSDK_TIMED_OUT == status)
///     {
///         if (MSDK_ENDED == status)
///         {
///             nsdk_Result result = Ad_ResultPreload(requestId);
///             if(result == MSDK_SUCCESS)
///                 // Preload success;
///             else
///                 // Preload failed;
///         }
///         Ad_Display(requestId); // This will display the ad
///     }
/// }
/// @endcode
///
/// To display a preloaded ad you only need to call the Ad_Display function with the id of the Ad as argument.
///
/// Three events can be catch by your ads (AdEvents @ref msdk_adEvents).
///
/// <strong>EXAMPLE :</strong>
///
/// @code {.cpp}
/// Array_msdk_AdEvents* events =  Ad_Events(id);
/// if(events != NULL)
/// {
/// 	for(unsigned int i = 0; i < events->count; ++i)
///	{
///		if(events->elem[i] == MSDK_AD_EVENT_CLICKED)
///			std::cout << "Ad Clicked";
///		else if(events->elem[i] == MSDK_AD_EVENT_OPEN)
///			std::cout << "Ad Opened";
///		else if(events->elem[i] == MSDK_AD_EVENT_CLOSE)
///			std::cout << "Ad Closed";
///	}
/// }
/// @endcode
///
/// To delete an ad you have to call Ad_Delete with the id of the ad has parameter
///
/// @page ad_currency How to get the user's virtual currency
/// @section ad_currency_section Get the user's virtual currency
///
/// When you are using rewarded ads you want to be able to receive the virtual currency earned by the user.
///
/// <strong>EXAMPLE :</strong>
///
/// @code {.cpp}
/// msdk_s8 requestId;
///
/// GetVirtualCurrency()
/// {
///     requestId = Ad_CallVirtualCurrency();
/// }
///
/// CheckVirtualCurrencyRequest()
/// {
///     msdk_Status status = Ad_StatusVirtualCurrency(requestId);
///     if (MSDK_ENDED == status || MSDK_TIMED_OUT == status)
///     {
///         if (MSDK_ENDED == status)
///         {
///             Array_msdk_VirtualCurrency* result = Ad_ResultVirtualCurrency(requestId);
///             if(result != NULL)
///             {
///             	for(unsigned int i = 0; i < result->count; ++i)
///				std::cout << "Amount : " << result->elem[i].value << " for currency : " << result->elem[i].name;
///             }
///		Ad_ReleaseVirtualCurrency(requestId);
///         }
///     }
/// }
/// @endcode
/// Once you've retreived the virtual currency of the user it cannot be retreived again (until the next time the user will be rewarded) so you have to store it inside your game.
///
/// @page ad_interface How to add your own ad provider <strong>(Advanced)</strong>
/// @section Add your own ad provider
///
/// The MobileSDK allows you to add your own ad provider. To do so you have to create a msdk_AdInterface structure and pass it to Ad_AddProvider.
/// This structure contains function' pointers and several attributes (@ref msdk_AdInterface)
/// If the name field of the interface matches a name in the AdManager in ubi website the configuration of it will be taken from the server.
///
/// Here is an example of an interface for Flurry
/// <strong>EXAMPLE :</strong>
/// @code {.cpp}
/// #include "ad/FlurryAd.h"
/// #include "bindings/common/FlurryBindings.h"
/// #include "ad/Ad.h"
/// #include "common/MemManager.h"
///
/// std::map<const char_utf8*, msdk_s8, CharCompFunctor>	e_flurryAdSpaces;
///
/// void	Flurry_Init()
/// {
/// 	Common_Log(MSDK_LOG_VERBOSE, "Enter Ad_InitFlurryAds()");
/// 	MobileSDKAPI::FlurryBindings::FlurryInitAds();
/// 	if(MobileSDKAPI::Init::s_mode == MSDK_DEBUG)
/// 	{
/// 		MobileSDKAPI::FlurryBindings::FlurryEnableTestAds(true);
/// 	}
/// 	MobileSDKAPI::FlurryBindings::FlurrySetUserId(DeviceUID());
/// 	Common_Log(MSDK_LOG_VERBOSE, "Leave Ad_InitFlurryAds");
/// }
///
/// void Flurry_CallPreload(struct msdk_AdInterface* p_ad)
/// {
/// 	if(p_ad != NULL)
/// 	{
/// 		e_flurryAdSpaces[msdk_AdType_String(p_ad->adType)] = p_ad->adId;
/// 		MobileSDKAPI::FlurryBindings::FlurryFetchAd(msdk_AdType_String(p_ad->adType), p_ad->adType, p_ad->adView);
/// 	}
/// }
///
/// void Flurry_Display(struct msdk_AdInterface* p_ad)
/// {
/// 	MobileSDKAPI::FlurryBindings::FlurryDisplayAd(msdk_AdType_String(p_ad->adType), p_ad->adView);
/// }
///
/// void Flurry_Visibility(struct msdk_AdInterface* p_ad, char p_visibility)
/// {
/// 	if(p_visibility == MSDK_TRUE)
/// 		MobileSDKAPI::FlurryBindings::FlurryDisplayAd(msdk_AdType_String(p_ad->adType), p_ad->adView);
/// 	else
/// 		MobileSDKAPI::FlurryBindings::FlurryRemoveAd(msdk_AdType_String(p_ad->adType), p_ad->adView);
/// }
///
/// void Flurry_Delete(struct msdk_AdInterface* p_ad)
/// {
/// 	MobileSDKAPI::FlurryBindings::FlurryRemoveAd(msdk_AdType_String(p_ad->adType), p_ad->adView);
/// }
///
/// void Flurry_DestroyInstance(struct msdk_AdInterface* p_ad)
/// {
/// 	msdk_Free(p_ad);
/// }
///
/// struct msdk_AdInterface* Flurry_CreateInstance()
/// {
/// 	struct msdk_AdInterface* interface = static_cast<struct msdk_AdInterface*>(msdk_Alloc(sizeof(struct msdk_AdInterface)));
/// 	interface->adId = -1;
/// 	interface->adType = MSDK_BANNER_TOP;
/// 	interface->adView = NULL;
/// 	interface->events = NULL;
/// 	interface->internals = NULL;
/// 	interface->preloadResult = MSDK_UNKNOW_ERROR;
/// 	interface->preloadStatus = MSDK_NOT_ALLOCATED;
/// 	interface->name = "flurry";
/// 	interface->isClicked = MSDK_FALSE;
/// 	if(MobileSDKAPI::Init::s_ProductPreferences.GetValue(MSDK_FLURRY_REWARD_CHECK) != NULL)
/// 		interface ->rewardCheckUrl = const_cast<char_utf8*>(MobileSDKAPI::Init::s_ProductPreferences.GetValue(MSDK_FLURRY_REWARD_CHECK));
/// 	else
/// 		interface->rewardCheckUrl = NULL;
/// 	interface->CallPreload = Flurry_CallPreload;
/// 	interface->CreateInstance = Flurry_CreateInstance;
/// 	interface->Delete = Flurry_Delete;
/// 	interface->Display = Flurry_Display;
/// 	interface->Init = Flurry_Init;
/// 	interface->Visibility = Flurry_Visibility;
/// 	interface->DestroyInstance = Flurry_DestroyInstance;
/// 	return interface;
/// }
///
/// // Now we add the interface to the MobileSDK
/// Ad_AddProvider(Flurry_CreateInstance());
/// @endcode
///
/// @page dynamic_store How to use the Dynamic store
/// The Dynamic Store allow you to handle players' payments tgrough the Primary and the Secondary store.
/// It also contains functions to manage the items that the player possessed
/// @section dynamic_store_prerequisites Prerequisites
/// First of all you need to configure your items on the Houston server and download the MobileSDK database file : <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925257">Houston documentation</a>)
///
/// @section dynamic_store_initialization Initialization
/// The first step when using the DynamicStore
/// @code {.cpp}
/// void InitDynamicStore()
/// {
///    Store_CallInit();
/// }
///
/// void RunLoop()
/// {
///     if(Store_StatusInit() == MSDK_ENDED)
///		{
///			if(Store_ResultInit() == MSDK_SUCCESS)
///				std::cout << "Initialization succeeded";
///		}
/// }
/// @endcode
///
/// When you don't need the DynamicStore anymore call the @ref Store_Release function.
/// This function will free all the memory allocated by the dynamic store.
/// If you never destroy the Store, the items will be automatically updated on resume of the application.
///
/// @section dynamic_store_retreive_store Retreive a store
/// For each store there is a function to retreive it.<br/>
/// For the primary store you have to call @ref Store_GetPrimaryStore <br/>
/// For the secondary store you have to call @ref Store_GetSecondaryStore <br/>
///
/// @section dynamic_store_buy_item Buy an item
/// <strong>Secondary store :</strong><br/>
/// In order to buy an item in the secondqry store you only hqve to call @ref Store_BuySecondaryItem
/// it will return the number of the bought items now owned by the user.
///
/// <strong>Primary store :</strong><br/>
/// SAMPLE :
/// @code {.cpp}
/// msdk_PrimaryStoreItem* item; // Get it from a call to Store_GetPrimaryStore
/// void BuyItem()
/// {
///    Store_CallBuyPrimaryItem(item);
/// }
///
/// void RunLoop()
/// {
///     if(Store_StatusBuyPrimaryItem(item) == MSDK_ENDED)
///		{
///			msdk_PrimaryPurchaseResult result = Store_ResultBuyPrimaryItem(item);
///			if(result.transactionStatus == MSDK_TS_COMPLETED)
///				std::cout << "Transaction completed for item : " << result.item->storePack->productIdentifier;
///			Store_ReleaseBuyPrimaryItem(item);
///		}
/// }
/// @endcode
///
/// When items are bought the UserItem table is updated and persisted to the database so you can retreive them later.
/// When a purchase is made trough the secondary store the price of the items (x coins for instance) are substracted to the user's items
///
/// @section dynamic_store_items Game's and Player's items
/// The MobileSDK store the game's and the player's item in a database.
///
/// To get the game items :
/// @code {.cpp}
///  Array_msdk_StoreItem* gameItems = Store_GetGameItems();
/// @endcode
///
/// To get the user items :
/// @code {.cpp}
///  Array_msdk_StoreItem* gameItems = Store_GetUserItems(); // read the amount attribute to see how many items of a type the user owns
/// @endcode
///
/// If you only want to know the quantity of 1 item that owns the player :
/// @code {.cpp}
/// msdk_u32 itemQuantity = Store_GetItemAmount(item);
/// @endcode
///
/// It can happen that your game credit an item to a player because he made an action in your game (without the player to buy it).
/// For this purpose we provide a function to directly add items to a player :
/// @code {.cpp}
/// Store_CreditItem(item) ;
/// @endcode
///
/// @section dynamic_store_custom Custom requests
/// On the Houston server you can specify custom requests to perform on the database.
/// You can access these requests trough the MobileSDK :
/// @code {.cpp}
/// Array_msdk_PrimaryStoreItem* primaryItems = Store_PrimaryStoreRequest("MyRequest1");
/// Array_msdk_SecondaryStoreItem* secondaryItems == Store_SecondaryStoreRequest("MyRequest2");
/// @endcode
///
/// @page push_notifications How to manage push notifications
/// When you initialize the MobileSDK it will try to register your device for push notifications.
/// On Android it is automatically done if you setup the key <strong>Google GCM sender ID</strong>
/// On iOS since the user has to validate the use of push notifications for your app you have to call it when the user is in a good mood.
/// To do so use the following function :
///
/// @code {.cpp}
/// #include "MobileSDK/social/communication/Notification.h"
/// Notification_RegisterForPush();
/// @endcode
///
/// If you don't want to be registered for push notfications on Android, add the key PUSH_NOTIFICATION inside the environment switcher and set it to 0.
///
/// If you want to process the datas sent inside the notifications you need to register a callback.
///
/// @code{.cpp}
/// void PushCallback(const char_utf8* p_notifications, msdk_s32 p_index)
/// {
/// // Some code
/// }
///
/// void SomeFunction()
/// {
/// 	// ... some code
/// 	msdk_Init(&initFields);
/// 	Notification_RegisterPushCallback(PushCallback); // to do after the init of the MobileSDK
/// }
/// @endcode
///
/// On iOS you have other functions to call to make it work :
/// <ul>
/// <li>@ref Notification_OnLaunch</li>
/// <li>@ref Notification_OnRegisterSuccess</li>
/// <li>@ref Notification_OnRegisterFail</li>
/// <li>@ref Notification_OnRemote</li>
/// </ul>
///
/// @page local_notifications How to manage local notifications
/// You can create local notifications and schedule them so they will be trigerred when you want to.
/// You can set Several properties for your notification (title, spund, body, icon,...).
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// msdk_LocalNotification*	notif = Notification_CreateLocal();
/// Notification_SetBody(notif, "test");
/// Notification_SetFireTime(notif, 30);
/// Notification_ScheduleLocal(notif);
/// @endcode
///
/// If the notification is trigered while the user is playing or if the user press the notification outside
/// of the game the callback you've register trough Notification_RegisterPushCallback will be called (see @ref local_notifications).
///
/// @page gamePreferences How to save / get key values from a local database
/// @section GamePreferences
///
/// We provide a way to store key/values inside a local database using SQLite 3.0.
/// The stored values are encrypted using the key you've given trough the initialization fields.
///
/// <strong>SAMPLE :</strong>
///
/// @code {.cpp}
/// //Store values
/// GamePreferences_UpdateString("MyKeyString", "MyValue");
/// GamePreferences_UpdateInt("MyKeyint", 12);
/// GamePreferences_UpdateFloat("MyKeyint", 12.99f);
///
/// //Get Values (if no value is found the default value will be returned but not stored inside de database)
/// const char_utf8* valueString = GamePreferences_GetString("MyKeyString", "MyDefaultResult");
/// int valueInt = GamePreferences_GetInt("MyKeyInt",0);
/// float valueFloat = GamePreferences_GetFloat("MyKeyFloat", 0f);
/// @endcode
///
/// The changes you make are cached and only persisted to the database when the msdk_Pause or msdk_UnInit function is called.
/// To force the changes to be saved during the game you can call GamePreferences_Flush
///
/// <strong>SAMPLE :</strong>
///
/// @code {.cpp}
/// GamePreferences_Flush();
/// @endcode
///
/// @page user_profile_init How to initialize the UserProfile
/// Here is the presentation of the UserProfile : <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/display/oms/User-Profile">UserProfile</a>
///
/// You must call UserProfile_CallInit before using any function related to the UserProfile.
///
/// Since the UserProfile is linked to the connected social accounts in the game you need to call SocialConnection_CallAutoConnect before initializing the UserProfile.
///
/// The UserProfile won't be initialized until the auto connection is done.
///
/// While initializing the UserProfile or connect to a social network your game must display popups to the user.
/// This is needed to merge social accounts to the device or together and to switch idendity.
///
/// 3 popup might be displayed :
/// <ul>
/// <li>One popup to merge accounts together (the 2 accounts will be linked to a single identity).</li>
/// <li>One popup to choose which identity to use in case of conflict</li>
/// <li>One popup to alert the player that he has logged out and changed its identity</li>
/// </ul>
/// MobileSDK provides a function to register implementation of these 3 popups.
/// You <strong>MUST</strong> implement these popups and pass it to the MobileSDK before initializing the UserProfile.
///
/// Here is a simple example that only uses alert window on iOS :
///
/// @code {.cpp}
///#include "MobileSDK/social/profile/UserProfile.h"
///
///void DisplayAccountSelection(msdk_ProfileInfo* p_currentprofile, msdk_ProfileInfo* p_newProfile)
///{
///    UIAlertController* alert = [UIAlertController alertControllerWithTitle:@"Account selection" message:@"Which account would you like to use ?" preferredStyle:UIAlertControllerStyleAlert];
///    NSMutableString* currentProfile = [[NSMutableString alloc] init];
///
///    for(msdk_u32 i = 0; i < p_currentprofile->accounts.count; ++i)
///    {
///        if(p_currentprofile->accounts.elem[i]->username != NULL)
///        {
///            [currentProfile appendFormat:@"%s", p_currentprofile->accounts.elem[i]->username];
///        }
///        [currentProfile appendFormat:@"(%s) ", msdk_NetworkId_string(p_currentprofile->accounts.elem[i]->networkId)];
///    }
///    UIAlertAction* account1 = [UIAlertAction actionWithTitle:currentProfile style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
///            UserProfile_OnAccountSelected(p_currentprofile); // Send which account has been selected to the MobileSDK
///            [alert dismissViewControllerAnimated:YES completion:nil];
///    }];
///
///    NSMutableString* newProfile = [[NSMutableString alloc] init];
///
///    for(msdk_u32 i = 0; i < p_newProfile->accounts.count; ++i)
///    {
///        if(p_newProfile->accounts.elem[i]->username != NULL)
///        {
///            [newProfile appendFormat:@"%s(%s) ", p_newProfile->accounts.elem[i]->username, msdk_NetworkId_string(p_newProfile->accounts.elem[i]->networkId)];
///        }
///    }
///    UIAlertAction* account2 = [UIAlertAction actionWithTitle:newProfile style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
///        UserProfile_OnAccountSelected(p_newProfile);// Send which account has been selected to the MobileSDK
///        [alert dismissViewControllerAnimated:YES completion:nil];
///    }];
///
///    [alert addAction:account1];
///    [alert addAction:account2];
///
///     UIViewController* mainView = [[UIApplication sharedApplication].keyWindow rootViewController];
///    [mainView presentViewController:alert animated:YES completion:nil];
///}
///
///void DisplayMergeAccounts(msdk_ProfileInfo* p_currentprofile, msdk_ProfileInfo* p_newProfile)
///{
///    NSMutableString* currentProfile = [[NSMutableString alloc] init];
///
///    for(msdk_u32 i = 0; i < p_currentprofile->accounts.count; ++i)
///    {
///        if(p_currentprofile->accounts.elem[i]->username != NULL)
///        {
///            [currentProfile appendFormat:@"%s", p_currentprofile->accounts.elem[i]->username];
///        }
///        [currentProfile appendFormat:@"(%s) ", msdk_NetworkId_string(p_currentprofile->accounts.elem[i]->networkId)];
///    }
///
///    NSMutableString* newProfile = [[NSMutableString alloc] init];
///
///    for(msdk_u32 i = 0; i < p_newProfile->accounts.count; ++i)
///    {
///        if(p_newProfile->accounts.elem[i]->username != NULL)
///        {
///            [newProfile appendFormat:@"%s(%s) ", p_newProfile->accounts.elem[i]->username, msdk_NetworkId_string(p_newProfile->accounts.elem[i]->networkId)];
///        }
///    }
///
///    NSString* message = [[NSString alloc] initWithFormat:@"Would you like to merge your current account : %@ with this account %@ ?", currentProfile, newProfile];
///
///    UIAlertController* alert = [UIAlertController alertControllerWithTitle:@"Account merge" message:message preferredStyle:UIAlertControllerStyleAlert];
///
///    UIAlertAction* account1 = [UIAlertAction actionWithTitle:@"Yes" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
///        UserProfile_OnMergeAction(MSDK_TRUE);// Send if the accounts have been merged to the MobileSDK
///        [alert dismissViewControllerAnimated:YES completion:nil];
///    }];
///
///    UIAlertAction* account2 = [UIAlertAction actionWithTitle:@"No" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
///        UserProfile_OnMergeAction(MSDK_FALSE);// Send if the accounts have been merged to the MobileSDK
///        [alert dismissViewControllerAnimated:YES completion:nil];
///    }];
///
///    [alert addAction:account1];
///    [alert addAction:account2];
///
///    UIViewController* mainView = [[UIApplication sharedApplication].keyWindow rootViewController];
///    [mainView presentViewController:alert animated:YES completion:nil];
///}
///
///void DisplayUnregisterAccount(msdk_ProfileInfo* p_currentProfile)
///{
///    //You've logged out from your account <span id="sns"></span>. You will now use the data associated to this device. To retreive your previous data, please log again with this account.
///
///    UIAlertController* alert = [UIAlertController alertControllerWithTitle:@"Account merge" message:@"You've logged out from your account. You will now use the data associated to this device. To retreive your previous data, please log again with this account." preferredStyle:UIAlertControllerStyleAlert];
///
///    UIAlertAction* ok = [UIAlertAction actionWithTitle:@"Ok" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
///        UserProfile_OnUnregisterAction();// Send that the user has read the popup to the MobileSDK
///        [alert dismissViewControllerAnimated:YES completion:nil];
///    }];
///
///    [alert addAction:ok];
///
///    UIViewController* mainView = [[UIApplication sharedApplication].keyWindow rootViewController];
///    [mainView presentViewController:alert animated:YES completion:nil];
///}
///
/// void InitUserprofile()
/// {
/// 	msdk_UserProfileWindowManager* windowManager = static_cast<msdk_UserProfileWindowManager*>(malloc(sizeof(msdk_UserProfileWindowManager)));
///    	windowManager->DisplayAccountSelection = DisplayAccountSelection;
///     windowManager->DisplayMergeAccounts = DisplayMergeAccounts;
///     windowManager->DisplayUnregisterAccount = DisplayUnregisterAccount;
///     UserProfile_RegisterWindowManager(windowManager);//Pass the window manager to the MobileSDK.
///		UserProfile_CallInit();//Init the UserProfile
/// }
///
/// @endcode
///
/// @page user_profile_data How to use the user profile to save data on the server
///
/// It is possible to save data in the UserProfile of the player. More information <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/display/oms/Game+Data">here</a>.
///
/// Before using the functions to save/get data you <strong>MUST</strong> register a callback that will be called when 2 accounts are merged.
/// Indeed, when merging 2 accounts the data might be different on these. Also when using several devices it may be different.
///
/// Since we don't know inside the MobileSDK which behaviour you want to have, we let the game do it itself.
/// You can do it be using the Userprofile_RegisterMergeCallback function.
///
/// Here is an example :
/// @code {.cpp}
/// // In this example we don't merge but return a dummy data
/// char_utf8* ProfileMergeCallback(char_utf8* saves)
///{
///    char_utf8* mergedData = NULL;
///    if(saves != NULL)
///    {
///        NSLog(@"Save to merge is : %s", saves);
///        mergedData = "{\"private\":[], \"public\":{\"key1\":\"123\", \"key2\":\"testValue\"}}";
///    }
///    return mergedData;
///}
///
/// void InitUserprofile()
/// {
///     UserProfile_RegisterMergeCallback(ProfileMergeCallback();
///		UserProfile_CallInit();//Init the UserProfile
/// }
///
/// @endcode
///
/// At the start of your game or during the resume of the application you should retrieve the data from the server.
/// To do so you should use Userprofile_CallGameData.
/// Once the request is ended you can use the following functions to modify the data :
/// <ul>
/// <li>UserProfile_GetGameData</li>
/// <li>UserProfile_DeleteGameData</li>
/// <li>UserProfile_UpdateGameData</li>
/// </ul>
/// You then need to call UserProfile_SaveGameData to persist the changes.
/// If connected to the internet the data will be uploaded to the server, otherwise it will be saved locally and it will automatically be uploaded the next time there is an internet connection.
///
/// @page user_profile_mailbox How to use the mailbox
///
/// As soon as you use the UserProfile you can access the mailbox service. For more information about the service click <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/display/oms/Mailbox">here</a>.
///
/// You can get your current emails by using the asynchronous function Mailbox_CallGet.
/// The content of the mails are not retrieved in this step. You need to call Mailbox_CallRead to get it.
///
/// Once you've read an email you can delete it by calling Mailbox_Delete.
///
/// To send an email you first need to create it using Mailbox_CreateMail. You can then add recipients by using Mailbox_AddRecipient.
/// When your email is well configured just call Mailbox_SendMail.
/// If there is a connection issue the email will be automatically resent on next resume of the application.
///
/// @page user_profile_game_session How to use the game sessions
///
/// For more information about the Game sessions please click <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/display/oms/Game-Se">here</a>
///
///
///
/// @page deviceInfo How to get informations about the device
/// @section DeviceInfo
/// MobileSDK provides functions to access device informations easily. The functions are located in the common/DeviceInfo.h header.
///
/// @code {.cpp}
/// // Gets the UDID of the device (use OpenUDID)
/// const char_utf8* udid = DeviceUID();
///
/// // Get the time in ms since the app has been launched
/// msdk_u32 elapsedTime = DeviceTime();
///
/// // Get the timezone of the device
/// const char_utf8* timezone = DeviceTimezone();
///
/// // Get the country of the device
/// const char_utf8* country = DeviceCountry();
///
/// // Get the GPS location of the device
/// msdk_LocationInfo gpsLocation = DeviceGPSLocation();
///
/// // Get the language of the device
/// const char_utf8* language = DeviceLanguage();
///
/// // Get the Mobile Network Code of the device
/// const char_utf8* mnc = DeviceMNC();
///
/// // Get the Mobile Country Code of the device
/// const char_utf8* mcc = DeviceMCC();
///
/// // Get the Public game data folder
/// const char_utf8* gameDataFolder = DevicePublicGameDataFolder();
///
/// // Get the Public user data folder
/// const char_utf8* userDataFolder = DevicePublicUserDataFolder();
///
/// // Get the Private user data folder
/// const char_utf8* userPrivateDataFolder = DevicePrivateUserDataFolder();
///
/// // Get the Public cache folder
/// const char_utf8* cacheFolder = DevicePublicCacheFolder();
///
/// // Get the Private cache folder
/// const char_utf8* privateCacheFolder = DevicePrivateCacheFolder();
/// @endcode
///
/// @page memManager How to use your own memory manager
/// @section MemManager
/// MobileSDK allows you to use your own Memory Manager to perform its allocations.
/// To do so you have to Call the function MemManager_RegisterAllocator located in common/MemManager.h
/// The argument of this function is a structure containing functions pointer to perform the allocations. This structure is located in common/MemInterface.h
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/common/MemInterface.h"
/// #include "MobileSDK/common/MemManager.h"
///
/// msdk_MemInterface* memInterface = (msdk_MemInterface*)my_malloc(sizeof(memInterface));
/// memInterface->Alloc = &my_malloc;
/// memInterface->Free = &my_free;
/// memInterface->Realloc = &my_realloc;
/// memInterface->name = "MyName";
///
/// MemManager_RegisterAllocator(memInterface);
/// @endcode
///
/// @page unactivateService How to unactivate a service
/// You can simply unactivate a service by using msdk_ActivateService function defiend in Init.h.
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/common/Init.h"
/// //here we unactivate Flurry
/// msdk_ActivateService(MSDK_FLURRY, MSDK_FALSE);
/// @endcode
///
/// @page customHouston How to send custom parameter trough requests to Houston
/// It is possible to add parameters to the requests made to Houston inside the MobileSDK.
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/common/Init.h"
/// msdk_AddHoustonParameter("key", "value");
/// @endcode
///
/// @page logging How to use your own logging sytem
/// @section Logging
/// MobileSDK allows you to use your own Logging system to perform.its logs
/// To do so you have to Call the function MemManager_RegisterAllocator located in common/Logging.h
/// The argument of this function is a structure containing functions pointer to perform the logs.
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// #include "MobileSDK/common/Logging.h"
///
/// msdk_LogInterface* logInterface = (msdk_LogInterface*)malloc(sizeof(msdk_LogInterface));
/// logInterface->Log = &myLogFunction;
/// logInterface->Logv = &myLogvFunction;
/// logInterface->SetMinimalLogLvl = &mySetMinimalLogLvlFunction;
///
/// Logging_RegisterLogger(logInterface);
/// @endcode
///
/// MobileSDK also allows you to use its own logging system.
///
/// <strong>SAMPLE :</strong>
/// @code {.cpp}
/// Common_Log(MSDK_LOG_VERBOSE, "Test message number %d", 1);
/// @endcode
///
// \defgroup mobileSDK_main mobileSDK
// @{

// @}

// global headers uses all through the sdk

#include <stdlib.h>
#include <vector>
#include <list>
#include <map>
#include <assert.h>
#include <string>
#include <stack>

#include <inttypes.h>

#endif //MOBILE_SDK_H
