///
/// DeviceInfo.h
///
///  Created on: 24 oct. 2012
///  Author: dmabin
///

#ifndef MOBILE_SDK_API_DEVICE_INFO_H
#define MOBILE_SDK_API_DEVICE_INFO_H

#include "MobileSDK/common/Utils.h"

const msdk_u32 MSDK_PATH_MAX_SIZE = 255;
const msdk_u32 MSDK_BAD_FILE_SIZE = (msdk_u32)0xFFFFFFFF;

///
/// Contains informations about location
///
typedef struct msdk_LocationInfo
{
	double latitude;
	double longitude;
	double horizontalAccuracy;
	double verticalAccuracy;
}msdk_LocationInfo;

#ifdef __cplusplus
extern "C"
{
#endif
    ///
    /// Get a unique ID for the device. This is an old version, only use it for retrocompatibility purposes
    /// @return A unique UID for the device
    /// @deprecated
    ///
    MSDK_EXPORT const char_utf8*        DeviceLegacyUID();

	///
	/// Get a unique ID that represent the device
	/// @return A unique UID for the device
	///
        MSDK_EXPORT const char_utf8*        DeviceUID();

	///
	/// Get the time in ms since the app has been launched
	/// @return The time in ms since some unspecified starting point.
	///
        MSDK_EXPORT msdk_u32         		DeviceTime();

	///
	/// Get the number of MS since 1st january 1970 (UTC)
	/// @return The time in ms since some unspecified starting point.
	///
		MSDK_EXPORT msdk_u64				DeviceTimeEpoch();

	///
	/// Get the date and the time on the user clock (including the timezone of the user)
    /// WARNING: Don't use that for time rewards (rewards received every 20 minutes, every days, etc) because the user can change that value by changing the time on the device
	/// @return Return a string having that format: YYYY-MM-DD HH:MM:SS (Years Months Days Hours Minutes Seconds)
	///
        MSDK_EXPORT const char_utf8*   		DeviceDateTime();

	///
	/// Get the timezone of the device
	/// @return The timezone of the device
	///
        MSDK_EXPORT const char_utf8*        DeviceTimezone();

	///
	/// Get the country of the device
	/// @return The country of the device
	///
        MSDK_EXPORT const char_utf8*        DeviceCountry();

	///
	/// Get the GPS location fo the device
	/// @return The GPS location
	///
        MSDK_EXPORT msdk_LocationInfo       DeviceGPSLocation();

	///
	/// Get the current language of the device
	/// @return The language of the device
	///
        MSDK_EXPORT const char_utf8*        DeviceLanguage();

	///
	/// Get the MNC(Mobile Network Code) of the device
	/// @return The MNC(Mobile Network Code) of the device
	///
        MSDK_EXPORT const char_utf8*        DeviceMNC();

	///
	/// Get the MCC(Mobile Country Code) of the device
	/// @return The MCC(Mobile Country Code) of the device
	///
        MSDK_EXPORT const char_utf8*        DeviceMCC();

	///
	/// Get the path of the public game data folder WITH "/" at the end (since it's a folder)
	/// @return Path of the public game data
	///
        MSDK_EXPORT const char_utf8*        DevicePublicGameDataFolder();

	///
	/// Get the path of the public user data folder WITH "/" at the end (since it's a folder)
	/// @return Path of the public user data
	///
        MSDK_EXPORT const char_utf8*        DevicePublicUserDataFolder();

	///
	/// Get the path of the private user data folder WITH "/" at the end (since it's a folder)
	/// @return Path of the private user data
	///
        MSDK_EXPORT const char_utf8*        DevicePrivateUserDataFolder();

	///
	/// Get the path of the public cache data folder WITH "/" at the end (since it's a folder)
	/// @return Path of the public cache data
	///
        MSDK_EXPORT const char_utf8*        DevicePublicCacheFolder();

	///
	/// Get the path of the private cache data folder WITH "/" at the end (since it's a folder)
	/// @return Path of the private cache data
	///
        MSDK_EXPORT const char_utf8*        DevicePrivateCacheFolder();

        MSDK_EXPORT const char_utf8*        DeviceUserAgent();

        MSDK_EXPORT const char_utf8*        DeviceGameVersion();

	#ifdef UBI_ANDROID

        MSDK_EXPORT msdk_s8        			DeviceIsTv();

    #endif

#ifdef __cplusplus
}
#endif

#endif // MOBILE_SDK_API_DEVICE_INFO_H
