//
// Created on: 13 sept. 2012
//

#ifndef MOBILE_SDK_API_INIT_H
#define MOBILE_SDK_API_INIT_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/common/Logging.h"

//#ifndef UNITY_SCRIPT
#ifdef UBI_ANDROID
	#include <jni.h>
#elif defined(UBI_WIN8) || defined(UBI_WIN8)
	#include <memory>
    #include <Windows.h>
#elif defined(UBI_IOS)
#import <UIKit/UIKit.h>
#endif
//#endif

///
/// Structure to pass to the msdk_Init function in order to initialize th MobileSDK
///
typedef struct msdk_InitializationFields
{
	msdk_LogLevel		minLogLevel;
	msdk_Mode			mode; // DEBUG or RELEASE
	const char_utf8*	sqliteKey;
#if defined(UBI_ANDROID)
	JavaVM* 			vm;
	jobject 			androidActivity;
#endif
}msdk_InitializationFields;


#ifdef __cplusplus
extern "C"
{
#endif
///
/// Set default value to a InitializationFields struct
/// @param p_initFields The fields to initialize
///
MSDK_EXPORT void		InitializationFields_Init(msdk_InitializationFields* const p_initFields);

///
/// Initialize the mobile SDK. Must call this function before any other one
/// @param p_initFields Parameters of the initialization
///
MSDK_EXPORT void 		msdk_Init(const msdk_InitializationFields* const p_initField);

    ///
    /// Returns the status of the init. It ends when we've finished to update the local database with the values from Houston.
    /// Only useful in development to wait for the environment to be set to dev
    ///
    MSDK_EXPORT msdk_Status msdk_StatusInit();

///
/// Change the default name of the embeded sqlite database file (default is msdk)
/// Must be called priori to msdk_Init
/// <strong>MUST NOT BE USED ON PROJECTS ALREADY LIVE WITHOUT THIS FUNCTION</strong>
/// @param p_name Name of the file to use as local database
///
MSDK_EXPORT void        msdk_ChangeDbName(const char_utf8* p_name);

#ifndef UBI_ANDROID
///
///	To call when the application goes to background (it will clear the cache)
///
MSDK_EXPORT void		msdk_Pause();

///
///	To call when the application goes to foreground
///
MSDK_EXPORT void		msdk_Resume();
#else
#define msdk_Pause msdk_OnActivityStop
#define msdk_Resume msdk_OnActivityRestart
#endif

///
/// Unallocate the memory used by the mobile SDK. Must be called when the SDK is not used anymore.
///
MSDK_EXPORT void		msdk_Uninit();

    ///
    /// Activate or deactivate a service (Facebook, Flurry, ...). By default all servives are activated
    /// @param p_service The service to activate/deactivate
    /// @param p_activated Te state to put the service to (MSDK_FALSE = deactivated, MSDK_TRUE = activated)
    ///
MSDK_EXPORT void		msdk_ActivateService(msdk_Service p_service, msdk_s8 p_activated);


    ///
    /// Add a parameter to Houston requests
    /// @param p_key The key of the parameter
    /// @param p_value The value of the parameter
    ///
    MSDK_EXPORT void    msdk_AddHoustonParameter(const char_utf8* p_key, const char_utf8* p_value);

#ifdef UBI_ANDROID

///
/// Android only function to get the game activity (that was gave to msdk during the initialization call)
/// @return the android activity
///
MSDK_EXPORT jobject		msdk_GetGameActivity();

MSDK_EXPORT void		msdk_OnActivityCreate(jobject jbundle);
MSDK_EXPORT void		msdk_OnActivityResume();
MSDK_EXPORT void		msdk_OnActivityPause();
MSDK_EXPORT void		msdk_OnActivityStop();
MSDK_EXPORT void		msdk_OnActivityRestart();
MSDK_EXPORT void		msdk_OnActivityDestroy();
MSDK_EXPORT void		msdk_OnActivityNewIntent(jobject jintent);
MSDK_EXPORT void		msdk_OnActivitySaveInstanceState(jobject joutBundle);
MSDK_EXPORT void		msdk_OnActivityResult(msdk_s32 requestCode, msdk_s32 resultCode, jobject intentData);
#endif // UBI_ANDROID

#ifdef UBI_WP8
void		msdk_HandleOpenURL(const wchar_t* p_url);
#endif

#ifdef UBI_IOS
	msdk_s8		msdk_HandleOpenURL(UIApplication *application, NSURL *url, NSString * sourceApplication, id annotation);
#endif

#ifdef __cplusplus
}
#endif

#endif // MOBILE_SDK_API_INIT_H
