//
// Created on: 11 sept. 2012
//

#ifndef MOBILE_SDK_API_LOGGING_H
#define MOBILE_SDK_API_LOGGING_H

#include "MobileSDK/common/Utils.h"

///
/// Enum describing the caracteristic of a log.
///
typedef enum msdk_LogLevel
{
	MSDK_LOG_DBG = 0, 		/// < Removed in release mode. Sensible information that will not be displayed in the final executable logs.
	MSDK_LOG_VERBOSE, 		/// < Removed in release mode. Very detailed informations, may slow the final executable.
	MSDK_LOG_INFORMATION, 	/// < Kept in release mode. Should be displayed when everything goes well, but it's an important information for debuging (Ex: Connection succesfull, User logged succesfully, Save loaded properly, etc).
	MSDK_LOG_WARNING, 		/// < Kept in release mode. Should be displayed when there is an issue, but the case was manage. Some functionnalities may not be available but the application will not crash. (Ex: User entered wrong password, Can't reach server, ...).
	MSDK_LOG_ERR,			/// < Kept in release mode. Should be displayed only if the error imply that the application can't run properly afterwards. (Save corrupted, Available OpenGL version is too low, ...).
	MSDK_LOG_NO_LOG
} msdk_LogLevel;


typedef struct msdk_LogInterface
{
	void (*Log) (msdk_LogLevel p_lvl, const char*  p_Message, ...);
	void (*Log_v)	(msdk_LogLevel p_priority, const char* p_message, va_list p_ap);
	void (*SetMinimalLogLvl) (msdk_LogLevel p_lvl);
	void (*LogT) (const char* p_tag, msdk_LogLevel p_lvl, const char*  p_message, ...);

}msdk_LogInterface;

#ifdef __cplusplus
extern "C"
{
#endif

	///
	/// Register a LogInterface to use inside the MobileSDK
	/// @param p_interface The interface to use
	///
	void Logging_RegisterLogger(const msdk_LogInterface* const p_interface);

	///
	/// Log a message with a given priority
	/// @param p_priority The priority of the message (the message will be shown or not depending of the minimal log level)
	/// @param p_message The message to show
	/// @param p_ap The parameters of the message
	///
	extern void (*Common_Log_v)	(msdk_LogLevel p_priority, const char*  p_message, va_list p_ap);

	///
	/// Log a message with a given priority
	/// @param p_priority The priority of the message (the message will be shown or not depending of the minimal log level)
	/// @param p_message The message to show
	/// @param ... The parameters of the message
	///
	extern void (*Common_Log) 	(msdk_LogLevel p_lvl, const char*  p_message, ...);

	extern void (*Common_LogT) 	(const char* p_tag, msdk_LogLevel p_lvl, const char*  p_message, ...);
	///
	/// Set the minimal log level. The logs with a level superior of this will not be shown
	/// @param p_lvl The minimal log level
	///
	extern void (*Common_SetMinimalLogLvl) (msdk_LogLevel p_lvl);

#ifdef __cplusplus
}
#endif

#ifdef MSDK_BUILD_DEBUG
#define MSDK_DBG_LOG(tag, a, ...) Common_LogT(tag, a, __VA_ARGS__)
#else
#define MSDK_DBG_LOG(tag, a, ...)
#endif

#endif // MOBILE_SDK_API_LOGGING_H
