//
//  Created on: 6 sept. 2012
//

#ifndef MOBILE_SDK_API_UTILS_H
#define MOBILE_SDK_API_UTILS_H

#include "MobileSDK/MobileSDK.h"

extern const char* MOBILE_SDK_VERSION;

#define STRINGIFY(name) #name

#ifdef UBI_WIN8
#define MSDK_EXPORT __declspec(dllexport)
#else
#define MSDK_EXPORT
#endif

#if defined(UBI_WIN8) || defined(UBI_WP8)
#define snprintf _snprintf
#endif
// ==================================================================================
// Define for compatibility

// ==================================================================================
// Typedef and structure
//
typedef uint8_t 				msdk_u8;  // 0 -> +255
typedef uint16_t 				msdk_u16; // 0 -> +65,535
typedef uint32_t 				msdk_u32; // 0 -> +4,294,967,295
typedef uint64_t 				msdk_u64; // 0 -> +18,446,744,073,709,551,615
typedef int8_t 					msdk_s8;  // -128 -> +127
typedef int16_t 				msdk_s16; // -32,768 -> +32,767
typedef int32_t 				msdk_s32; // -2,147,483,648 -> +2,147,483,647
typedef int64_t 				msdk_s64; // -9,223,372,036,854,775,808 to +9,223,372,036,854,775,807

// typedef unsigned char			msdk_u8;  // 0 -> +255
// typedef unsigned short int		msdk_u16; // 0 -> +65,535
// typedef unsigned long			msdk_u32; // 0 -> +4,294,967,295
// typedef unsigned long long int	msdk_u64; // 0 -> +18,446,744,073,709,551,615
// typedef signed char				msdk_s8;  // -128 -> +127
// typedef signed short int		msdk_s16; // -32,768 -> +32,767
// typedef signed long				msdk_s32; // -2,147,483,648 -> +2,147,483,647
// typedef signed long long int	msdk_s64; // -9,223,372,036,854,775,808 to +9,223,372,036,854,775,807

#ifndef PRIu32	// android does not define inttypes for all arch
	#ifdef __LP64__
		#define __PRI_64_prefix	"l"
	#else
		#define __PRI_64_prefix	"ll"
	#endif
	// for printf family
	#define PRIu8	"u"
	#define PRIu16	"u"
	#define PRIu32	"u"
	#define PRIu64	__PRI_64_prefix"u"

	#define PRId8	"d"
	#define PRId16	"d"
	#define PRId32	"d"
	#define PRId64	__PRI_64_prefix"d"

	// for scanf family
	#define	SCNd8	"hhd"
	#define	SCNd16	"hd"
	#define	SCNd32	"d"
	#define	SCNd64	__PRI_64_prefix"d"

	#define	SCNu8	"hhu"
	#define	SCNu16	"hu"
	#define	SCNu32	"u"
	#define	SCNu64	__PRI_64_prefix"u"
#endif


typedef char				 	char_utf8;

// ==================================================================================
// Const values

// Boolean
const char MSDK_TRUE  = 1;
const char MSDK_FALSE = 0;

#if defined(UBI_IOS) || defined(UBI_ANDROID)
#define MSDK_PACKED(x) __attribute__((packed, aligned(x)))
#else
#define MSDK_PACKED(x)
#endif

///
///	Contains two strings representing a pair
///

typedef struct msdk_key_value
{
	const char_utf8* key;
	const char_utf8* value;
}MSDK_PACKED(4) msdk_key_value;

typedef struct msdk_key_value_s32
{
    const char_utf8*    key;
    msdk_s32            value;
}msdk_key_value_s32;

typedef struct msdk_key_value_u32
{
    const char_utf8*    key;
    msdk_u32            value;
}msdk_key_value_u32;

typedef struct msdk_key_value_float
{
    const char_utf8*    key;
    float               value;
}msdk_key_value_float;

msdk_key_value CreateKeyValue(const char_utf8*, const char_utf8*);

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_u8);
DECLARE_ARRAYS(msdk_key_value);
DECLARE_ARRAYS(msdk_key_value_s32);
DECLARE_ARRAYS(msdk_key_value_u32);
DECLARE_ARRAYS(msdk_key_value_float);

// ==================================================================================
// All the enums

///
/// \enum msdk_NetworkId
/// \brief List of available 3rd party sdk (Excluding IAP providers)
///
#define msdk_NetworkId msdk_Service
typedef enum msdk_Service
{
	MSDK_NOT_A_NETWORK,
	MSDK_FACEBOOK,  ///< iOS and Android
	MSDK_GAME_CENTER,  ///< iOS only
	MSDK_GAME_CIRCLE,  ///< Android only
	MSDK_GAME_SERVICES, ///< Android only

	MSDK_GOOGLE_ANALYTICS,  ///< iOS and Android
	MSDK_FLURRY, ///< iOS and Android
	MSDK_MAGNET, ///< Deprecated: Not functional, do not use
	MSDK_APPSPERSE, ///< iOS only

	MSDK_TAPJOY	, ///< iOS and Android: WIP, not functional
    MSDK_TRIALPAY,
	MSDK_XBOX_LIVE, ///< Only WP8 and W8.1
	MSDK_SINA_WEIBO, // iOS and Android

	MSDK_SERVER_NOTIFICATION,
	MSDK_LOCAL_NOTIFICATION,

    MSDK_DNA,
    MSDK_HOUSTON
}msdk_Service;

typedef enum msdk_Mode
{
	MSDK_RELEASE = 0, ///< Disable debug logs
	MSDK_DEBUG ///< Enable logs
}msdk_Mode;

typedef enum msdk_Gender
{
	MSDK_MALE = 0,
	MSDK_FEMALE,
	MSDK_UNDEFINED
}msdk_Gender;

// When you modify the list below please reflect the change in msdk_Result_Str in Utils.cpp
typedef enum msdk_Result
{
	MSDK_SUCCESS = 0,
	MSDK_USER_CANCEL,
	MSDK_USER_CLOSE,
	MSDK_NETWORK_ERROR,
	MSDK_BAD_PLIST_FILE, // iOS
	MSDK_NETWORK_NOT_CONNECTED,
	MSDK_NETWORK_NOT_FOUND,
	MSDK_NOT_AUTHORIZED,
	MSDK_BAD_PARAMETER,
	MSDK_USER_UNACTIVATED,
	MSDK_UNKNOW_ERROR,
	MSDK_SAMSUNG_IAP_NOT_INSTALLED, // Android
	MSDK_SAMSUNG_IAP_NEED_UPDATE, // Android
	MSDK_MISSING_HOUSTON_PRODUCT_KEY,
	MSDK_LIKED, // For Wall_IsLiked request
	MSDK_NOT_LIKED, // For Wall_IsLiked request
	MSDK_NOT_INITIALIZED,
    MSDK_TIMEOUT,
    MSDK_REQUEST_NOT_ENDED,
	MSDK_XBL_XBL_UPDATE_REQUIRED, // For XBoxLive, it's a fail (XBL will NOT work) the user should/must update the game.
	MSDK_XBL_APP_UPDATE_REQUIRED, // For XBoxLive, it's a success (XBL will work in offline mode) but the user should/must update the game.
    MSDK_SUCCESS_OFFLINE, // used for dynamic store when no internet connection
	MSDK_GGP_RESULT_BILLING_UNAVAILABLE, // Android
    MSDK_AD_UNAVAILABLE,
    MSDK_FB_APP_NEEDED
}msdk_Result;

typedef enum msdk_Status
{
	MSDK_IN_PROGRESS = 0x1,
	MSDK_ENDED,
	MSDK_ID_OUT_OF_RANGE,
	MSDK_NOT_ALLOCATED,
}msdk_Status;

typedef struct msdk_FileContent
{
	const char_utf8*	content;
	msdk_u32			size;
}msdk_FileContent;

// ==================================================================================
// Functions

///
/// Helper function to convert a ::msdk_NetworkId into a string (Useful for debug logs)
/// @param p_status a value of ::msdk_NetworkId
/// @return a string equal to the value gave in parameter
const char_utf8* const msdk_NetworkId_string	(msdk_NetworkId p_networkId);

///
/// Helper function to convert a ::msdk_Service into a string (Useful for debug logs)
/// @param p_status a value of ::msdk_Service
/// @return a string equal to the value gave in parameter
const char_utf8* const msdk_Service_string	(msdk_Service p_service);

///
/// Helper function to convert a ::msdk_Result into a string (Useful for debug logs)
/// @param p_status a value of ::msdk_Result
/// @return a string equal to the value gave in parameter
///
const char_utf8* const msdk_Result_string	(msdk_Result p_status);

///
/// Helper function to convert a ::msdk_Status into a string (Useful for debug logs)
/// @param p_status a value of::msdk_Status
/// @return a string equal to the value gave in parameter
///
const char_utf8* const msdk_Status_string	(msdk_Status p_status);

///
/// Returns the current version of the MobileSDK
/// @return The version of the MobileSDK
///
const char_utf8* msdk_Version();

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(char_utf8)

#endif //MOBILE_SDK_API_UTILS_H
