#ifndef MOBILESDKAPI_GAME_PREFERENCES_H_
#define MOBILESDKAPI_GAME_PREFERENCES_H_

#include "MobileSDK/common/Utils.h"

#ifdef __cplusplus
extern "C"
{
#endif

	///
	///	Gets the value mapped to the key from the DB as a string
	/// @param p_key The key
	/// @param p_default It will be returned if the key is not found. Could be NULL
	/// @result The value as a string
	///
	const char_utf8*	GamePreferences_GetString(const char_utf8* p_key, const char_utf8* p_default);
	
	///
	///	Gets the value mapped to the key from the DB as an integer
	/// @param p_key The key
	/// @param p_default It will be returned if the key is not found. Could be 0
	/// @result The value as an integer
	///
	msdk_s32			GamePreferences_GetInt(const char_utf8* p_key, msdk_s32 p_default);
	
	///
	///	Gets the value mapped to the key from the DB as a float
	/// @param p_key The key
	/// @param p_default It will be returned if the key is not found. Could be 0
	/// @result The value as a float
	///
	float				GamePreferences_GetFloat(const char_utf8* p_key, float p_default);
	
	///
	/// Add or update the value correponding to a key in the DB as a string
	/// @param p_key The key
	/// @param p_value The value as a string
	///
	void				GamePreferences_UpdateString(const char_utf8* p_key, const char_utf8* p_value);
	
	///
	/// Add or update the value correponding to a key in the DB as an integer
	/// @param p_key The key
	/// @param p_value The value as an integer
	///
	void				GamePreferences_UpdateInt(const char_utf8* p_key, msdk_s32 p_value);
	
	///
	/// Add or update the value correponding to a key in the DB as a float
	/// @param p_key The key
	/// @param p_value The value as a float
	///
	void				GamePreferences_UpdateFloat(const char_utf8* p_key, float p_value);
	
	///
	/// Remove a key from the database
	/// @param p_key The key to remove
	///
	void				GamePreferences_DeleteKey(const char_utf8* p_key);
	
	///
	/// Save all changes made into the database
	///
	void				GamePreferences_Flush();

#ifdef __cplusplus
}
#endif

#endif //MOBILESDKAPI_GAME_PREFERENCES_H_
