#ifndef GETTING_STARTED_ANDROID
#define GETTING_STARTED_ANDROID

/// @page getting_started_android android
/// @section title_android_prerequisites Prerequisites
/// <ul>
/// <li>Download the MSDK package <a href="http://www.r-mob.ubi.com/?a=latest-msdk" target="_blank">here</a>.</li>
/// <li>Unzip the package. You should have:
/// <pre>MobileSDKv1.XX |
///                |-misc |
///                |      |-icon.pmg
///                |      |-wp8
///                |-UnityBinding
///                |-MobileSDK |
///                            |-ios
///                            |-win8
///                            |-wp8
///                            |-android |
///                                      |-amazon-gamecircle-sdk
///                                      |-facebook-android-sdk
///                                      |-googleplay-game-services
///                                      |-libcurl
///                                      |-msdk
///                                      |-msdk_gnustl</pre>
/// </li>
/// </ul>
/// We will use all the content of the android folder in that introduction.
/// Plus you'll need to get your msdk database file. It's a file litteraly called "msdk".
/// This file contain all the configuration of your project (later you'll use flurry ids, facebok id, items for the iap etc).
/// All these informations has to be setup on <a href="https://ubimobile-admin.ubisoft.org/ubimobiadm_dev" target="_blank">Houston website</a> and from there you'll generate your msdk file.
/// Houston documentation is available here:
/// <ul>
/// <li>Setup your ids: <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925175" target="_blank">doc confluence</a></li>
/// <li>Generate msdk file: <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925219" target="_blank">doc confluence</a></li>
/// </ul>
/// Once ready you'll have these 3 elements: the msdk package, the msdk file and your project.
/// @image html get_all_elem.png
/// @section title_android_eclipse_setup Eclipse setup
/// <ol>
/// <li>First setup will be to link all the project in your eclipse project. In Eclipse create a new project and select "Android Project from existing code" as in the screenshot below.
/// <br>@image html android_getting_started/import_projects_0.png</li>
/// <li>Select the folder called "android" inside the msdk package at that place: MobileSDKv1.XX/MobileSDK/android
/// <br>@image html android_getting_started/import_projects_1.png</li>
/// <li>Right click on your project and select "Properties". In that screen on the left panel select "Android". Click on the button "Add" as in the screenshot below:
/// <br>@image html android_getting_started/import_projects_2.png</li>
/// <li>Add the project FacebookSDK and repeat the operation to add all the other projects: GameCircleSDK, googleplay-game-services, Libcurl and msdk.
/// <br>@image html android_getting_started/import_projects_3.png</li>
/// <li>Now that the jave project are linked, we will link the native part of msdk to your project. In your "Android.mk" file you'll need to add the lines:
/// </li>
/// <li>Open or create the "jni" folder, inside open or create the file "Application.mk". Add the line "APP_STL := stlport_static" like on the screenshot below:
/// <br>@image html android_eclipse_1.png</li>
/// <li>In your file Android.mk, please add the lines below:
/// <ul><li>Between: "include $(CLEAR_VARS)" and : "include $(BUILD_SHARED_LIBRARY)":
/// <ul style="list-style:none"><li>LOCAL_CFLAGS := -Wno-psabi</li>
/// <li>LOCAL_STATIC_LIBRARIES := mobilesdk libcurl</li>
/// <li>LOCAL_LDLIBS := -llog</li></ul>
/// </li>
/// <li>After the line : include $(BUILD_SHARED_LIBRARY):
/// <ul style="list-style:none"><li>$(call import-add-path, $(realpath ../RELATIVE/PATH/TO/MSDK/PROJECTS))</li>
/// <li>$(call import-module,msdk)</li>
/// <li>$(call import-module,libcurl)</li></ul>
/// </li>
/// </ul>
/// <br>@image html android_getting_started/link_native_1.png</li>
/// </ol>
/// @section title_android_manifest Manifest modification
/// <b>Inside</b> the mark <manifest> you should add an attribut for amazon game circle:
/// <b>xmlns:amazon="http://schemas.amazon.com/apk/res/android"</b>
/// <br/>It's very similar to the one already present in that mark but it's still necessary for game circle integration.
/// <b>Outside</b> <application></application> mark, please add these lines:
/// <div style=" margin-left:5em;"><pre><code class="xml">&lt;!-- Permission to access internet used for MobileSDK Social module (access facebook, scoreloop, etc) -->
/// &lt;uses-permission android:name="android.permission.INTERNET"/>
/// &lt;!-- Google analytics -->
/// &lt;uses-permission android:name="android.permission.ACCESS_NETWORK_STATE" />
/// &lt;!-- GooglePlay In App Billing -->
/// &lt;uses-permission android:name="com.android.vending.BILLING" />
/// &lt;uses-permission android:name="android.permission.READ_PHONE_STATE"/>
/// &lt;uses-permission android:name="android.permission.ACCESS_WIFI_STATE" />
/// &lt;uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ===================================== Google GCM ===================================== -->
/// &lt;!-- Optional: only if you want to be able to vibrate when you receive a push notification-->
/// &lt;uses-permission android:name="android.permission.VIBRATE" />
/// &lt;!-- Only necessary for android version priori to 4.0.4 (api lvl 15) -->
/// &lt;uses-permission android:name="android.permission.GET_ACCOUNTS" />
/// &lt;!-- Compulsory -->
/// &lt;uses-permission android:name="com.google.android.c2dm.permission.RECEIVE" />
/// &lt;permission android:name="MY.PACKAGE.NAME.permission.C2D_MESSAGE" android:protectionLevel="signature" />
/// &lt;uses-permission android:name="MY.PACKAGE.NAME.permission.C2D_MESSAGE" />
/// &lt;!-- ================================================================================ -->
/// &lt;!-- ============================= GooglePlay In App Billing ============================== -->
/// &lt;uses-permission android:name="com.android.vending.BILLING" />
/// &lt;!-- ================================================================================= -->
/// &lt;!-- ============================= Samsung In App Billing ================================= -->
/// &lt;uses-permission android:name="com.sec.android.iap.permission.BILLING" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ==================================== Amazon PN ======================================= -->
/// &lt;permission android:name="ubi.mobilesdk.SampleGame.permission.RECEIVE_ADM_MESSAGE" android:protectionLevel="signature" />
/// &lt;uses-permission android:name="ubi.mobilesdk.SampleGame.permission.RECEIVE_ADM_MESSAGE" />
/// &lt;uses-permission android:name="com.amazon.device.messaging.permission.RECEIVE" />
/// &lt;uses-permission android:name="android.permission.WAKE_LOCK" />
/// &lt;!-- ================================================================================= -->
/// </code></pre></div>
/// <b>Inside</b> <application></application> mark you must add these declarations:
/// <div style=" margin-left:5em;"><pre><code class="xml">&lt;!-- ====================================================================================== -->
/// &lt;!-- ====================================== OpenUDID ====================================== -->
/// &lt;service android:name="org.openudid.OpenUDID_service">
/// &lt;intent-filter>
/// &lt;action android:name="org.openudid.GETUDID" />
/// &lt;/intent-filter>
/// &lt;/service>
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- =================================== Facebook ========================================= -->
/// &lt;activity android:name="com.facebook.LoginActivity" />
/// &lt;meta-data android:name="com.facebook.sdk.ApplicationId" android:value="@string/fb_app_id"/>
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ================================== Game Circle ======================================= -->
/// &lt;activity android:name="com.amazon.ags.html5.overlay.GameCircleUserInterface" android:theme="@style/GCOverlay"></activity>
/// &lt;activity
///   android:name="com.amazon.identity.auth.device.authorization.AuthorizationActivity"
///   android:theme="@android:style/Theme.NoDisplay"
///   android:allowTaskReparenting="true"
///   android:launchMode="singleTask">
///   &lt;intent-filter>
///     &lt;action android:name="android.intent.action.VIEW" />
///     &lt;category android:name="android.intent.category.DEFAULT" />
///     &lt;category android:name="android.intent.category.BROWSABLE" />
///     &lt;data android:host="ubi.mobilesdk.SampleGame" android:scheme="amzn" />
///   &lt;/intent-filter>
/// &lt;/activity>
/// &lt;receiver
///   android:name="com.amazon.identity.auth.device.authorization.PackageIntentReceiver"
///   android:enabled="true" >
///   &lt;intent-filter>
///     &lt;action android:name="android.intent.action.PACKAGE_INSTALL" />
///     &lt;action android:name="android.intent.action.PACKAGE_ADDED" />
///     &lt;data android:scheme="package" />
///   &lt;/intent-filter>
/// &lt;/receiver>
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ================================== Amazon IAB ======================================== -->
/// &lt;receiver android:name="com.amazon.device.iap.ResponseReceiver">
/// &lt;intent-filter>
/// &lt;action android:name="com.amazon.inapp.purchasing.NOTIFY" android:permission="com.amazon.inapp.purchasing.Permission.NOTIFY" />
/// &lt;/intent-filter>
/// &lt;/receiver>
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- =================================== Game Services ==================================== -->
/// &lt;meta-data android:name="com.google.android.gms.games.APP_ID" android:value="@string/gs_app_id" />
/// &lt;meta-data android:name="com.google.android.gms.version" android:value="@integer/google_play_services_version" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ====================== Google Analytics Campaign tracking ============================ -->
/// &lt;service android:name="com.google.analytics.tracking.android.CampaignTrackingService" />
/// &lt;receiver android:name="com.google.analytics.tracking.android.CampaignTrackingReceiver" android:exported="true">
///   &lt;intent-filter>
///     &lt;action android:name="com.android.vending.INSTALL_REFERRER" />
///   &lt;/intent-filter>
/// &lt;/receiver>
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ==================================== FLURRY ========================================== -->
/// &lt;activity android:name="com.flurry.android.FlurryFullscreenTakeoverActivity"
///                 android:configChanges="keyboard|keyboardHidden|orientation|screenLayout|uiMode" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ==================================== TAPJOY ========================================== -->
/// &lt;activity android:name="com.tapjoy.TJCOffersWebView" android:configChanges="orientation|keyboardHidden" />
/// &lt;activity android:name="com.tapjoy.TapjoyFullScreenAdWebView" android:configChanges="orientation|keyboardHidden" />
/// &lt;activity android:name="com.tapjoy.TapjoyDailyRewardAdWebView" android:configChanges="orientation|keyboardHidden" />
/// &lt;activity android:name="com.tapjoy.TapjoyVideoView" android:configChanges="orientation|keyboardHidden" />
/// &lt;activity android:name="com.tapjoy.TJAdUnitView" android:configChanges="orientation|keyboardHidden" android:theme="@android:style/Theme.Translucent.NoTitleBar.Fullscreen" />
/// &lt;activity android:name="com.tapjoy.mraid.view.ActionHandler" android:configChanges="orientation|keyboardHidden" />
/// &lt;activity android:name="com.tapjoy.mraid.view.Browser" android:configChanges="orientation|keyboardHidden" />
/// &lt;!-- <meta-data android:name="tapjoy.disable_video_offers" android:value="true" /> -->
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ================================== TrialPay ========================================== -->
/// &lt;activity android:name="ubisoft.mobile.mobileSDK.ads.trialpay.OfferwallActivity" droid:configChanges="orientation|keyboardHidden" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ==================================== IAP ============================================= -->
/// &lt;receiver android:name="com.amazon.inapp.purchasing.ResponseReceiver">
///   &lt;intent-filter>
///     &lt;action android:name="com.amazon.inapp.purchasing.NOTIFY"
///                android:permission="com.amazon.inapp.purchasing.Permission.NOTIFY" />
///   &lt;/intent-filter>
/// &lt;/receiver>
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ================================== Local PN ========================================== -->
/// &lt;receiver android:name="ubisoft.mobile.mobileSDK.localPN.LocalPnReceiver" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ================================= Google GCM ========================================= -->
/// &lt;receiver
///         android:name="ubisoft.mobile.mobileSDK.social.communication.serverPN.MsdkGcmReceiver"
///         android:permission="com.google.android.c2dm.permission.SEND" >
///   &lt;intent-filter>
///     &lt;action android:name="com.google.android.c2dm.intent.RECEIVE" />
///     &lt;category android:name="MY.PACKAGE.NAME" />
///   &lt;/intent-filter>
/// &lt;/receiver>
/// &lt;service android:name=".GcmIntentService" />
/// &lt;!-- ====================================================================================== -->
/// &lt;!-- ===================================== Amazon PN ====================================== -->
/// &lt;amazon:enable-feature android:name="com.amazon.device.messaging" android:required="false" />
/// &lt;service android:name="ubisoft.mobile.mobileSDK.social.communication.serverPN.MsdkAmzHandler" android:exported="false" />
/// &lt;receiver android:name="ubisoft.mobile.mobileSDK.social.communication.serverPN.MsdkAmzReceiver" android:permission="com.amazon.device.messaging.permission.SEND">
/// 	&lt;intent-filter>
/// 		&lt;action android:name="com.amazon.device.messaging.intent.REGISTRATION" />
/// 		&lt;action android:name="com.amazon.device.messaging.intent.RECEIVE" />
/// 		&lt;category android:name="ubi.mobilesdk.SampleGame" />
/// 	&lt;/intent-filter>
/// &lt;/receiver>
/// &lt;!-- ====================================================================================== --></code></pre></div>
/// <br/>In your Android ressources folder ("res" folder) you'll need to add in the file "res/values/strings.xml" 2 value:
/// <div style=" margin-left:5em;"><pre><code class="xml">&lt;string name="gs_app_id">YOUR_GAME_SERVICES_ID</string>
/// &lt;string name="fb_app_id">YOUR_FACEBOOK_APP_ID</string></code></pre></div>
/// @section title_android_activity Activity modification
/// For IAP and some social networks you need to override the onActivityCallback, and warn msdk when the application is put in bacground and come back from background. In your main activity you must call a mobile sdk function in 3 functions: onActivityResult onRestart and onStop. Here is the sample code to put in your main activity in Java:
/// <div style=" margin-left:5em;"><pre><code class="java">\@Override
/// public void onActivityResult(int requestCode, int resultCode, Intent data)
/// {
///     super.onActivityResult(requestCode, resultCode, data);
///     Log.d(TAG, "onActivityResult called");
///     // The line below called a function defined in C++ by your game (not by msdk)
///     CPPLib.onActivityResult(requestCode, resultCode, data);
/// }
/// \@Override
/// public void onRestart()
/// {
/// 	// The line below called a function defined in C++ by your game (not by msdk)
///     CPPLib.onRestart();
/// }
/// \@Override
/// public void onStop()
/// {
/// 	// The line below called a function defined in C++ by your game (not by msdk)
///     CPPLib.onStop();
/// }</code></pre></div>
/// On <b>C++ side</b>, in the functions defined in your project, you should then warn msdk by calling these functions:
/// @code {.cpp}
/// void onActivityResult(JNIEnv* env, jobject obj, jint p_requestCode, jint p_resultCode, jobject p_data)
/// {
/// 	msdk_OnActivityResult(p_requestCode, p_resultCode, p_data);
/// }
/// void onStop(JNIEnv* env, jobject obj)
/// {
/// 	msdk_Pause();
/// }
/// void onRestart(JNIEnv* env, jobject obj)
/// {
/// 	msdk_Resume();
/// }
/// @endcode
/// @section title_android_init MSDK Initialisation
/// MSDK require a pointer to your JavaVM and to your main game activity during the initialisation. To get the pointer of the JavaVM you'll need to implement the <b>JNI_OnLoad</b> function. You can find more information about that function <a href="http://developer.android.com/guide/practices/jni.html#native_libraries" target="_blank">here</a>.
/// You'll just need to keep the pointer available as first parameter of that function and give it during msdk init call.
/// You'll find below a sample implemetation of msdk init with JNI_OnLoad implementation.
/// Finally to get your main activity pointer you'll just need to send it through JNI to your C++ code and make it a <b>GlobalRef</b> (more information <a haref="http://developer.android.com/training/articles/perf-jni.html#local_and_global_references" target="_blank">here</a>).
/// Once you gathered both pointer you'll just need to call:
/// <div style=" margin-left:5em;">
/// @code {.cpp}
/// #include "MobileSDK/common/Init.h"
/// msdk_InitializationFields initFields;
/// InitializationFields_Init(&initFields);
/// initFields.mode = MSDK_DEBUG;
/// initFields.minLogLevel = MSDK_LOG_VERBOSE;
/// initFields.androidActivity = s_androidActivity; // Your main activity pointer
/// initFields.vm = s_javaVM; // You JavaVM pointer
/// initFields.sqliteKey = "mySqliteKey"; // You find it on Houston in the same place you download the msdk file
/// msdk_Init(&initFields);
/// @endcode
/// @image html android_getting_started/init_msdk.png
/// </div>
/// @section title_android_assets Assets file
/// Finally you'll need to add in your <b>assets</b> folder 2 files:
/// <ul>
/// <li>You <b>msdk</b> file</li>
/// <li>The <b>icon.pmg</b> file that can be found in msdk package under the folder "MobileSDKv1.XX/misc/" </li>
/// </ul>


#endif
