#ifndef GETTING_STARTED_IOS
#define GETTING_STARTED_IOS

/// @page getting_started_ios ios
/// @section title_ios_link Include the MobileSDK in your iOS project
/// This page contains informations about how to include the MobileSDK in your iOS project
/// @subsection prerequisites Prerequisites
/// Firstofall you'll need the MobileSDK.zip package. This package should contain :<br/>
/// <ul>
/// 	<li>An iOS framework named MobileSDK</li>
///	<li>A libraries folder wich should contain the flurry and google analytics libraries and the FacebookSDK</li>
/// </ul>
/// You should define the preprocessor macro : UBI_IOS
///
/// @subsection xcode Using Xcode
/// <h2>Requierements</h2>
/// If you are using Xcode you can :
/// <ul>
/// 	<li>Drag and drop the MobileSDK.framework folder in your solution</li>
/// </ul>
/// or
/// <ul>
/// 	<li>Add the MobileSDK framework in the "Link Binary With Libraries" section of the "Build phases" of your project</li>
/// </ul>
/// The MobileSDK is using third party libraries.
/// The same way you have added the MobileSDK in your project you need to add :
/// <ul>
/// 	<li>libFlurry_5.3.0.a (can be found in the libraries folder of the package)</li>
/// 	<li>libGoogleAnalytics.a (can be found in the libraries folder of the package)</li>
///		<li>libFlurryAds_5.3.0.a</li>
/// 	<li>libcurl.a If you use your own be aware that it must be built <strong>with SSL support</strong>). Otherwise it can be found here : http://pdc-mob-hwh01.ubisoft.org/buildServer/MobileAuth/doc/index.html </li>
///		<li>libcrypto.a</li>
///		<li>libssl.a</li>
///		<li>CoreData.framework</li>
/// 	<li>SystemConfiguration.framework</li>
/// 	<li>CoreTelephony.framework</li>
/// 	<li>UIKit.framework</li>
/// 	<li>Foundation.framework</li>
/// 	<li>CFNetwork.framework</li>
/// 	<li>MobileCoreServices.framework</li>
///		<li>Security.framework</li>
/// 	<li>FacebookSDK.framework (can be found in the libraries folder of the package)</li>
/// 	<li>Tapjoy.framework (can be found in the libraries folder of the package)</li>
/// 	<li>MobileAuth.framework (can be found at http://pdc-mob-hwh01.ubisoft.org/buildServer/MobileAuth/doc/index.html)</li>
///	<li>libWeiboSDK.a</li>
/// 	<li>GameKit.framework</li>
/// 	<li>StoreKit.framework</li>
/// 	<li>CoreLocation.framework</li>
/// 	<li>MediaPlayer.framework</li>
/// 	<li>EventKit.framework</li>
/// 	<li>Accounts.framework(weak link)</li>
/// 	<li>Social.framework(weak link)</li>
/// 	<li>AdSupport.framework (weak link)</li>
/// 	<li>libxml2.dylib</li>
/// 	<li>libz.dylib</li>
/// 	<li>libsqlite3.0.dylib</li>
/// </ul>
///
/// If you are using Weibo you need to add WeiboSDK.bundle in the Copy Bundle Resources section.
///
/// If you are using Tapjoy you need to add TapjoyResources.bundle in the Copy Bundle Resources section.
///
/// For the AdSupport, Accounts and Social frameworks you have to weak link with it. Set this frameworks as optional in the "Link Binary With Libraries Section" (default is set to "Required")
/// <br/>
///
/// @subsection commandline Using the command line
/// The libraries and frameworks to link against are the same as mentionned above. To link against and iOS framework you need to specified the FrameWork search path with the -F option. Then to link against you need to use the -framework option.<br/>
/// For instance to link against the FacebookSDK framework you need to add : -F libraries -framework FacebookSDK<br/><br/>
/// For the system frameworks only available on the latest ios version (Social, AdSupport, Accounts) you should use the -weak_framework option to link against if you want to build for an older version of the ios SDK.
/// For instance : -weak_framework AdSupport
///
/// @subsection Include
/// To include a part of the SDK in you project you just have to prefix the path to the header by MobileSDK/
/// @code {.cpp}
/// #include "MobileSDK/common/Init.h"
/// @endcode
///
/// @section title_msdk_file Include the "msdk" and "icon.pmg" files
/// MSDK need to have your Facebook ID, Flurry ID, and other information needed for each features. To setup that:
/// <ul>
/// <li>Setup on Houston your 3rd party ids on Houston. Doc <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925175" target="_blank">here</a></li>
/// <li>Generate on Houston the "msdk" file. Doc <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925219" target="_blank">here</a></li>
/// <li>Include in XCode the "msdk" file: Select your project then "Build Phases", here you should drag and drop the msdk file in the "Copy Bundle Resources" section</li>
/// <li>To perform https request we need some certificates included in the icon.pmg file</li>
/// In the end you should have something like that:
/// @image html copy_bundle_resources.png
///
/// @section title_msdk_file Activate the keychain and add the com.ubisoft.data keychain group
/// This is needed to generate a Deviice Unique identifier that will last even after the deletion of the application.
/// Here com.ubisoft.test.pushapp is our bundleId. Do not use it.
/// @image html keychain.png
///
/// @section title_init Initialize the SDK
/// The SDK need to be initialize before any use of it. To do so you just need to call the @ref  msdk_Init function.
/// This function take a reference to an InitializationFields structure.
/// The fields to initialize in this structure are :
/// <ul>
///	<li>mode MSDK_RELEASE or MSDK_DEBUG (add extras logs)</li>
///	<li>minLogLevel The minimal log level you want to use</li>
///	<li>sqliteKey The encryption key of the MSDK. More information <a href="https://mdc-web-tomcat17.ubisoft.org/confluence/pages/viewpage.action?pageId=232925219" target="_blank">here</a></li>
/// </ul>
/// For Android you also need to provide a pointer to the Java virtual machine (vm field) and the android activity (androidActivity field)
///    @code{.cpp}
///    #include "MobileSDK/common/Init.h"
///
///    msdk_InitializationFields initFields;
///    InitializationFields_Init(&initFields);
///    initFields.mode = MSDK_DEBUG;
///    initFields.minLogLevel = MSDK_LOG_VERBOSE;
///    initFields.sqliteKey = "mySqliteKey";
///    msdk_Init(&initFields);
///    @endcode
///
/// @section title_ios_openUrl Return to your ios application once connected to Facebook
/// When you try to connect the user to Facebook through the @ref SocialConnection_CallConnectFacebook function your application will go to background and Safari will be bring to the foreground.
/// Once your user has validated or cancelled the connection to Facebook you need to call a function so the SDK will know the answer. To get the answer from Safari you need to listen to the iOS openURL event and pass its result to the  @ref SocialConnection_HandleOpenURL function.
///
/// <strong>SAMPLE</strong> :
/// @code {.mm}
/// - (BOOL)application:(UIApplication *)application
///             openURL:(NSURL *)url
///   sourceApplication:(NSString *)sourceApplication
///          annotation:(id)annoation {
///	NSString* str = [url absoluteString];
///	return SocialConnection_HandleOpenURL([str UTF8String]);
///	}
/// @endcode
///
/// @section title_ios_facebook_settings Requirements to use Facebook in your application
/// In order to be able to come back to your application after the user has validate or cancelled is login with Facebook you need to add some fields in the Info.plist file of your application. You need to add an URL Scheme.
/// Here is an example of URL Scheme :
/// @image html urltypes.png
/// The URL Schemes field contains the callback adress; This one is composed by the string "fb" followed by the facebook id of your application. The URL identifier field contains the name of your bundle.
///
/// You also need to add a field named FacebookAppID with the value of the id int the plist file.

#endif

