//
//  IAPProduct.h
//  InAppPurchaseManagerTest
//
//  Created by Andrei Andreiana, Emanuel Jarnea on 9/19/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#ifndef MOBILESDKAPI_IAP_MANAGER_H_
#define MOBILESDKAPI_IAP_MANAGER_H_

#include "MobileSDK/common/Array.inl"
#include "MobileSDK/common/Utils.h"

///
/// Enum that describes types of products.
/// @param  ManagedConsumable   Consumable product that the vendor supplies ownership information
/// @param  Consumable          Product which can be bought once.
/// @param  Subscription        Consumable product automatically purchased every week/month (depending on vendor).
///
enum msdk_ProductType
{
    MSDK_PT_MANAGED,
    MSDK_PT_CONSUMABLE,
    MSDK_PT_SUBSCRIPTION,
};

///
/// Enum to the describe the product status of a product for a client (user).
/// @param  Available           Available to be purchased.
/// @param  Locked              Product can be purchased after the game logic unlocks it.
/// @param  Owned               The user owns the product.
/// @param  OwnedSubscription   The user owns a subcription to this product.
///
enum msdk_ProductStatus
{
    MSDK_PS_UNDEFINED,
    MSDK_PS_AVAILABLE,
    MSDK_PS_LOCKED,
    MSDK_PS_OWNED,
    MSDK_PS_OWNED_SUBSCRIPTION,
    MSDK_PS_DELETED,
};

///
/// Informations of an IAP product
///
typedef struct msdk_IAPProduct
{
    const char_utf8*	productIdentifier;
    const char_utf8*	localizedTitle;
    const char_utf8*	localizedDescription;
    float		price;
    const char_utf8*	localCurrency;
    const char_utf8*	formattedPrice;
    msdk_ProductType	productType;
    msdk_ProductStatus	productStatus;
    const char_utf8*	ubiTransactionId;
} msdk_IAPProduct;

DECLARE_ARRAYS(msdk_IAPProduct)

extern "C"
{
    
void IAPProduct_Init(msdk_IAPProduct*);

void IAPProduct_InitWithArgs(
        msdk_IAPProduct*    product,
        const char_utf8*    productIdentifier,
        const char_utf8*    localizedTitle,
        const char_utf8*    localizedDescription,
        float               price,
        const char_utf8*    localCurrency,
        const char_utf8*    formattedPrice,
        msdk_ProductType    productType,
        msdk_ProductStatus  productStatus
        );

MSDK_EXPORT void IAPProduct_InitByCopy(msdk_IAPProduct* product, const msdk_IAPProduct* other);
    
MSDK_EXPORT void IAPProduct_Destroy(msdk_IAPProduct* product);
    
}

#endif // MOBILESDKAPI_IAP_MANAGER_H_
