//
// FriendList.h
//

#ifndef MOBILE_SDK_API_FRIENDLIST_H
#define MOBILE_SDK_API_FRIENDLIST_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserInfo.h"

#if defined(UBI_ANDROID) && !defined(UNITY_SCRIPT)
	#include <jni.h>
#endif // UBI_ANDROID

typedef struct FriendListResult
{
	PointerArray_msdk_UserInfo* fl; // fl stand for friend list
	msdk_u32 	targetMask;
	msdk_u32 	currentMask;
	#if defined(UBI_ANDROID) || defined(UBI_WIN8)
		msdk_s8 graphAPIRequestID;
	#endif // UBI_ANDROID || UBI_WIN8
}FriendListResult;

#ifdef __cplusplus
extern "C"
{
#endif

///
/// Send a request to retrieve the user's friends on a network.
/// Available for :
/// |       iOS        |      Android       |
/// | :--------------: | :----------------: |
/// | MSDK_GAME_CENTER |                    |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK      |
/// |                  | MSDK_GAME_SERVICES |
/// @param p_networkId The network to retreive the friends from
///
MSDK_EXPORT void                        FriendList_CallFriendsList			(msdk_NetworkId p_network);

///
/// Get the status of a request
/// @param p_networkId The network of the processing request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status 				FriendList_StatusFriendsList		(msdk_NetworkId p_network);

///
/// Get the friends list retrieved
/// @param p_networkId The id of the network the request has been made to
/// @return The list of friends
///
MSDK_EXPORT Array_msdk_UserInfo*        FriendList_ResultFriendsList		(msdk_NetworkId p_network);

///
/// Release the memory and the slot allocated by the request
/// @param p_networkId The id of the request
///
MSDK_EXPORT void						FriendList_ReleaseFriendsList		(msdk_NetworkId p_network);

#ifdef __cplusplus
}
#endif


#endif // MOBILE_SDK_API_FRIENDLIST_H
