#ifndef MOBILE_SDK_API_SOCIALCONNECTION_H
#define MOBILE_SDK_API_SOCIALCONNECTION_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserInfo.h"

#ifdef __cplusplus
extern "C"
{
#endif

#ifdef UBI_IOS
///
/// Method to call on iOS when the application is put to foreground after connecting to Facebook
/// Need to be call when the event :
/// - (BOOL)application:(UIApplication *)application
///
/// Method to call on iOS when the application is put to foreground after connecting to Facebook
/// Need to be call when the event :
/// - (BOOL)application:(UIApplication *)application
///     openURL:(NSURL *)url
///     sourceApplication:(NSString *)sourceApplication
///     annotation:(id)annotation
/// is triggered
/// @param p_url a wchar_t* that contains the address of the application (URL Schemes in the .plist file)
///
bool SocialConnection_HandleOpenURL(const char_utf8* p_url);
#endif

///
/// Check if a user is connected to a given network
/// @return MSDK_TRUE on success, MSDK_FALSE otherwise
///
MSDK_EXPORT msdk_s8					SocialConnection_IsConnected(msdk_NetworkId p_network);

///
/// Return the informations of the local user on the network included in the given mask
/// @param  p_network The network to get the informations from
/// @return The user informations related to the network
///
MSDK_EXPORT const msdk_UserInfo*        SocialConnection_GetMyInfo					(msdk_NetworkId p_network);

///
/// Create a request to connect to a network. Must be called when you want to connect your user to this network. If it is the first time the user connect to that app,
/// most of the time it'll display a webview (or launch the local application) and let the user accept or reject the
/// @param p_network The netowrk to connect to
///
MSDK_EXPORT void                        SocialConnection_CallConnect					(msdk_NetworkId p_network);

///
/// Return the status of a request for connection
/// @param  p_network The network to connect to
/// @return The status of the request
///
MSDK_EXPORT msdk_Status 				SocialConnection_StatusConnect				(msdk_NetworkId p_network);

///
/// Get the result of a connection attempt (if the result is MSDK_UNKNOWN_ERROR watch the logs on Android and watch the content of the URL returned by facebook in your openURL callback)
/// @param  p_netowkr The network the request has been made to
/// @return The result of the request
///
MSDK_EXPORT msdk_Result 				SocialConnection_ResultConnect				(msdk_NetworkId p_network);

///
/// Release a request from the memory
/// @param p_network The network the request has been made to
///
MSDK_EXPORT void						SocialConnection_ReleaseConnect				(msdk_NetworkId p_network);

///
/// Try to launch an automatic connection to every social networks(if the user is already logged in)
///
MSDK_EXPORT void 						SocialConnection_CallAutoConnect			();

///
/// Get the status of an auto connection request
/// @return The status of the connection
///
MSDK_EXPORT msdk_Status 				SocialConnection_StatusAutoConnect			();

///
/// Release the auto connection request
///
MSDK_EXPORT void 						SocialConnection_ReleaseAutoConnect			();

///
/// @return MSDK_FALSE if you need to call ::SocialConnection_CallAutoConnect ; MSDK_TRUE if you doN'T.
///
MSDK_EXPORT char 						SocialConnection_IsAutoConnected			();

///
/// If your user want to logout from a social network, you can call that function.
///		- For Facebook: Remove te rights of the app on user's information (Friends, post on the wall, etc)
///	@param p_network The network to logout
///
MSDK_EXPORT void 						SocialConnection_Logout						(msdk_NetworkId p_network);

///
/// Returns the permissions of the user. The user <strong>must</strong> be connected.
/// @param The network to check the permissions
/// @return The permissions
///
MSDK_EXPORT PointerArray_char_utf8*     SocialConnection_GetUserPermissions(msdk_NetworkId p_networkId);
#ifdef __cplusplus
}
#endif


#endif // MOBILE_SDK_API_SOCIALCONNECTION_H
