#ifndef MOBILE_SDK_API_USER_ACHIEVEMENT_H_
# define MOBILE_SDK_API_USER_ACHIEVEMENT_H_

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserInfo.h"

#ifdef UBI_IOS
	//#import <UIKit/UIImage.h>
	/// In some case on iOS we return an image (GameCenter achievement for example)
	#if __OBJC__
		#import <UIKit/UIImage.h>
	#else
		class UIImage;
	#endif
	typedef	UIImage* msdk_Image;
#endif

///
/// Stores achievement's informations
///
typedef struct msdk_UserAchievement
{
	const char_utf8*	achievementId;
	const char_utf8*	title;
	const char_utf8*	achievedDescription;
	const char_utf8*	unachievedDescription;
	const char_utf8*	pictureUrl;
	const msdk_UserInfo*userInfo;
#ifdef UBI_IOS
	msdk_Image			image;
#endif
	const char_utf8*	achievementUrl;
	msdk_NetworkId		networkId;
	msdk_u16			percentComplete;
	msdk_u16			difficulty;
	msdk_s8				hidden;
}msdk_UserAchievement;

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_UserAchievement)

///
/// Allocate and initialize a msdk_UserAchievement structure
/// @return An allocated @ref msdk_UserAchievement with all fields set to NULL.
///
msdk_UserAchievement* UserAchievement_Create();

///
/// Deallocate a @ref msdk_UserInfo (checking for NULL field for safety)
/// @param the @ref msdk_UserInfo pointer to deallocate.
///
void UserAchievement_Release(msdk_UserAchievement* p_ach);

void UserAchievement_Copy(msdk_UserAchievement* p_dest, const msdk_UserAchievement* p_src);

#endif //! MOBILE_SDK_API_USER_ACHIEVEMENT_H_
