/*
 * UserInfo.h
 *
 *  Created on: 7 sept. 2012
 *      Author: dmabin
 */

#ifndef MOBILE_SDK_API_USERINFO_H
#define MOBILE_SDK_API_USERINFO_H

#include "MobileSDK/common/utils.h"

#ifdef __cplusplus
extern "C"
{
#endif

    enum msdk_PlayedStatus
    {
        MSDK_PLAYED,
        MSDK_NOT_PLAYED,
        MSDK_UNKNOWN
    };

	///
	/// Contains informations of a user
	///
	typedef struct msdk_UserInfo
	{
		const char_utf8* 		userId;			//!< The id of the user, Compulsory
		msdk_NetworkId 			networkId;		//!< The id of the network,Compulsory
		const char_utf8*		username;		//!< The name of the user, Compulsory
		const char_utf8*		pictureUrl;		//!< The url of the users' picture, Optional
        msdk_s8           		ageMin;
        msdk_s8					ageMax;
        msdk_Gender             gender;
        msdk_PlayedStatus       playedStatus;  //!< Status specifying if the user has already played the game
        const char_utf8*        birthday;       //!< The birthday of the user (only filled if the user_birthday permission is set)
		const char_utf8*		federatedId;	//!< The federate Id of the friends if he exist on Houston, Optional
        const char_utf8*        email;
	}msdk_UserInfo;

	#include "MobileSDK/common/Array.inl"
	DECLARE_ARRAYS(msdk_UserInfo)

	///
	/// Allocate and initialize a msdk_UserInfo structure
	/// @return An allocated @ref msdk_UserInfo with all fields set to NULL.
	///
	MSDK_EXPORT msdk_UserInfo* UserInfo_Create();

	///
	/// Initialize an already allocated msdk_UserInfo structure
	/// @param An allocated @ref msdk_UserInfo.
	///
	MSDK_EXPORT void UserInfo_Init(msdk_UserInfo* p_userInfo);

	///
	/// Deallocate a @ref msdk_UserInfo (checking for NULL field for safety)
	/// @param the @ref msdk_UserInfo pointer to deallocate.
	///
	MSDK_EXPORT void UserInfo_Release(msdk_UserInfo* p_userInfo);

#ifdef __cplusplus
}
#endif


#endif // MOBILE_SDK_API_USERINFO_H
