#ifndef MOBILE_SDK_API_INVITATION_H
#define MOBILE_SDK_API_INVITATION_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserInfo.h"

enum msdk_InvitationActionType
{
    MSDK_INVITATION_NONE,
    MSDK_INVITATION_SEND_OBJECT,
    MSDK_INVITATION_TURN,
    MSDK_INVITATION_ASK_OBJECT,
};

enum msdk_InvitationFilter
{
    MSDK_INVITATION_FILTER_NONE,
    MSDK_INVITATION_FILTER_APP_USER,
    MSDK_INVITATION_FILTER_APP_NON_USER,
};

typedef struct msdk_InvitationRequestInternal
{
    msdk_Service              	networkId;
    char_utf8*                  title;
    char_utf8*                  message;
    char_utf8*                  objectId;
    char_utf8*                  appLink;
    char_utf8*                  appLinkPicture;
    PointerArray_msdk_UserInfo  recipients; //UserInfo and not userId in case other network invitation system does not take a string as argument
    Array_msdk_key_value        extraData;
    Array_msdk_key_value        fbParam;
    msdk_InvitationFilter       filter;
    bool                        isAppInvite;
    msdk_InvitationActionType   actionType;
}msdk_InvitationRequestInternal;

typedef struct msdk_InvitationRequest
{
/*    #ifdef MSDK_BUILD_LIB
        msdk_InvitationRequestInternal* internal;
    #else
        void* internal;
    #endif
    */
    void* internal;
}msdk_InvitationRequest;

typedef struct msdk_InvitationResult
{
    msdk_Result                 result;
    const char_utf8*            requestId;
    const char_utf8*            message;
    PointerArray_msdk_UserInfo* users;
    Array_msdk_key_value        extraData;
    char_utf8*                  objectId;		//!< The id of the object to add, Compulsory only with MSDK_INVITATION_SEND_OBJECT
    char_utf8*                  objectTitle;
    msdk_InvitationActionType   actionType;
}msdk_InvitationResult;


#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_InvitationRequest);
DECLARE_ARRAYS(msdk_InvitationResult);

#ifdef __cplusplus
extern "C"
{
#endif

    ///
	/// @deprecated Please use ::Invitation_CreateRequest and ::Invitation_CallSendRequest instead.
    ///	Send a request to invite a friends to play
    ///	@param p_mask	A mask to select which network to use (currently only MSDK_FACEBOOK is supported for invitations)
    ///	@param p_title The title of the invitation
    /// @param p_message The message that you want to display in the invitation "Join me" for example.
    ///	@param p_users	Optionnal, the users to send invitations to (For Facebook if you set it NULL it will show an interface to select friends to invite)
    ///	@param p_extraDatas Optional, extras datas to send with the invitation (can be use for gifting for instance)
    ///	@return The id of the request
    ///
    MSDK_EXPORT void	Invitation_CallInvite(msdk_NetworkId p_network, const char_utf8* p_title, const char_utf8* p_message, Array_msdk_UserInfo* p_users, Array_msdk_key_value* p_extraDatas);

    ///
	/// @deprecated Please use ::Invitation_StatusSendRequest instead.
    ///	Get the status of an invitation request
    ///	@param p_requestId The id of the request
    ///	@return The status of the request
    ///
    MSDK_EXPORT msdk_Status Invitation_StatusInvite(msdk_NetworkId p_network);

    ///
	/// @deprecated Please use ::Invitation_ResultSendRequest instead.
    ///	Get the result of a request
    ///	@param p_requestId The id of the request
    ///	@return A list of msdk_InvitationResult (contains the result for each network)
    ///
    MSDK_EXPORT msdk_Result Invitation_ResultInvite(msdk_NetworkId p_network);

    ///
	/// @deprecated Please use ::Invitation_ReleaseSendRequest instead.
    ///	Release a request
    ///	@param p_requestId The id of the request
    ///
    MSDK_EXPORT void Invitation_ReleaseInvite(msdk_NetworkId p_network);


    ///
    /// Create a request to send later. Only works for Facebook at the moment.
    /// Can be used to invite friends or send them gifts
    /// @param p_service The service to use (Facebook)
    /// @param p_title The title of the request
    /// @param p_message The message of the request
    /// @return The created request
    ///
    MSDK_EXPORT msdk_InvitationRequest* Invitation_CreateRequest(msdk_Service p_service, const char_utf8* p_title, const char_utf8* p_message);

    ///
    /// Release the memory allocated for the request
    /// @param p_request The request to release
    ///
    MSDK_EXPORT void Invitation_ReleaseRequest(msdk_InvitationRequest* p_request);

    ///
    /// Add a appLink invitation to the request.
    /// AppLink need to be handle in AppDelegate
    /// @param p_request The request to configure
    /// @param p_appLinkURL The appLink to add
    /// @param p_pictureURL The picture to show with the invite, can be NULL
    ///
    MSDK_EXPORT void Invitation_SetAppInvite(msdk_InvitationRequest* p_request, const char_utf8* p_appLinkURL, const char_utf8* p_pictureURL);

    ///
    /// Add a recipient to the request.
    /// @param p_request The request to configure
    /// @param p_recipient The recipient to add
    ///
    MSDK_EXPORT void Invitation_AddRecipient(msdk_InvitationRequest* p_request, const msdk_UserInfo* p_recipient);

    ///
    /// Add a filter to the request.
    /// @param p_request The request to configure
    /// @param p_filter The filter to add
    ///
    MSDK_EXPORT void Invitation_AddFilter(msdk_InvitationRequest* p_request, msdk_InvitationFilter p_filter);

    ///
    /// Add extra data to a request. This has to be called for each key/value pair you want to add to the extra data
    /// @param p_request The request to configure
    /// @param p_key The key of the extra data
    /// @param p_value The value associated to the key
    ///
    MSDK_EXPORT void Invitation_AddExtraData(msdk_InvitationRequest* p_request, const char_utf8* p_key, const char_utf8* p_value);

    ///
    /// Add an object and/or an action to a request
    /// @param p_request The request to configure
    /// @param p_actionType The type of request you want to send
    /// @param p_objectId The id of the object you want to send. Set to NULL if no object is sent.
    ///
    MSDK_EXPORT void Invitation_AddObject(msdk_InvitationRequest* p_request, msdk_InvitationActionType p_actionType, const char_utf8* p_objectId);

    ///
    /// Start the sending of a social request
    /// @param p_request The request to send
    ///
    MSDK_EXPORT void Invitation_CallSendRequest(msdk_InvitationRequest* p_request);

    ///
    /// Get the status of a social request
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status Invitation_StatusSendRequest();

    ///
    /// Get the result of a social request
    /// @return The id of the request, the users to whom it has been sent and a msdk_Result to check if the request has been sent successfuly
    ///
    MSDK_EXPORT msdk_InvitationResult* Invitation_ResultSendRequest();

    ///
    /// Release the memory allocated for a social request
    ///
    MSDK_EXPORT void Invitation_ReleaseSendRequest();

    ///
    /// Start a request to get the pending requests of a user
    /// @param p_service The service to use (only work for Facebook at the moment)
    ///
    MSDK_EXPORT void Invitation_CallGetRequests(msdk_Service p_service);

    ///
    /// get the status of a GetRequests request
    /// @param p_service The service to use
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status Invitation_StatusGetRequests(msdk_Service p_service);

    ///
    /// Get the result of the pending social requests
    /// @param p_service The service to use
    /// @return The pending requests sent to the user
    ///
    MSDK_EXPORT Array_msdk_InvitationResult* Invitation_ResultGetRequests(msdk_Service p_service);

    ///
    /// Release the memory allocated to get the requests
    /// @param The service to use
    ///
    MSDK_EXPORT void Invitation_ReleaseGetRequests(msdk_Service p_service);

    ///
    /// Delete a request from the pending requests. Every request needs to be deleted once it has been processed
    /// @param p_service The service to use
    /// @param p_requestId The id of the request to delete
    ///
    MSDK_EXPORT msdk_s8 Invitation_CallDeleteRequest(msdk_Service p_service, const char_utf8* p_requestId);

    ///
    /// Get the status of the deletion of a request
    /// @param p_service The servcie to use
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status Invitation_StatusDeleteRequest(msdk_s8 p_requestId);

    ///
    /// Get the result of the deletion of a request
    /// @param p_service The service to use
    /// @return The result of the request
    ///
    MSDK_EXPORT msdk_Result Invitation_ResultDeleteRequest(msdk_s8 p_requestId);

    ///
    /// Release the memory allocated for the request
    /// @param p_service The service to use
    ///
    MSDK_EXPORT void Invitation_ReleaseDeleteRequest(msdk_s8 p_requestId);


    MSDK_EXPORT msdk_s8 Invitation_CallReadRequest(msdk_Service p_service, const char_utf8* p_requestId);

    MSDK_EXPORT msdk_Status Invitation_StatusReadRequest(msdk_s8 p_requestId);

    MSDK_EXPORT Array_msdk_key_value* Invitation_ResultReadRequest(msdk_s8 p_requestId);

    MSDK_EXPORT void Invitation_ReleaseReadRequest(msdk_s8 p_requestId);

//    MSDK_EXPORT msdk_Status Invitation_StatusReadRequest(msdk_s8 p_requestId);

#ifdef __cplusplus
}
#endif


#endif //MOBILE_SDK_API_INVITATION_H
