///
/// Wall.h
///
///  Created on: 30 oct. 2012
///  Author: dmabin
///
#ifndef WALL_H
#define WALL_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/communication/BaseMessage.h"

#ifdef __cplusplus
extern "C"
{
#endif

///
/// Open the application (if installed) or the browser to the network page of the id.
/// |       iOS        |      Android       |
/// | :--------------: | :----------------: |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK      |
/// |                  | MSDK_GAME_SERVICES |
/// For facebook you need to specify the facebook ID of the page to open (not the displayed name). If you want to open the ubisoft facebook page: https://www.facebook.com/ubisoft
/// You can't call that function with (MSDK_FACEBOOK, "ubisoft"), you need to call it with (MSDK_FACEBOOK, "81174647292"). How to get the facebook id? Navigate to https://graph.facebook.com/<displayed name>
/// For instance: https://graph.facebook.com/ubisoft and look into the displayed json for the key: "id". The value of that key is the facebook id.
/// For Game services you need to specify as wallID the complete url of the page on Google+, for Ubisoft page it would be: https://plus.google.com/+Ubisoft/posts
///
msdk_Result Wall_Open(msdk_NetworkId p_network, const char_utf8* p_wallId);

void Wall_CallIsLiked(msdk_NetworkId p_network, const char_utf8* p_wallId);
msdk_Status Wall_StatusIsLiked(msdk_NetworkId p_network);
msdk_Result Wall_ResultIsLiked(msdk_NetworkId p_network);
void Wall_ReleaseIsLiked(msdk_NetworkId p_network);

///
///  Create a request to publish a post message to an facebook wall
///  @param p_message The message to post
///  @return The id of the created request
///
MSDK_EXPORT void 	Wall_CallPublish	(msdk_BaseMessage* p_message);

///
///  Returns the status of a publish request
///  @param p_requestId The id of the request
///  @return The status of the request 
///
MSDK_EXPORT msdk_Status Wall_StatusPublish	();

///
///  Returns the result of a publish request
///  @param p_requestId The id of the request
///  @return The result of the request
/// |     Value	       |    Description   |
/// | :--------------: | :--------------: |
/// | MSDK_SUCCESS     | The post has been successfully published |
/// | MSDK_USER_CANCEL     | The post has been cancelled by the user |
/// | MSDK_NETWORK_ERROR | An error occured while publishing the score. Check the internet connection before making the call. |
/// | MSDK_NETWORK_NOT_CONNECTED | Trying to published a post on a network where the user is not connected |
///
MSDK_EXPORT msdk_Result Wall_ResultPublish	();

///
///  Release a publish request
///  @param p_requestId The id of the request
///
MSDK_EXPORT void 		Wall_ReleasePublish	();

#ifdef __cplusplus
}
#endif
#endif // WALL_H
