#ifndef MOBILE_SDK_API_MAILBOX_H
#define MOBILE_SDK_API_MAILBOX_H

#include "MobileSDK/social/profile/UserProfile.h"

typedef struct msdk_Mail
{
    char_utf8*              message;        //!< The content of the mail
    char_utf8*              title;          //!< The title of the mail
    char_utf8*              messageId;      //!< The id of the mail
    char_utf8*              from;           //!< The sender federatedId of the mail
    char_utf8*              date;           //!< The date of the mail
    char_utf8*              status;         //!< The status of the mail (unread, read)
    char                    autodestruct;   //!< True means that the mail will be autodestructed when read
    msdk_u32                expiringTime;   //!< The expiring time of the mail
    Array_msdk_UserInfo     recipients;     //!< The recipients of the mail
}msdk_Mail;

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_Mail)

#ifdef __cplusplus
extern "C"
{
#endif
///
/// Create a mail to send through the mailbox
/// @param p_message The message of the mail
/// @param p_title The title of the message
/// @return The create mail
///
MSDK_EXPORT msdk_Mail*  Mailbox_CreateMail(const char_utf8* p_message, const char_utf8* p_title);

///
/// Add a recipient to a mail
/// @param p_mail The mail to attach a recipient to
/// @param p_user The new recipient
///
MSDK_EXPORT void        Mailbox_AddRecipient(msdk_Mail* p_mail, msdk_UserInfo* p_user);

///
/// Send a mail.
/// If there is no internet connection, the mail will be saved locally and resent the next time you initialize the UserProfile
/// @param p_mail The mail to send
///
MSDK_EXPORT void        Mailbox_SendMail(msdk_Mail* p_mail);

///
/// Start a request to retreive the user's mails
///
MSDK_EXPORT void        Mailbox_CallGet();

///
/// Get the status of a mail request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status Mailbox_StatusGet();

///
/// Get the result of a mail request. The core message of the mails are not retrieved.
/// You'll need to call Mailbox_CallRead to have it
/// @return The mails of the user.
///
MSDK_EXPORT Array_msdk_Mail*    Mailbox_ResultGet();

///
/// Release the current mailbox request
///
MSDK_EXPORT void        Mailbox_ReleaseGet();

///
/// Start to read the content of a mail
/// @param The mail to read from
///
MSDK_EXPORT void        Mailbox_CallRead(msdk_Mail* p_mail);

///
/// Get the status of the reading of a mail
/// @param p_mail The mail to process
/// @return the status of the request
///
MSDK_EXPORT msdk_Status Mailbox_StatusRead(msdk_Mail* p_mail);

///
/// Get the result of the reading of a mail
/// @param p_mail The mail to process
/// @return The content of the mail
///
MSDK_EXPORT const char_utf8* Mailbox_ResultRead(msdk_Mail* p_mail);

///
/// Release a CallRead request
/// @param p_mail The mail to process
///
MSDK_EXPORT void            Mailbox_ReleaseRead(msdk_Mail* p_mail);

///
/// Delete a mail from the mailbox. If no internet connection the mail will be automatically deleted later
/// @param p_mail The mail to delete
///
MSDK_EXPORT void            Mailbox_Delete(msdk_Mail* p_mail);

#ifdef __cplusplus
}
#endif

#endif
