#ifndef MOBILE_SDK_API_TRACKING_H
# define MOBILE_SDK_API_TRACKING_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/tracking/TrackingInterface.h"

#if defined(UBI_ANDROID) && !defined(UNITY_SCRIPT)
#include <jni.h>
#endif // UBI_ANDROID

#ifdef __cplusplus
extern "C"
{
#endif

    ///
    ///	Add a tracking service that will be used when tracking calls are made
    /// @param p_interface The interface to register
    ///
MSDK_EXPORT void    Analytics_RegisterInterface(msdk_Service p_service, msdk_TrackingInterface* p_interface);

///
///	Send a custom event to the trackers
/// @deprecated Please use ::Analytics_TrackEvent instead.
/// @param p_eventId The event to track (unique name for the event)
///	@param p_keyValues The parameters associated to the event to track. To end a timed event you have to call Analytics_EndCustomTimedEvent or pause/leave your app.
///
MSDK_EXPORT void	Analytics_TrackCustomEvent	(const char_utf8* p_eventId, const PointerArray_msdk_key_value* p_keyValues, char p_isTimed);

///
///	Send a custom event to the trackers
/// @param p_eventId The event to track (unique name for the event)
///	@param p_keyValues The parameters associated to the event to track. To end a timed event you have to call Analytics_EndCustomTimedEvent or pause/leave your app.
///
MSDK_EXPORT void	Analytics_TrackEvent	(const char_utf8* p_eventId, const PointerArray_msdk_key_value* p_keyValues, char p_isTimed);

///
///	Send an event to a specific service
/// @param p_eventId The event to track (unique name for the event)
///	@param p_keyValues The parameters associated to the event to track. To end a timed event you have to call Analytics_EndCustomTimedEvent or pause/leave your app.
///
MSDK_EXPORT void	Analytics_TrackEventOnService	(const char_utf8* p_eventId, const PointerArray_msdk_key_value* p_keyValues, char p_isTimed, msdk_Service p_service);

///
///	End a custom timed event
/// @deprecated Please use Analytics_EndTimedEvent instead.
/// @param p_eventID The ID of the event
///
MSDK_EXPORT void	Analytics_EndCustomTimedEvent	(const char_utf8* p_eventID, const PointerArray_msdk_key_value* p_keyValues);

///
///	End a custom timed event
/// @param p_eventID The ID of the event
///
MSDK_EXPORT void	Analytics_EndTimedEvent	(const char_utf8* p_eventID, const PointerArray_msdk_key_value* p_keyValues);

///
/// Add a parameter for an event that will be tracked
/// @param p_eventId The event that will be sent
/// @param p_key The key of the parameter
/// @param p_value The value of the parameter
///
MSDK_EXPORT void    Analytics_AddEventParameter(const char_utf8* p_eventId, const char_utf8* p_key, const char_utf8* p_value);

MSDK_EXPORT void    Analytics_AddEventParameterInt(const char_utf8* p_eventId, const char_utf8* p_key, msdk_s32 p_value);

MSDK_EXPORT void    Analytics_AddEventParameterUInt(const char_utf8* p_eventId, const char_utf8* p_key, msdk_u32 p_value);

MSDK_EXPORT void    Analytics_AddEventParameterFloat(const char_utf8* p_eventId, const char_utf8* p_key, float p_value);

///
/// Send an event. If parameters have been added to the event they will  be sent with it.
/// @param p_eventId The event to track
/// @param p_isTimed MSDK_TRUE if it is a timed event, MSDK_FALSE otherwise
///
MSDK_EXPORT void	Analytics_SendEvent(const char_utf8* p_eventId, char p_isTimed);

///
/// Send an event on a specific service. If parameters have been added to the event they will  be sent with it.
/// @param p_eventId The event to track
/// @param p_isTimed MSDK_TRUE if it is a timed event, MSDK_FALSE otherwise
/// @param p_service The service to send the event to
///
MSDK_EXPORT void	Analytics_SendEventOnService(const char_utf8* p_eventId, char p_isTimed, msdk_Service p_service);

///
/// End a timed event
/// @param p_eventId The event to end
///
MSDK_EXPORT void	Analytics_EndEvent(const char_utf8* p_eventId);

#ifdef __cplusplus
}
#endif


#endif //! MOBILE_SDK_API_TRACKING_H
