#ifndef MOBILE_SDK_API_DNA_TRACKING_H
# define MOBILE_SDK_API_DNA_TRACKING_H

#include "MobileSDK/common/Utils.h"

extern const char_utf8* MSDK_DNA_UNLOCK_ACHIEVEMENT;
extern const char_utf8* MSDK_DNA_PLAYER_ACHIEVEMENT;
extern const char_utf8* MSDK_DNA_GAME_START;
extern const char_utf8* MSDK_DNA_PLAYER_START;
extern const char_utf8* MSDK_DNA_CONTEXT_START;
extern const char_utf8* MSDK_DNA_CONTEXT_STOP;
extern const char_utf8* MSDK_DNA_PLAYER_STAT;
extern const char_utf8* MSDK_DNA_LOCALIZATION;
extern const char_utf8* MSDK_DNA_PROGRESSION;
extern const char_utf8* MSDK_DNA_PLAYER_PROGRESSION;
extern const char_utf8* MSDK_DNA_PLAYER_DLC;
extern const char_utf8* MSDK_DNA_PLAYER_POSTION;

extern const char_utf8* MSDK_DNA_ACHIEVEMENT_ID;
extern const char_utf8* MSDK_DNA_CONTEXT_TYPE;
extern const char_utf8* MSDK_DNA_CONTEXT_NAME;
extern const char_utf8* MSDK_DNA_LOCALE_CODE;
extern const char_utf8* MSDK_DNA_GAME_LANGUAGE;
extern const char_utf8* MSDK_DNA_SUBTITLES_ACTIVATION;
extern const char_utf8* MSDK_DNA_SUBTITLES_LANGUAGE;
extern const char_utf8* MSDK_DNA_PROGRESSION_TYPE;
extern const char_utf8* MSDK_DNA_PROGRESSION_VALUE;
extern const char_utf8* MSDK_DNA_DLC_ID;
extern const char_utf8* MSDK_DNA_STAT_NAME;
extern const char_utf8* MSDK_DNA_STAT_VALUE;

enum msdk_UbiServicesTokenType
{
    MSDK_UBISERVICES_FACEBOOK_TOKEN,
    MSDK_UBISERVICES_UPLAY_TOKEN,
    MSDK_UBISERVICES_UBISERVICES_TOKEN,
    MSDK_UBISERVICES_UBIMOBILE_TOKEN,
    MSDK_UBISERVICES_GAMECENTER_TOKEN,
    MSDK_UBISERVICES_GAMESERVICES_TOKEN
};

#ifdef __cplusplus
extern "C"
{
#endif

    ///
    /// Activate the DNA tracking on Prod
    /// Must be called if you want to use DNA
    ///
    MSDK_EXPORT    void    Analytics_ActivateDNA();

    ///
    /// Activate the DNA tracking on UAT
    /// Must be called if you want to use DNA on UAT
    /// <strong>CAUTION : You must not use UAT on a live build<strong>
    ///
    MSDK_EXPORT    void    Analytics_ActivateDNAOnUAT();

    ///
    /// Activate DNA tracking. By using this function you'll need to pass a UbiServices token later using Analytics_SetDNAToken.
    ///
    MSDK_EXPORT    void    Analytics_ActivateDNAWithCustomToken();

    ///
    /// Set the token that will be used to connect to UbiServices
    /// @param p_token The token to use
    /// @param p_type The type of token
    ///
    MSDK_EXPORT     void   Analytics_SetDNAToken(const char_utf8* p_token, msdk_UbiServicesTokenType p_type);

    ///
    /// Add a parameter that will be sent along with the player start event
    /// <strong>Must be called before msdk_Init</strong>
    /// @param p_key The key of the parameter
    /// @param p_value The value of the parameter
    ///
    MSDK_EXPORT     void Analytics_AddDNAStartEventParameter(const char_utf8* p_key, const char_utf8* p_value);

#ifdef __cplusplus
}
#endif

#endif
