#ifndef MOBILE_SDK_API_FACEBOOK_TRACKING_H
# define MOBILE_SDK_API_FACEBOOK_TRACKING_H

#include "MobileSDK/common/Utils.h"

#ifdef __cplusplus
extern "C"
{
#endif
    
    enum MSDK_FACEBOOK_EVENT
    {
        MSDK_FB_EVENT_ACHIEVED_LEVEL, // LEVEL
        MSDK_FB_EVENT_ACTIVATED_APP,
        MSDK_FB_EVENT_ADDED_PAYMENT_INFO, // SUCCESS
        MSDK_FB_EVENT_ADDED_TO_CART, // CONTENT_TYPE, CONTENT_ID, CURRENCY
        MSDK_FB_EVENT_ADDED_TO_WISHLIST, // CONTENT_TYPE, CONTENT_ID, CURRENCY
        MSDK_FB_EVENT_COMPLETED_REGISTRATION, // REGISTRATION_METHOD
        MSDK_FB_EVENT_COMPLETED_TUTORIAL, // SUCCESS, CONTENT_ID
        MSDK_FB_EVENT_INITIATED_CHECKOUT, // CONTENT_TYPE, CONTENT_ID, NUM_ITEMS, PAYMENT_INFO_AVAILABLE, CURRENCY
        MSDK_FB_EVENT_PURCHASED, // NUM_ITEMS, CONTENT_TYPE, CONTENT_ID, CURRENCY
        MSDK_FB_EVENT_RATED, // CONTENT_TYPE, CONTENT_ID, MAX_RATING_VALUE
        MSDK_FB_EVENT_SEARCHED, // CONTENT_TYPE, SEARCH_STRING, SUCCESS
        MSDK_FB_EVENT_SPENT_CREDITS, // CONTENT_TYPE, CONTENT_ID
        MSDK_FB_EVENT_UNLOCKED_ACHIEVEMENT, // DESCRIPTION
        MSDK_FB_EVENT_VIEWED_CONTENT // CONTENT_TYPE, CONTENT_ID, CURRENCY
    };
    
    enum MSDK_FACEBOOK_PARAM
    {
        MSDK_FB_PARAM_CONTENT_ID,
        MSDK_FB_PARAM_CONTENT_TYPE,
        MSDK_FB_PARAM_CURRENCY,
        MSDK_FB_PARAM_DESCRIPTION,
        MSDK_FB_PARAM_LEVEL,
        MSDK_FB_PARAM_MAX_RATING_VALUE,
        MSDK_FB_PARAM_NUM_ITEMS,
        MSDK_FB_PARAM_PAYMENT_INFO_AVAILABLE,
        MSDK_FB_PARAM_REGISTRATION_METHOD,
        MSDK_FB_PARAM_SEARCH_STRING,
        MSDK_FB_PARAM_SUCCESS
    };
    
    ///
    /// Add a parameter to a Facebook event.
    /// @param p_eventId The event that you want to track with Facebook. Please check Facebook documentation <a href="https://developers.facebook.com/docs/android/app-events#events">here</a> for more information.
    /// @param p_param The parameter you want to add to that event. Please check facebook doc <a href="https://developers.facebook.com/docs/android/app-events#params">here</a> for more information.
    /// @param p_value The value of the parameter you want to track.
    ///
    MSDK_EXPORT void Analytics_AddParamFbInt(MSDK_FACEBOOK_EVENT p_eventId, MSDK_FACEBOOK_PARAM p_param, msdk_s16 p_value);
    
    ///
    /// Add a parameter to a Facebook event.
    /// @param p_eventId The event that you want to track with Facebook. Please check Facebook documentation <a href="https://developers.facebook.com/docs/android/app-events#events">here</a> for more information.
    /// @param p_param The parameter you want to add to that event. Please check facebook doc <a href="https://developers.facebook.com/docs/android/app-events#params">here</a> for more information.
    /// @param p_value The value of the parameter you want to track.
    ///
    MSDK_EXPORT void Analytics_AddParamFbString(MSDK_FACEBOOK_EVENT p_eventId, MSDK_FACEBOOK_PARAM p_param, const char_utf8* p_value);
    
    ///
    /// Track an event using Facebook event tracking solution (doc <a href="https://developers.facebook.com/docs/android/app-events#events">here</a>)
    /// Currently no custom events or custom parameters are supported.
    /// To add a parameter to that event, please use ::Analytics_AddParamFbInt or ::Analytics_AddParamFbString.
    /// @param p_eventId The event that you want to track with Facebook. Please check Facebook documentation <a href="https://developers.facebook.com/docs/android/app-events#events">here</a> for more information.
    ///
    MSDK_EXPORT void Analytics_TrackFbEvent(MSDK_FACEBOOK_EVENT p_eventId);
    
    ///
    /// Track an event using Facebook event tracking solution (doc <a href="https://developers.facebook.com/docs/android/app-events#events" target="_blanck">here</a>)
    /// Currently no custom events or custom parameters are supported.
    /// To add a parameter to that event, please use ::Analytics_AddParamFbInt or ::Analytics_AddParamFbString.
    /// @param p_eventId The event that you want to track with Facebook. Please check Facebook documentation <a href="https://developers.facebook.com/docs/android/app-events#events">here</a> for more information.
    /// @param p_value The "valueToSum" given Facebook documentation (see link above).
    ///
    MSDK_EXPORT void Analytics_TrackFbEventWithValue(MSDK_FACEBOOK_EVENT p_eventId, float p_value);
    
#ifdef __cplusplus
}
#endif

#endif
