#ifndef MOBILE_SDK_API_UBI_SERVICES_H
# define MOBILE_SDK_API_UBI_SERVICES_H

#include "MobileSDK/common/Utils.h"

namespace ubiservices {
    class Facade;
}

typedef void (*Ubiservices_AuthCallback)(const char_utf8* p_token, const char_utf8* p_ticket, ubiservices::Facade* p_facade, msdk_s32 p_errorCode, const char_utf8* p_errorMessage);

typedef void (*Ubiservices_UnityAuthCallback)(const char_utf8* p_token, const char_utf8* p_ticket, msdk_s32 p_errorCode, const char_utf8* p_errorMessage, ubiservices::Facade* p_facade);

#ifdef __cplusplus
extern "C"
{
#endif

///
/// Register a callback that will be called when the internal connection to ubiservices is a success. The token and the ticket are given to the callback. The Ubiservices facade is also retuned ad a const void*. If you want to use it you'll need to cast it to a US_NS::Facade
///
void    Ubiservices_RegisterAuthCallback(Ubiservices_AuthCallback);

/// Relaunch the login to Ubiservices.
void    Ubiservices_RestartLogin();

///
/// Configure the init of ubiservices.<strong>Must be called before msdk_Init</strong>
/// @param p_applicationBuildId The version of the game using the UbiServicesSDK. If NULL Ubiservices App Version field from Houston will be used
/// @param p_gameStartEventGameVersion The game version to send in the game.start event
/// @param p_gameStartEventCustomData Json that will be sent as custom data in the game.start event
///
void Ubiservices_ConfigureSDK(const char_utf8* p_applicationBuildId, const char_utf8* p_gameStartEventGameVersion, const char_utf8* p_gameStartEventCustomData);

#ifdef MSDK_UNITY_BINDING
    MSDK_EXPORT void Ubiservices_DefinePointers(long long authPointer);
#endif

#ifdef __cplusplus
}
#endif

#endif
