#ifndef MOBILESDKPRIVATE_AD_ADSMANAGER
#define MOBILESDKPRIVATE_AD_ADSMANAGER

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/ad/Ad.h"
#include "MobileSDKprivate/ad/AdInterface.h"
#include "MobileSDKprivate/common/RequestPool.h"
#include "MobileSDKprivate/common/Thread.h"
#include "MobileSDKprivate/common/CriticalSection.h"
#include "MobileSDKprivate/common/db/AdsPreferences.h"

struct msdk_CurrencyResult
{
	Array_msdk_VirtualCurrency*				currencies;
	msdk_u32								targetNetworkNumber;
	msdk_u32								currentNetworkNumber;
	std::vector<MobileSDKAPI::ThreadStruct*> threads;
};

struct msdk_CurrencyParam
{
	msdk_s8				requestId;
	const char_utf8*	rewardCheckUrl;
	const char_utf8*	currencyName;
};

const	msdk_u32	MSDK_BANNER_BITMASK = 0x00001;
const	msdk_u32	MSDK_INTERSTICIAL_BITMASK = 0x00010;
const	msdk_u32	MSDK_VIDEO_BITMASK = 0x00100;
const	msdk_u32	MSDK_MOREGAMES_BITMASK = 0x01000;
const	msdk_u32	MSDK_PAUSE_BITMASK = 0x10000;
const	msdk_u32	MSDK_EMPTY_BITMASK = 0x00000;

const int msdk_AdBitmasks[8] = {MSDK_EMPTY_BITMASK,
								MSDK_INTERSTICIAL_BITMASK,
								MSDK_INTERSTICIAL_BITMASK,
								MSDK_BANNER_BITMASK,
								MSDK_BANNER_BITMASK,
								MSDK_BANNER_BITMASK,
								MSDK_BANNER_BITMASK,
								MSDK_EMPTY_BITMASK
								};

struct msdk_ThreadParams;

class AdsManager
{
private:
	MobileSDKAPI::RequestPool<msdk_Result, INIT_AD>						m_initPool;
	MobileSDKAPI::RequestPool<msdk_CurrencyResult*, VIRTUAL_CURRENCY>	m_currencyRequestPool;
	MobileSDKAPI::CriticalSectionStruct									m_mutex;
	MobileSDKAPI::CriticalSectionStruct									m_adsMutex;
    std::map<const char_utf8*, charCharMap, CharCompFunctor>    m_currencies;

	msdk_AdsConfig											m_adsConfig;
	std::map<msdk_s8, msdk_AdInterface*>					m_ads;
	std::map<msdk_s8, msdk_ThreadParams*>					m_threads;
	bool													m_isInit;
	MobileSDKAPI::ThreadStruct								m_initThread;

	std::vector<msdk_AdInterface*>							m_interfaces;

	std::map<msdk_AdType, msdk_u32>							m_displaysCount;
	msdk_u32												m_lastIntersticialDisplayTime;

	msdk_AdInterface*		GetNextAd(msdk_AdType adType);

	static unsigned int	RefreshAd					(void* p_params);
	static void			PreloadAdRotation			(void* p_param);
	static void			DisplayAdRotation			(void* p_param);
	static void			RemoveOldAdRotation			(void* p_param);
	static void			InitInterface				(void* p_param);

	void					InsertAd					(msdk_AdInterface* p_ad);
	void					RemoveAd					(msdk_AdInterface* p_ad);
	static unsigned int		CurrencyThread				(void* p_param);
	static unsigned int		UpdatePreferencesThread		(void* p_param);

	void					UpdateAdConfig(msdk_s8 p_requestId = -1);
	static					AdsPreferences				s_preferences;

	inline msdk_AdsConfig GetAdsConfig() { return m_adsConfig; };
	inline std::map<msdk_s8, msdk_AdInterface*> GetAds() { return m_ads; };
public:
	AdsManager();

	~AdsManager();

	void				AddInterface(msdk_AdInterface*);

	void				Resume();

    void                ConfigureCurrency(const char_utf8* p_currencyName, const char_utf8* p_service, const char_utf8* p_currencyId);

	msdk_s8				CallInit			();

	msdk_Status			StatusInit			(msdk_s8 p_id);

	msdk_Result			ResultInit			(msdk_s8 p_id);

	void				ReleaseInit			(msdk_s8 p_id);

	msdk_s8 			CallPreloadAd		(msdk_AdInfo* p_adInfo);

	msdk_Status 		StatusPreloadAd		(msdk_s8 p_id);

	msdk_Result			ResultPreloadAd		(msdk_s8 p_id);

	void				UpdatePreloadAd		(msdk_s8 p_id, msdk_Result p_result);

	void 				DisplayAd			(msdk_s8 p_id);

	void 				AdVisibility		(msdk_s8 p_id, char p_visibility);

	Array_msdk_AdEvent*	GetAdEvents			(msdk_s8 p_id);

	void				UpdateAdEvents		(msdk_s8 p_id, msdk_AdEvent p_event);

	void				DeleteAd			(msdk_s8 p_id);

	msdk_s8				CallVirtualCurrency	();

	msdk_Status			StatusVirtualCurrency	(msdk_s8 p_id);

	Array_msdk_VirtualCurrency*		ResultVirtualCurrency	(msdk_s8 p_id);

	void				ReleaseVirtualCurrency(msdk_s8 p_id);

	void				UpdateVirtualCurrency(msdk_s8 p_requestId, Array_msdk_VirtualCurrency* p_currencies);

};

struct msdk_ThreadParams
{
	AdsManager*					manager;
	msdk_s8						adId;
	MobileSDKAPI::ThreadStruct*	thread;
	bool						endThread;
};

#endif //MOBILE_SDK_API_ADS_MANAGER_H_
