#ifndef MOBILESDKPRIVATE_COMMON_CRITICALSECTION
#define MOBILESDKPRIVATE_COMMON_CRITICALSECTION

#ifdef UBI_ANDROID
	#include <cstddef>
#endif
#include <cstddef>

namespace MobileSDKAPI
{
	struct CriticalSectionStruct
	{
		CriticalSectionStruct();
		CriticalSectionStruct(const CriticalSectionStruct& css);
		CriticalSectionStruct& operator =(const CriticalSectionStruct& css);
		~CriticalSectionStruct();

		struct CriticalSectionStructImpl;
		CriticalSectionStructImpl* m_impl;
	};

	bool CriticalSectionInitialize(CriticalSectionStruct& criticalSectionStruct, unsigned int spinCount = 0);

	void CriticalSectionEnter(CriticalSectionStruct& criticalSectionStruct);

	void CriticalSectionLeave(CriticalSectionStruct& criticalSectionStruct);

	bool CriticalSectionTryEnter(CriticalSectionStruct& criticalSectionStruct);

	// must used for scopes that trigger C++ exceptions
	struct AutoLockCriticalSection
	{
        inline AutoLockCriticalSection()
        :m_criticalSection(NULL)
        {

        }

        inline void SetLock(CriticalSectionStruct* p_lock)
        {
            m_criticalSection = p_lock;
        }

		inline AutoLockCriticalSection(CriticalSectionStruct* criticalSection)
			: m_criticalSection(criticalSection)
		{
            if(m_criticalSection != NULL)
                CriticalSectionEnter(*m_criticalSection);
		}

		inline ~AutoLockCriticalSection()
		{
            if(m_criticalSection != NULL)
                CriticalSectionLeave(*m_criticalSection);
		}

		CriticalSectionStruct* m_criticalSection;
	};
}

#endif //! MOBILE_SD_API_CRITICAL_SECTION_H
