#ifndef MOBILESDKPRIVATE_COMMON_FILESYSTEMINTERFACE
#define MOBILESDKPRIVATE_COMMON_FILESYSTEMINTERFACE

#ifndef MOBILESDK_HELPERS_API_FILE_SYSTEM_H

#ifdef UNITY_SCRIPT
#include "common/Utils.h"
#endif

// -------------------------------------------------------------------------
// Files
typedef enum msdk_FileOpenMode
{
	MSDK_MODE_READ				= 1,		// Read
	MSDK_MODE_WRITE				= 2,		// Write
	MSDK_MODE_APPEND			= 4,		// Append (to combine with read or/and write)
	MSDK_MODE_BINARY			= 8,		// Binary mode
	MSDK_MODE_CREATE			= 16		// Create the file if it doesn't exist
}msdk_FileOpenMode;

typedef enum msdk_FileSeekOrigin
{
	MSDK_ORIGIN_BEGIN	= 0x01,		// Seek offset is taken from begin of the file
	MSDK_ORIGIN_END		= 0x02,		// Seek offset is taken from end of the file
	MSDK_ORIGIN_CURRENT	= 0x03,		// Seek offset is taken from current position of the file
}msdk_FileSeekOrigin;

typedef enum msdk_FileArea
{
	MSDK_PUBLIC_GAME_DATA,
	MSDK_PUBLIC_USER_DATA,
	MSDK_PRIVATE_DATA,
	MSDK_PRIVATE_CACHE_DATA,
	MSDK_PUBLIC_CACHE_DATA
}msdk_FileArea;


typedef enum msdk_FileType
{
	MSDK_TEXT,
	MSDK_BINARY
}msdk_FileType;
//-------------------------------------------------------------------------
// Directories
typedef struct msdk_FileStruct
{
	const char_utf8*	fullname;
	msdk_FileArea		area;
	void*				fileHandle;
	void*				extraData;

}msdk_FileStruct;


typedef enum msdk_DirectoryItemType
{
	TYPE_NONE	= 0x00,
	TYPE_FILE	= 0x01,
	TYPE_DIR	= 0x02,
}msdk_DirectoryItemType;

typedef struct msdk_DirectoryStruct
{
	const char_utf8*	fullname;
	msdk_FileArea		type;
	void*				dirHandle;
	void*				currentDirEntry;
}msdk_DirectoryStruct;

typedef struct msdk_DirectoryEntry
{
	char const*				name;
	msdk_DirectoryItemType	type;
}msdk_DirectoryEntry;

//-------------------------------------------------------------------------

typedef struct msdk_AreaPermissions
{
	msdk_u32	permissionsPublicGameData;
	msdk_u32	permissionsPublicUserData;
	msdk_u32	permissionsPrivateData;
	msdk_u32	permissionsPrivateCacheData;
	msdk_u32	permissionsPublicCacheData;
}msdk_AreaPermissions;

typedef struct msdk_FileSystemInterface
{

	char						(*FileExist)(char_utf8 const* p_fullname, msdk_FileArea p_fileArea);
	msdk_FileStruct*			(*FileOpen)(char_utf8 const* p_fullname, msdk_u32 p_mode, msdk_FileArea p_fileArea);
	size_t						(*FileSize)(msdk_FileStruct* p_fileHandle);
	size_t						(*FileRead)(msdk_FileStruct* p_fileHandle, void* p_buffer, msdk_u32 p_bufferSize);
	size_t						(*FileSeek)(msdk_FileStruct* p_fileHandle, size_t p_offset, msdk_FileSeekOrigin p_origin);
	size_t						(*FileWrite)(msdk_FileStruct* p_fileHandle, void const* p_buffer, msdk_u32 p_bufferSize);
	char						(*FileClose)(msdk_FileStruct* p_fileHandle);

	char						(*DirectoryExist)(char_utf8 const* p_fullname, msdk_FileArea p_type);
	char						(*DirectoryCreate)(char_utf8 const* p_fullname, msdk_FileArea p_type, msdk_u16 p_mode);
	msdk_DirectoryStruct*		(*DirectoryOpen)(char_utf8 const* p_fullname, msdk_FileArea p_type);
	char						(*DirectoryClose)(msdk_DirectoryStruct* p_dirHandle);
	char						(*DirectoryGetFirst)(msdk_DirectoryStruct* p_dirHandle, msdk_DirectoryEntry* p_dirEntry);
	char						(*DirectoryGetNext)(msdk_DirectoryStruct* p_dirHandle, msdk_DirectoryEntry* p_dirEntry);
} msdk_FileSystemInterface;

#endif //! MOBILESDK_HELPERS_API_FILE_SYSTEM_H

#endif //! MOBILE_SDK_FILESYSTEM_INTERFACE_H
