#ifndef MOBILESDKPRIVATE_COMMON_JNITOOLS
#define MOBILESDKPRIVATE_COMMON_JNITOOLS

#ifdef UBI_ANDROID

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/common/Init.h"

// ==================================================================================
// Mobile SDK internal stuff
#ifdef MSDK_BUILD_LIB

	typedef struct _JNIEnv JNIEnv;

	namespace MobileSDKAPI
	{

		// JNIEnvHandler provides 2 functionalities in a function scope
		// 1. Obtains a correct JNIEnv* according to the thread the function is called from
		//    If that thread was not attached as a Java thread,
		//	  JNIEnvHandler will attach and deattach as necessary
		// 2. Pushes and pops a Java frame so that local references will be cleaned
		//    (this is very important for native activities or threads with a native entry point)
		//
		//	Usage:
		//  void foo() {
		//    JNIEnvHandler envHandler(32); //16 default, increment as necessary
		//    JNIEnv* env = envHandler.Get();
		//  }
		//
		//  If you create functions that return a local java reference:
		//	  e.g.   jclass FindClass(JNIEnv* env, const char* className);
	    //  you should use JNIEnvHandler in the parent scope and pass in the JNIEnv*
		//  otherwise you leave the VM the oportunity to collect your returned result
		//
		class JNIEnvHandler
		{
		public:
			JNIEnvHandler(msdk_u32 localFrameCount = 16);
			~JNIEnvHandler();

			inline JNIEnv* Get()		{ return m_env; }
			inline operator JNIEnv* ()	{ return Get(); }

		protected:
			JNIEnv*	m_env;
			bool	m_obtainedByAttach;
		};


		// worst case Java local reference use count: 8
		jclass FindClass(JNIEnv* env, jobject activity, const char* className);

		void PrepareLooper(JNIEnv* p_env, jobject p_androidActivity);
	} // namespace MobileSDKAPI
#endif
// ==================================================================================

#endif // defined(UBI_ANDROID)


#endif // defined(_COMMON_JNITOOLS_H_)
