#ifndef MOBILESDKPRIVATE_COMMON_MEMMANAGER
#define MOBILESDKPRIVATE_COMMON_MEMMANAGER
	#include "MobileSDK/common/MemManager.h"


// ==================================================================================
// Mobile SDK internal stuff
#ifdef MSDK_BUILD_LIB

	namespace MobileSDKAPI
	{
		class MemManager
		{
		private:
			static msdk_MemInterface* 	m_interfaces[];
			static signed char 			m_currentAllocatorId;
			static signed char 			m_lastAllocatorId;
			#ifdef UBI_IOS
				static CFAllocatorRef 		m_allocator;
			#endif
		public:
			static void RegisterAllocator	(msdk_MemInterface* p_memInterface);

			static bool UnRegisterAllocator(msdk_MemInterface*);
			static bool UnRegisterAllocator(char p_id);
			static bool UnRegisterAllocator(char_utf8* p_name);

			static bool UseAllocator(msdk_MemInterface* p_memInterface);
			static bool UseAllocator(char p_id);
			static bool UseAllocator(char_utf8* p_name);

			static inline msdk_MemInterface* GetCurrentAllocator()
			{
				return m_interfaces[m_currentAllocatorId];
			};

			#ifdef UBI_IOS
				static void 				InitIosAllocator();
				static CFAllocatorRef 		GetIosAllocator();
				static void* 				Allocate(CFIndex allocSize, CFOptionFlags hint, void* info);
				static void* 				ReAllocate(void* ptr, CFIndex newSize, CFOptionFlags hint, void* info);
				static void 				Release(const void* info);
			#endif
		};
	}

	#ifdef MSDK_BUILD_PROFILE
	    #define msdk_Alloc(x) msdk_Alloc(x, __FILE__, __LINE__, __func__)
	    #define msdk_Realloc(x, y) msdk_Realloc(x, y, __FILE__, __LINE__, __func__)
	    #define msdk_Free(x) msdk_Free(x, __FILE__, __LINE__, __func__)
	    extern void* (*msdk_Alloc) (size_t, const char*, int, const char*);
	    extern void* (*msdk_Realloc) (void*, size_t, const char*, int, const char*);
	    extern void (*msdk_Free)(void*, const char*, int, const char*);
	#else
	    extern void* (*msdk_Alloc) (size_t);
	    extern void* (*msdk_Realloc) (void*, size_t);
	    extern void (*msdk_Free)(void*);
	#endif
	extern void* (*msdk_AllocAligned) (size_t, size_t);
	extern void* (*msdk_ReallocAligned) (void*, size_t, size_t);
	extern char (*msdk_Protect) (void*, size_t);
	extern char (*msdk_UnProtect) (void*, size_t);

	extern size_t (*msdk_GetMaxAllocationSize) ();
	extern size_t (*msdk_GetAlignement) ();
	extern const char_utf8* (*msdk_GetName) ();
#endif

#endif // MOBILE_SDK_API_MEM_MANAGER_H
