#ifndef MOBILESDKPRIVATE_COMMON_THREAD
#define MOBILESDKPRIVATE_COMMON_THREAD

// ==================================================================================
// Mobile SDK internal stuff
#ifdef MSDK_BUILD_LIB

	#include "MobileSDKprivate/common/Utils.h"

	namespace MobileSDKAPI
	{
		// Thread callback function interface
		typedef unsigned int (*ThreadCallBack) (void*);

		//------------------------------------

		// Special cases of thread affinity
		enum eThreadAffinity
		{
			MOBILE_SDK_THREAD_NO_AFFINITY = -1
		};

		//------------------------------------

		// Priority are not the same on all platform
		// this is an abstraction, implementation can be different depending on platform
		enum eThreadPriority
		{
			MOBILE_SDK_THREAD_PRIORITY_IDLE,
			MOBILE_SDK_THREAD_PRIORITY_LOWEST,
			MOBILE_SDK_THREAD_PRIORITY_BELOW_NORMAL,
			MOBILE_SDK_THREAD_PRIORITY_NORMAL,
			MOBILE_SDK_THREAD_PRIORITY_ABOVE_NORMAL,
			MOBILE_SDK_THREAD_PRIORITY_HIGHEST,
			MOBILE_SDK_THREAD_PRIORITY_TIME_CRITICAL
		};

		//-------------------------------

		// Thread opaque structure container
		struct ThreadStruct
		{
			ThreadStruct();
			ThreadStruct(const ThreadStruct& rhs);
			ThreadStruct& operator =(const ThreadStruct& rhs);
			~ThreadStruct();

			int affinity;
			eThreadPriority priority;

			struct ThreadStructImpl;
			ThreadStructImpl* m_impl;
		};

		//-------------------------------

		void Sleep(msdk_u32 secs);
		void MiliSleep(msdk_u64 milisecs);

		// Set the thread affinity mask (bit field corresponding to processors)
		// ex : NoAffinity = no affinity
		// ex : (1 << 0) | (1 << 3) = affinity with processor 0 and 3
		// depending on processor number... System info get Nb Processor ?
		bool SetThreadAffinityMask(ThreadStruct& thread, int affinity);

		// Set the priority of the thread
		// this is a logical priority, system implementation will interprete this.
		bool SetThreadPriority(ThreadStruct& thread, eThreadPriority priority);

		// Instanciate the thread to call : callbackAddress(parameter)
		// default stack size is 0
		bool StartThread(ThreadStruct& thread, ThreadCallBack callbackAddress, void* parameter, unsigned int stackSize = 0, const char* threadName = "MSDK thread");

		// ! Pause can easily cause a deadlock (pause during using a mutex...) - use a rendez vous instead !
		// Resume a paused thread
		// this decrement the suspend count. when suspend count is zero, thread is resumed.
		// return the previous thread suspend count
		//	2 or more : it was suspended, it is still suspended (by returned value -1)
		//	1 : it was suspended, now it is running
		//	0 : it was not suspended
		//	-1 : function fail
		int ResumeThread(ThreadStruct& thread);

		// ! Pause can easily cause a deadlock (pause during using a mutex...) - use a rendez vous instead !
		// Suspend a thread
		// this increment the suspend count. if thread was running, it is paused
		// return the previous thread suspend count
		//	0 : it was running, now it is stopped
		//	1 or more : it was not running, current suspend count is returned value + 1
		// -1 : function fail
		int SuspendThread(ThreadStruct& thread);

		// ! This function should not be used !
		// Terminate the thread
		// TerminateThread can result in the following problems:
		//	If the target thread owns a critical section, the critical section will not be released.
		//	If the target thread is allocating memory from the heap, the heap lock will not be released.
		//	If the target thread is executing certain kernel32 calls when it is terminated, the kernel32 state for the thread's process could be inconsistent.
		//	If the target thread is manipulating the global state of a shared DLL, the state of the DLL could be destroyed, affecting other users of the DLL.
		bool TerminateThread(ThreadStruct& thread, short exitCode);

		// Wait until the specified thread is finished
		// return true if the thread finished, false if there was an error
		bool JoinThread(ThreadStruct& thread);

		bool IsFinished(ThreadStruct& thread);

	} // namespace MobileSDKAPI

#endif
// ==================================================================================

#endif /*MOBILE_SDK_API_THREAD_H*/
