#ifndef MOBILESDKPRIVATE_COMMON_UTILS
#define MOBILESDKPRIVATE_COMMON_UTILS

#include "MobileSDK/common/Utils.h"
#include <json.h>

// ==================================================================================
// Mobile SDK internal stuff
#ifdef MSDK_BUILD_LIB

#ifdef UBI_IOS
#include <Foundation/Foundation.h>
#endif
	#define MSDK_TAG_COMMUNICATION "Comm"
	#define MSDK_TAG_PN "Pn"
	#define MSDK_TAG_SOCIAL "Social"
	#define MSDK_TAG_UP "UP"
	#define MSDK_TAG_DEVICE "Device"
	#define MSDK_TAG_TRACKING "Tracking"
	#define MSDK_TAG_COMMON "Common"
	#define MSDK_TAG_IAB "IAB"
	#define MSDK_TAG_AD "Ad"

	#include "MobileSDKprivate/Common/MemManager.h"
	void msdk_PopupError(const char_utf8*  p_message, ...);
	// ERROR CODES
	const int IAP_BAD_GOOGLE_DEVELOPER_KEY = -1;
    const int MSDK_MAX_ID_MANAGER = 255;
	namespace MobileSDKAPI
	{
		typedef void  (*callback)();
	}

	typedef struct CharCompFunctor
	{
		bool operator() (const char* lhs, const char* rhs) const
		{return (strcmp(lhs, rhs) < 0);}
	}CharCompFunctor;

	typedef std::map<const char_utf8*, const char_utf8*, CharCompFunctor> charCharMap;

	const char_utf8* msdk_NetworkIdToStr(msdk_NetworkId);
	msdk_NetworkId msdk_StrToNetworkId(const char_utf8* p_networkId);
    DECLARE_ARRAYS(bool);
	typedef enum msdk_RequestType
	{
		// -----------------------------------
		// used by : FriendList
		GET_FRIENDS_INFO = 0x1,

		// -----------------------------------
		// used by : SocialConnection
		CONNECTION,
		AUTO_CONNECTION,
		GET_USER_OPTIONAL_INFOS,

		// -----------------------------------
		// used by : Leaderboard
		ADD_SCORE,
		GET_SCORES,
		// -----------------------------------
		// used by : Achievement
		UPDATE_ACHIEVEMENT,
		GET_ACHIEVEMENTS,

		// -----------------------------------
		// used by : Matchmaking
		GET_PLAYERS,
		//GET_ACHIEVEMENTS,

		// -----------------------------------
		// used by FacebookGraphAPI
		FACEBOOK_GRAPH_API, // 10

		// -----------------------------------
		// used by In App Purchase
		IAP_INIT,
		IAP_LOAD_STORE,
		IAP_BUY,
		IAP_CONSUME,
		// By Damien
		IAB_BUY, // 15
		IAB_CONSUME,
	    IAB_VALIDATE,

		// -----------------------------------
		// Used by Wall
		WALL_PUBLISH,
		WALL_IS_LIKED,

		// -----------------------------------
		// Used by Invite
		INVITE, // 20
		GET_INVITATIONS,

		// -----------------------------------
		// Used by Ad
		INIT_AD,
		PRELOAD_AD,
		VIRTUAL_CURRENCY,

	    // -----------------------------------
	    // Used by Reachability

	    CHECK_REACHABILITY,

	    SOCIAL_REQUEST,
		// -----------------------------------
		// special
		NOT_A_REQUEST // 26
	}msdk_RequestType;

	struct msdk_SocialRequestParam
	{
	    msdk_s8 requestId;
	    msdk_Service service;
	};

	msdk_u32 		GetMilliseconds			();
	msdk_u64 		GetMicroseconds			();

	void 			msdk_itoa_s				(long value, char* str, int base);
	void 			msdk_strreverse			(char* p_begin, char* p_end);
	void 			msdk_itoa				(int value, char* str, int base);
	void 			msdk_ulltoa				(msdk_u64 value, char* str, int base);
	void 			msdk_ultoa				(msdk_u32 value, char* str, int base);
	msdk_s32		msdk_atoi				(const char_utf8* p_value);
	double			msdk_atof				(const char_utf8* p_value);
	msdk_u64		msdk_atoull				(const char_utf8*  p_value);
	int 			msdk_isalnum			(int c);
#ifdef UBI_ANDROID
#include <stddef.h>
#endif // UBI_ANDROID


    class IdManager
    {
#ifdef UBI_IOS
    private:
		NSMutableArray *Ids;
#endif
#ifdef UBI_ANDROID
    private:
		Array_bool Ids;
#endif

    public:
        IdManager();
        msdk_s8 GetNextId();
        void FreeId(msdk_s8 Id);
    };

	typedef enum msdk_ZipFormat
	{
		MSDK_GZIP
	}msdk_ZipFormat;

	typedef enum msdk_HashFormat
	{
		MSDK_CRC32,
	    MSDK_ADLER32
	}msdk_HashFormat;

	unsigned char*	msdk_Unzip(unsigned char* p_stream, msdk_u32 p_size, msdk_ZipFormat p_format);
	unsigned char*  msdk_Zip(unsigned char* p_stream, msdk_u32 p_size, msdk_ZipFormat p_format, msdk_u32* p_zippedLength);
	msdk_u32        msdk_Hash(const unsigned char* p_stream, msdk_u32 p_size, msdk_HashFormat p_format);

	// Return 1 if date1 > date2, 0 if ==, -1 if <
	msdk_s32        msdk_dateCompare(const char_utf8* p_date1, const char_utf8* p_date2);

	char_utf8* msdk_GetCallStack();

    char_utf8*      JsonToString(json_value* p_json, char_utf8* result, msdk_u32* currentLength);

    char_utf8* allocAndCopyStr(const char_utf8* str);

	#if defined(UBI_WIN8) || defined(UBI_WP8)
	#include <Windows.h>
	#endif // UBI_WIN8 || UBI_WP8

	#ifdef UBI_ANDROID
	#include <stddef.h>
	#endif // UBI_ANDROID

	#ifdef UBI_IOS
	#ifdef __OBJC__
	#define SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(v)  [[[[UIDevice currentDevice] systemVersion] substringToIndex:1] intValue] >= [[v substringToIndex:1] intValue]
	#endif
	#endif

	#ifdef MSDK_UNITY_BINDING
	extern "C" {

	    // because UnitySendMessage is declared extern "C" by unity

	    extern void UnitySendMessage(const char *, const char *, const char *);

	}
	#endif

#endif
// ==================================================================================

#endif //MOBILE_SDK_API_UTILS_H
