#ifndef MOBILESDKPRIVATE_DYNAMICSTORE_STOREMANAGER
#define MOBILESDKPRIVATE_DYNAMICSTORE_STOREMANAGER

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/common/Thread.h"
#include "MobileSDKprivate/dynamicStore/DynamicStore.h"
#include "MobileSDKprivate/common/db/UserItems.h"
#include "MobileSDKprivate/common/db/StoreViews.h"
#include "MobileSDKprivate/common/db/StoreTransactions.h"
#include "MobileSDKprivate/common/CriticalSection.h"

class StorePrimary;
class StoreSecondary;
class StorePromos;

namespace MobileSDKAPI {
    class StoreManager
    {
    public:
        static  StoreManager*   Instance();
        static  void            Destroy();

        void    CallInit();
        msdk_Status StatusInit();
        msdk_Result ResultInit();

        Array_msdk_PrimaryStoreItem*    GetPrimaryStore();
        Array_msdk_SecondaryStoreItem*  GetSecondaryStore();
        Array_msdk_StoreItem*           GetUserItems();
        void                            ResetUserItems();
        msdk_FileContent                ExportUserItems();
        msdk_s8                         ImportUserItems(const char_utf8* p_dump, msdk_u32 p_size);
        Array_msdk_StoreItem*           GetGameItems();
        msdk_u32                        GetItemAmount(msdk_StoreItem* p_item);
        // item given by the game directly, update the item with the updated amount
        msdk_s8                         CreditItem(msdk_StoreItem* p_item);
        msdk_s8                         ConsumeItem(msdk_StoreItem* p_item);
        void                            CallRestoreItems();
        msdk_Status                     StatusRestoreItems();
        msdk_Result                     ResultRestoreItems();
        void                            ReleaseRestoreItems();
        Array_msdk_u32*                 RestoreSecondaryItems();
        void                            ReleaseRestoreSecondaryItems();
        void                            CallBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);
        msdk_Status                     StatusBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);
        msdk_PrimaryPurchaseResult      ResultBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);
        void                            ReleaseBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);
        Array_msdk_StoreItem*           BuySecondaryItem(msdk_SecondaryStoreItem* p_item);
        Array_msdk_PrimaryStoreItem*    PrimaryStoreRequest(const char_utf8* p_requestName);
        Array_msdk_SecondaryStoreItem*  SecondaryStoreRequest(const char_utf8* p_requestName);
        Array_msdk_IAPProduct*    GetResumedTransactions();
        void                            ClearResumedTransaction(const char_utf8* p_ubiTransactionId);
        void                            ReleaseResumedTransaction();
        void                            RegisterRefreshCallback(Store_RefreshCallback p_callback);
        void                            Refresh();

        inline bool                     CanDestroy() { return (initStatus != MSDK_IN_PROGRESS) && canUpdate;};
        inline void						ActivateCloud(bool value) { cloudActivated = value; };
    private:
        enum StoreBuyState
        {
            MSDK_BUYING,
            MSDK_CONSUMING
        };
        StoreManager();
        ~StoreManager();
        static StoreManager*                        instance;

        msdk_Status                                 initStatus;
        msdk_Result                                 initResult;
        static MobileSDKAPI::ThreadStruct           initThread;
        MobileSDKAPI::ThreadStruct                  initTimeoutThread;

        MobileSDKAPI::ThreadStruct                  refreshThread;

        msdk_Status                                 restoreStatus;
        msdk_Result                                 restoreResult;
        MobileSDKAPI::ThreadStruct                  restoreThread;

        msdk_s8                                     buyRequestId;
        MobileSDKAPI::ThreadStruct                  buyThread;
        msdk_Status                                 buyStatus;
        msdk_PurchaseResult*                        buyResult;
        StoreBuyState                               buyState;
        msdk_PrimaryStoreItem*                      currentBuyingItem;

        char_utf8*                                  language;
        char_utf8*                                  currency;

        Array_msdk_PrimaryStoreItem*                primaryStore;
        Array_msdk_SecondaryStoreItem*              secondaryStore;
        Array_msdk_StorePromo*                      promoStore;
        UserItems                                   userItems;

        std::vector<Array_msdk_StoreItem*>          storeItemsGarbage;
        std::vector<msdk_StoreViewResult*>          viewItemsGarbage;

        StoreTransactions                           transactions;
        Array_msdk_PrimaryStoreItem*                resumedTransactions;

        bool                                        isPrimaryStoreUpdated;
        bool                                        isPromoUpdated;
        bool                                        isSecondaryStoreUpdated;

        bool                                        canUpdate;

        MobileSDKAPI::CriticalSectionStruct         primaryStoreLock;
        
        Store_RefreshCallback                       refreshCallback;
        
        Array_msdk_u32*                             restoredItems;

        bool										cloudActivated;

        static unsigned int InitThread(void* p_param);
        static void RefreshCallback(void* p_param);
        static unsigned int RefreshThread(void* p_param);
        static unsigned int BuyThread(void* p_param);
        static unsigned int TimeoutThread(void* p_param);

        static void ParseFeed(const char_utf8* p_feed);
        static void InitIAP(StoreManager* store);

        Array_msdk_StorePromo*    GetPromos();

        inline msdk_s8  GetBuyRequestId() { return buyRequestId; };
        inline void SetInitStatus(msdk_Status value) { initStatus = value; };
        inline void SetInitResult(msdk_Result value) { initResult = value; };
        inline void SetRestoreStatus(msdk_Status value) { restoreStatus = value; };
        inline void SetRestoreResult(msdk_Result value) { restoreResult = value; };
        inline void SetBuyStatus(msdk_Status value) { buyStatus = value; };
        inline void SetBuyResult(msdk_PurchaseResult* value) { buyResult = value; };
        inline void SetState(StoreBuyState value) { buyState = value; };
        inline void SetLanguage(char_utf8* value) { language = value; };
        inline void SetCurrency(char_utf8* value) { currency = value; };
        inline void SetUpdatable(bool p_canUpdate) { canUpdate = p_canUpdate; };
        inline UserItems& GetUserItemDB() { return userItems; };
        inline msdk_PrimaryStoreItem* GetCurrentBuyingItem() { return currentBuyingItem; };
        inline StoreTransactions&    GetTransactions() { return transactions; };
        inline void SetUpdated(bool p_isUpdated) { isPrimaryStoreUpdated = p_isUpdated; isSecondaryStoreUpdated = p_isUpdated; isPromoUpdated = p_isUpdated; };

        void    DestroyStore(Array_msdk_PrimaryStoreItem*);
        void    DestroyStore(Array_msdk_SecondaryStoreItem*);
        void    DestroyPromo(Array_msdk_StorePromo*);
        
        void    UpdateStoreFromIap();

        static bool SortPrimary(StorePrimary* arg1, StorePrimary* arg2);
        static bool SortSecondary(StoreSecondary* arg1, StoreSecondary* arg2);
        static bool SortPromos(StorePromos* arg1, StorePromos* arg2);

        static unsigned int RestoreItemsThread(void* p_param);

        Array_msdk_PrimaryStoreItem*    GetInternalPrimaryStore();
        Array_msdk_StorePromo*          GetInternalPromos();
    };
}

#endif
