#ifndef MOBILESDKPRIVATE_SOCIAL_MATCHMAKING
#define MOBILESDKPRIVATE_SOCIAL_MATCHMAKING

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/common/RequestPool.h"

namespace MobileSDKAPI
{
	namespace SocialAPI
	{
        //
        // Contains static methods to perform matchmaking requests.
        //
        class Matchmaking
        {
            friend class FacebookBindings;
            friend class GameCenterBindings;
        private:

            static RequestPool<std::vector<std::string *>*, GET_PLAYERS>          s_GetPlayersRequestPool;

        public:
            //
            // Send a request to find random players wainting to start a game
			// | iOS           | Android       |
			// | :-----------: | :-----------: |
			// | MSDK_GAME_CENTER   |               |
			// |               |               |
            // @param  p_minPlayers    The minimum number of players to be found
            // @param  p_maxPlayers    The maximum number of players to be found
            //
            // @return The id of the created request as a s8, -1 if an error occured
            //
            static msdk_s8                                   CallFindPlayers				(msdk_u8 p_minPlayers, msdk_u8 p_maxPlayers);

            //
            // Get the state of a request
            // @param  p_requestID The id of the request
            // @return The state of the request
            //
            static msdk_Status                       StatusFindPlayers		(msdk_u8 p_requestID);

            //
            // Get the result of a FindPlayers request
            // @param p_requestID The id of the request
            // @return The result of the request as a vector of strings
            //
            static const std::vector<std::string *>&    ResultFindPlayers			(const msdk_u8& p_requestID);

            //
            // Release a request created to add a score
            // @param  p_requestID The id of the request
            //
            static void                                 ReleaseFindPlayers		(msdk_s8 p_requestID);

            //
            // Called internally. Once a score is added send the result to this function
            // @param  p_requestID The id of the request
            // @param  p_ids The vector of player ids
            //
            static void                                 UpdateFindPlayersRequest			(msdk_s8 p_requestID, const std::vector<std::string *>& p_ids);


        };
    }
}

#endif // MOBILE_SDK_API_MATCHMAKING_H
