#ifndef MOBILESDKPRIVATE_SOCIAL_PROFILE_BUCKETMANAGER
#define MOBILESDKPRIVATE_SOCIAL_PROFILE_BUCKETMANAGER

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/common/Thread.h"
#include "MobileSDKprivate/social/profile/GameSession.h"
#include <json.h>


namespace MobileSDKAPI
{
    class BucketManager
    {

    private:
        static BucketManager*   instance;
        BucketManager();

        static msdk_Status m_getBucketsStatus;
        static msdk_Result m_getBucketsResult;
        static ThreadStruct m_getBucketsThread;


        struct BucketParams
        {
            msdk_Bucket*    bucket;
            ThreadStruct    thread;
            msdk_Status     status;
            msdk_Result     result;
            msdk_u32        startingRecord;
            msdk_u32        endingRecord;
            BucketParams() {}
            ~BucketParams() {};
        };

        static std::map<msdk_Bucket*, BucketParams*>   m_pendingBuckets;

        static std::map<const char_utf8*, BucketParams*, CharCompFunctor>   m_deletingBuckets;
        static std::map<const char_utf8*, BucketParams*, CharCompFunctor>   m_dataBuckets;
        static std::map<const char_utf8*, BucketParams*, CharCompFunctor>   m_saveBuckets;

        static unsigned int CreateBucketThread(void* p_param);
        static unsigned int GetBucketsThread(void* p_param);
        static unsigned int DeleteBucketThread(void* p_param);
        static unsigned int GetBucketDataThread(void* p_param);
        static unsigned int SaveBucketDataThread(void* p_param);
    public:
        static BucketManager* Instance();

        msdk_Bucket* PrepareBucket(msdk_GameSession* p_session, const char_utf8* p_name, msdk_BucketType p_bucketType);
        void   BucketAddRight(msdk_Bucket* p_bucket, const char_utf8* p_federatedId, const char_utf8* p_right);

        void CallCreateBucket(msdk_Bucket* p_bucket);
        msdk_Status StatusCreateBucket(msdk_Bucket* p_bucket);
        msdk_Result ResultCreateBucket(msdk_Bucket* p_bucket);
        void ReleaseCreateBucket(msdk_Bucket* p_bucket);

        void CallGetBuckets(msdk_GameSession* p_session);
        msdk_Status StatusGetBuckets();
        msdk_Result ResultGetBuckets();
        void ReleaseGetBuckets();

        void CallDeleteBucket(msdk_Bucket* p_bucket);
        msdk_Status StatusDeleteBucket(msdk_Bucket* p_bucket);
        msdk_Result ResultDeleteBucket(msdk_Bucket* p_bucket);
        void ReleaseDeleteBucket(msdk_Bucket* p_bucket);

        void BucketUpdateValue(msdk_Bucket* p_bucket, const char_utf8* p_key, const char_utf8* p_value);
        void BucketDeleteKey(msdk_Bucket* p_bucket, const char_utf8* p_key);
        const char_utf8* BucketGetValue(msdk_Bucket* p_bucket, const char_utf8* p_key);

        void CallSaveBucket(msdk_Bucket* p_bucket);
        msdk_Status StatusSaveBucket(msdk_Bucket* p_bucket);
        msdk_Result ResultSaveBucket(msdk_Bucket* p_bucket);
        void ReleaseSaveBucket(msdk_Bucket* p_bucket);


        void CallBucketObjectData(msdk_Bucket* p_bucket);
        void CallBucketLogData(msdk_Bucket* p_bucket, msdk_u32 p_startingRecord, msdk_u32 p_endingRecord);
        msdk_Status StatusBucketData(msdk_Bucket* p_bucket);
        msdk_Result ResultBucketData(msdk_Bucket* p_bucket);
        void ReleaseBucketData(msdk_Bucket* p_bucket);
    };
}

#endif
