#ifndef MOBILESDKPRIVATE_SOCIAL_PROFILE_GAMESESSIONMANAGER
#define MOBILESDKPRIVATE_SOCIAL_PROFILE_GAMESESSIONMANAGER

	#include "MobileSDKprivate/common/Utils.h"
	#include "MobileSDKprivate/common/Thread.h"
	#include "MobileSDKprivate/social/profile/GameSession.h"
	#include <json.h>
	#include <map>

namespace MobileSDKAPI {
	class GameSessionManager {
		friend class BucketManager;
	private:
		static GameSessionManager* instance;
		GameSessionManager();

		static msdk_Status m_createSessionStatus;
		static ThreadStruct m_createSessionThread;
		static ThreadStruct m_getSessionsThread;
		static ThreadStruct m_matchmakingThread;
		static ThreadStruct m_joinMatchmakingThread;
		static ThreadStruct m_infoMatchmakingThread;
	    static ThreadStruct m_getClustersThread;
		static msdk_s32 m_creatingSessionIndex;
		static msdk_Status m_getSessionsStatus;
		static msdk_Status m_matchmakingStatus;
		static msdk_Status m_joinMatchmakingStatus;
		static msdk_Status m_infoMatchmakingStatus;
	    static msdk_Status m_getClustersStatus;
		//static Array_msdk_Match m_matchs;
		static Array_msdk_GameSession m_sessions;
	    static Array_msdk_GameSession m_matches;
		static msdk_Result m_infoMatchmakingResult;
		static msdk_Result m_joinMatchmakingResult;
	    static PointerArray_char_utf8 m_clusters;

		struct SessionParam {
			msdk_GameSession* session;
			msdk_u32 invitationtype;
			const char_utf8* message;
			msdk_Status status;
			msdk_Result result;
			ThreadStruct thread;

			SessionParam() {
			}
			;
			~SessionParam() {
			}
			;
		};

		struct UserAccessParam {
			msdk_UserInfo* user;
			msdk_GameSessionAccess access;
			msdk_GameSession* session;
			msdk_Status status;
			msdk_Result result;
			ThreadStruct thread;

			UserAccessParam() {
			}
			;
			~UserAccessParam() {
			}
			;
		};

		struct MatchmakingParam {
	        const char_utf8*        portal;
	        msdk_u8                 limit;
					msdk_u16                maxPlayers;
					Array_msdk_key_value    filter;
	        char_utf8*              cluster;

			MatchmakingParam() {
	            portal = NULL;
	            limit = 0;
							maxPlayers = 0;
	            filter.count = 0;
	            filter.elem = NULL;
	            cluster = NULL;
			}
			;
			~MatchmakingParam() {
	            if(cluster != NULL)
	                msdk_Free(cluster);
	            for(msdk_u32 i = 0; i < filter.count; ++i)
	            {
	                if(filter.elem[i].key != NULL)
	                    msdk_Free(const_cast<char_utf8*>(filter.elem[i].key));
	                if(filter.elem[i].value != NULL)
	                    msdk_Free(const_cast<char_utf8*>(filter.elem[i].value));
	            }
	            msdk_Free(filter.elem);
			}
			;
		};

		static std::map<msdk_GameSession*, char_utf8*> m_matchmakingIndexing;
		static MatchmakingParam* m_matchmakingParam;
		static std::map<msdk_UserInfo*, UserAccessParam*> m_pendingUserAccesses;
		static std::map<msdk_GameSession*, SessionParam*> m_pendingInvitations;
		static std::map<msdk_GameSession*, SessionParam*> m_stopingSessions;
	    static std::map<msdk_GameSession*, SessionParam*> m_sessionInfos;

		static unsigned int CreateSessionThread(void* p_param);
		static unsigned int GetSessionsThread(void* p_param);
		static unsigned int UserAccessThread(void* p_param);
		static unsigned int GameInviteThread(void* p_param);
		static unsigned int StopSessionThread(void* p_param);
	    static unsigned int GetClustersThread(void* p_param);
		static unsigned int MatchmakingThread(void* p_param);
		static unsigned int SessionInfoThread(void* p_param);
		static unsigned int JoinMatchmakingThread(void* p_param);

	    void InitSession(msdk_GameSession* p_session);
	    void InitMatch(msdk_Match* p_match);
	    void InitInfos(msdk_GameSessionInfo* p_infos);
		void ReleaseSession(msdk_GameSession* p_session);
		void ReleaseSessions(Array_msdk_GameSession p_sessions);

		void SendInvitation(msdk_GameSession* p_session, const char_utf8* p_action, SessionParam* p_params);
	public:
		static GameSessionManager* Instance();

		msdk_GameSession* PrepareSession(msdk_u32 p_expirationtime);
	    void SetPortal(msdk_GameSession* p_session, msdk_PortalType p_portal);
		void AddMatchmakingIndexing(msdk_GameSession* p_session, const char_utf8 *indexing);

		void SessionAddFriend(msdk_GameSession* p_session, const msdk_UserInfo* p_user);
		void CallCreateSession(msdk_GameSession* p_session);
		msdk_Status StatusCreateSession();
		msdk_GameSession* ResultCreateSession();
		void ReleaseCreateSession();

		void CallGet();
		msdk_Status StatusGet();
		Array_msdk_GameSession* ResultGet();
		void ReleaseGet();

		void CallUserAccess(msdk_GameSession* p_session, msdk_GameSessionAccess p_access, msdk_UserInfo* p_user);
		msdk_Status StatusUserAccess(msdk_UserInfo* p_user);
		msdk_Result ResultUserAccess(msdk_UserInfo* p_user);
		void ReleaseUserAccess(msdk_UserInfo* p_user);

		void CallInvite(msdk_GameSession* p_session, msdk_u32 p_invitationType, const char_utf8* p_message);
		msdk_Status StatusInvite(msdk_GameSession* p_session);
		msdk_Result ResultInvite(msdk_GameSession* p_session);
		void ReleaseInvite(msdk_GameSession* p_session);

		void CallStop(msdk_GameSession* p_session);
		msdk_Status StatusStop(msdk_GameSession* p_session);
		msdk_Result ResultStop(msdk_GameSession* p_session);
		void ReleaseStop(msdk_GameSession* p_session);

	    void CallGetClusters();
	    msdk_Status StatusGetClusters();
	    PointerArray_char_utf8* ResultGetClusters();
	    void ReleaseGetClusters();

		void CallMatchmaking(msdk_PortalType portal, msdk_u8 limit, Array_msdk_key_value *filter, const char_utf8* p_cluster);
		msdk_Status StatusMatchmaking();
		Array_msdk_GameSession* ResultMatchmaking();
		void ReleaseMatchmaking();

		void CallJoinMatch(msdk_GameSession* p_session);
		msdk_Status StatusJoinMatch();
		msdk_Result ResultJoinMatch();
		void ReleaseJoinMatch();

		void CallSessionInfo(msdk_GameSession* p_session);
		msdk_Status StatusSessionInfo(msdk_GameSession* p_session);
		msdk_Result ResultSessionInfo(msdk_GameSession* p_session);
		void ReleaseSessionInfo(msdk_GameSession* p_session);
	};
}

#endif
