#ifndef MOBILESDKPRIVATE_SOCIAL_PROFILE_MAILBOXMANAGER
#define MOBILESDKPRIVATE_SOCIAL_PROFILE_MAILBOXMANAGER

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/common/CriticalSection.h"
#include "MobileSDKprivate/common/Thread.h"
#include "MobileSDKprivate/social/profile/Mailbox.h"
#include <json.h>


namespace MobileSDKAPI
{
    class MailboxManager
    {
    private:
        struct MailInfo
        {
            msdk_Status     status;
            msdk_Mail*      mail;
            ThreadStruct    thread;
        };

        static MailboxManager*  instance;
        MailboxManager();

        static ThreadStruct     m_sendMailThread;
        static ThreadStruct     m_getMailboxThread;
        static ThreadStruct     m_synchronizeThread;
        static std::map<msdk_Mail*, MailInfo*>  m_processingMails;

        static unsigned int     SynchronizeMailboxThread(void* p_param);
        static unsigned int     DeleteMailThread(void* p_param);
        static unsigned int     ReadMailThread(void* p_param);
        static unsigned int     SendMailThread(void* p_param);
        static unsigned int     GetMailboxThread(void* p_param);

        static msdk_Status      m_getStatus;
        static Array_msdk_Mail  m_mailbox;

        const char_utf8* MailboxToFeed();
        bool             ParseMailbox(const char_utf8*);
        const char_utf8* JsonFromMail(msdk_Mail* p_mail);
        msdk_Mail*       MailFromJson(const char_utf8* p_json);
        void             ReleaseMailbox();

        static CriticalSectionStruct m_mailboxLock;
        static CriticalSectionStruct m_synchronizeLock;
    public:
        static MailboxManager*  Instance();

        static void             Synchronize();

        msdk_Mail*       CreateMail(const char_utf8* p_message, const char_utf8* p_title);

        void             AddRecipient(msdk_Mail*, msdk_UserInfo*);

        void             SendMail(msdk_Mail*);

        void             CallGet();

        msdk_Status      StatusGet();

        Array_msdk_Mail* ResultGet();

        void             ReleaseGet();

        void             CallRead(msdk_Mail*);

        msdk_Status      StatusRead(msdk_Mail*);

        const char_utf8* ResultRead(msdk_Mail*);

        void             ReleaseRead(msdk_Mail*);

        void             Delete(msdk_Mail*);
    };
}
#endif
