#ifndef MOBILESDKPRIVATE_SOCIAL_PROFILE_USERPROFILEMANAGER
#define MOBILESDKPRIVATE_SOCIAL_PROFILE_USERPROFILEMANAGER

#include "MobileSDKprivate/common/Utils.h"
#include "MobileSDKprivate/common/Init.h"
#include "MobileSDKprivate/common/CriticalSection.h"
#include "MobileSDKprivate/common/Thread.h"
#include "MobileSDKprivate/social/interface/Social_Interface.h"
#include "MobileSDKprivate/social/profile/UserProfile.h"
#include "MobileSDKprivate/social/profile/MergeProfile.h"
#include "MobileSDKprivate/common/http/HttpRequest.h"

namespace MobileSDKAPI
{
    class UserProfileManager
    {
    friend class GameDataManager;
    friend class MailboxManager;
    friend class GameSessionManager;
    friend class BucketManager;
    friend class MergeProfile;

    private:
        struct ConnectionParameters
        {
            ConnectionParameters()
            	: m_thread()
            {
            	m_status = MSDK_NOT_ALLOCATED;
                CriticalSectionInitialize(m_lock);
            }

            ThreadStruct            m_thread;
            msdk_Status             m_status;
            CriticalSectionStruct   m_lock;
        };

        UserProfileManager();
        char_utf8*                                                          m_userName;
        static Array_msdk_GameFriend                                        s_friends;

        static std::map<msdk_ConnectionInterface*, ConnectionParameters*>   s_connectingInterfaces;
        static CriticalSectionStruct                                        s_lock;
        static bool                                                         s_isLockInitialized;
        msdk_s8                                                             m_isInit;


        msdk_UserProfileWindowManager*                                      m_windowManager;
        
        bool                                                                m_manageNetworks;

        void RegisterNetwork(msdk_ConnectionInterface* p_interface);
        static unsigned int    ConnectionThread(void* p_param);
        static unsigned int    AutoConnectionThread(void* p_param);
        static unsigned int    LogoutThread(void* p_param);
        static unsigned int    InitThread(void* p_param);
        static unsigned int    FriendsThread(void* p_param);

        static UserProfileManager*      instance;
    public:
        static  UserProfileManager*     Instance();

        // Send request related to user profile (handle token expiration)
        static const char_utf8* SendRequest(msdk_HttpRequest& p_request, bool shouldRetry = true);

        void             RegisterWindowManager(msdk_UserProfileWindowManager* p_manager);
        void             OnMergeAction(msdk_s8 p_acceptMerge);
        void             OnAccountSelected(msdk_ProfileInfo* p_selectedProfile);
        void             OnUnregisterAction();

        inline void     DisableNetworkManagement() { m_manageNetworks = false; };
        inline bool     DoManageNetworks() { return m_manageNetworks; };
        void 			CallInit();
        msdk_Status		StatusInit();
        msdk_Result		ResultInit();
        void 			ReleaseInit();
        msdk_s8			IsInit();

        // Connection
        //static void            RegisterDevice();

        void WaitForConnectionThread(msdk_ConnectionInterface* p_interface);

        void            CallConnect(msdk_ConnectionInterface* p_interface);
        msdk_Status     StatusConnect(msdk_ConnectionInterface* p_interface);
        msdk_Result     ResultConnect(msdk_ConnectionInterface* p_interface);
        void            ReleaseConnect(msdk_ConnectionInterface* p_interface);

        void            CallAutoConnect(std::map<msdk_NetworkId, msdk_SocialNetwork*>* p_interfaces);
        msdk_Status     StatusAutoConnect();
        //static void            CallAutoConnect(msdk_ConnectionInterface* p_interface);
        //static msdk_Status     StatusAutoConnect(msdk_ConnectionInterface* p_interface);
        void            ReleaseAutoConnect(msdk_ConnectionInterface* p_interface);

        void            Logout(msdk_ConnectionInterface* p_interface);

        void            CallFriendsList(msdk_FriendsInterface* p_interface);
		msdk_Status     StatusFriendsList(msdk_FriendsInterface* p_interface);
		Array_msdk_UserInfo* ResultFriendsList(msdk_FriendsInterface* p_interface);
		void            ReleaseFriendsList(msdk_FriendsInterface* p_interface);

#ifdef MSDK_BUILD_LIB

        // Friends
        void                     CallFriends();
        msdk_Status              StatusFriends();
        Array_msdk_GameFriend*   ResultFriends();
        void						ReleaseFriends();

        //static void 					msdk_internal_federatedId_resolution(std::map<msdk_NetworkId, msdk_SocialNetwork*>*);
#endif
        //static

        inline const char_utf8* GetFederatedId() { return (MergeProfile::activeFedId != NULL)?MergeProfile::activeFedId->fedId:NULL; };
        inline const char_utf8* GetUbiToken() { return (MergeProfile::activeFedId != NULL)?MergeProfile::activeFedId->accessToken:NULL; };

        inline void SetInit(msdk_s8 value) { m_isInit = value; };

        inline msdk_UserProfileWindowManager* GetWindowManager() { return m_windowManager; };
    };
}
#endif
