#ifndef MOBILESDKPRIVATE_UBISERVICES_UBISERVICES
#define MOBILESDKPRIVATE_UBISERVICES_UBISERVICES
	#include "MobileSDK/ubiservices/UbiServices.h"


#ifdef MSDK_BUILD_LIB
	#include "MobileSDKprivate/common/Thread.h"
	#include "ubiservices/facade.h"
	#include "MobileSDKprivate/tracking/DNATracking.h"

	namespace MobileSDKAPI
	{
	    namespace UbiServices
	    {
	        void                Init(bool p_useUAT=false);
	        void                InitWithToken();
	        bool                IsInit();
	        const char_utf8*    GetToken();
	        const char_utf8*    GetTicket();
	        msdk_Status         GetStatus();
	        void                SetToken(const char_utf8* p_token, msdk_UbiServicesTokenType p_tokenType);
	        US_NS::Facade*      GetFacade();
	        void                AddStartEventParameter(const char_utf8* p_key, const char_utf8* p_value);
            void                OnPause();
            
            void 				CreateGameServicesSession();
            void                CreateGameCenterSession(const char_utf8* publicKeyUrl, const unsigned char* salt, msdk_u32 saltLength, uint64_t timestamp, const char_utf8* bundleId, const char_utf8* playerId, const unsigned char* signature, msdk_u32 signatureLength);
	    }
	}

	typedef struct msdk_UbiServicesResult
	{
	    const   char_utf8* result;
	    int     errorCode;
	}msdk_UbiServicesConnectResult;


	/// @brief get all existing ubiservice Ubimobile account existing on the device. start a "getAllExistingAccount" operation
	/// @param ubiserviceAppID ubiservices app id
	/// @note don't forget to release resources later calling @ref ubimobile_getAllExistingAccountsRelease
	 void    Ubiservices_CallGetAllExistingAccounts(const char_utf8* ubiserviceAppID);

	/// get current "getAllExistingAccount" operation status
	/// @note use only if no callback was specified in @ref ubimobile_getAllExistingAccountsInit
	/// @return The status of the request
	///	MSDK_NOT_ALLOCATED if no operation is in progress<br>
	/// MSDK_IN_PROGRESS if an operation is still in progress<br>
	/// MSDK_ENDED if current operation has finished (check @ref Ubiservices_ResultGetAllExistingAccounts to get result)
	msdk_Status Ubiservices_StatusGetAllExistingAccounts();

	/// get current "getAllExistingAccount" operation result
	/// @return a json object with the result of the request, or NULL if no operation was in progress
	const msdk_UbiServicesResult* Ubiservices_ResultGetAllExistingAccounts();

	/// release all resources used by "getAllExistingAccount" operation
	/// @note note that if you call @ref Ubiservices_ReleaseGetAllExistingAccounts, none of the other operations (@ref Ubiservices_GetAuthStringForUbiserviceNativeSDK) will work
	void Ubiservices_ReleaseGetAllExistingAccounts();


	/// @brief get auth string to be used with ubiservice native sdk to perform login.<br>
	/// @note to work, you need to enable keychain access group on your build and add "com.ubisoft.data" in the access group
	/// @note to work, you need to add the “Security.framework framework to your application
	/// @param p_accountId id of the account to get the auth token for
	/// @param [out] p_auth a char* buffer where the auth token string will be stored (set to NULL to get estimated of length of ret string)
	/// @param [in] p_length of the allocated uduid char* buffer (set to 0 to get estimated of length of ret string)
	/// @return size of auth string (or estimated size if authTokenString is NULL)
	msdk_u32 Ubiservices_GetAuthStringForUbiserviceNativeSDK(const char_utf8* p_accountId, char_utf8* p_auth, msdk_u32 p_length);

	/// @brief set ubimobile auth to work on uat env for the whole session.
	/// call before any other ubimobile_ fn to enable them on UAT
	/// @note !!!! do not enable this on build to be published/released, this is only for dev team to be able to work on uat env (and not prod/live env) !!!
	/// @note note that this get internally called if any newly account created is on UAT or if you try to get a ticket for an account which is registered on uat
	void Ubiservices_DO_NOT_CALL_IN_RELEASE_BUILD_enableUAT();

// #else
// namespace US_NS {
//     class Facade;
// }
#endif


#endif
